@@ -11,14 +11,22 @@ GSM_Error bluetooth_findchannel(GSM_StateMachine *s);
 
 typedef ULONGLONG 		BTH_ADDR, *PBTH_ADDR;
 
-#define NAP_MASK                0xFFFF00000000LL
-#define SAP_MASK                0x0000FFFFFFFFLL
-
 #define NAP_BIT_OFFSET          (8 * 4)
 #define SAP_BIT_OFFSET          (0)
 
+#ifdef __BORLANDC__
+#define NAP_MASK                0xFFFF00000000LL
+#define SAP_MASK                0x0000FFFFFFFFL
+
+#define GET_NAP(_bth_addr)  	((ULONGLONG)  ((_bth_addr) >> NAP_BIT_OFFSET))
+#define GET_SAP(_bth_addr)  	((ULONGLONG)  (((_bth_addr) & SAP_MASK) >> SAP_BIT_OFFSET))
+#else
+#define NAP_MASK                0xFFFF00000000LL
+#define SAP_MASK                0x0000FFFFFFFFLL
+
 #define GET_NAP(_bth_addr)  	((ULONGLONG)  (((_bth_addr) & NAP_MASK) >> NAP_BIT_OFFSET))
 #define GET_SAP(_bth_addr)  	((ULONGLONG)  (((_bth_addr) & SAP_MASK) >> SAP_BIT_OFFSET))
+#endif
 
 #ifndef AF_BTH
 #define AF_BTH  		32
