@@ -2,10 +2,6 @@
  * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot
  * GNU GPL version 2 or later
  */
-/* Some source from Minicom (http://alioth.debian.org/projects/minicom)
- * (C) 1991,1992,1993,1994,1995,1996 by Miquel van Smoorenburg
- * GNU GPL version 2
- */
 /* Due to a problem in the source code management, the names of some of
  * the authors have unfortunately been lost. We do not mean to belittle
  * their efforts and hope they will contact us to see their names
@@ -23,6 +19,7 @@
 #  include <io.h>
 #else
 #  include <errno.h>
+#  include <ctype.h>
 #  include <signal.h>
 #  include <sys/socket.h>
 #  include <sys/stat.h>
@@ -56,7 +53,7 @@ int bluetooth_checkservicename(GSM_StateMachine *s, const char *name)
 			}
 			return 3;
 		}
-		/* Ususally this name also contains OBEX, preffered for irmc */
+		/* Ususally this name also contains OBEX, prefered for irmc */
 		if (strstr(name, "IrMC Sync") != NULL || strstr(name, "OBEX Synchronisation")) {
 			if (strcmp(s->CurrentConfig->Model, "obexirmc") == 0 ||
 				strcmp(s->CurrentConfig->Model, "seobex") == 0) {
@@ -71,6 +68,20 @@ int bluetooth_checkservicename(GSM_StateMachine *s, const char *name)
 			}
 			return 2;
 		}
+		/* Phone Book Access Profile is subset of IrMC */
+		if (strstr(name, "PBAP") != NULL) {
+			if (strcmp(s->CurrentConfig->Model, "obexirmc") == 0) {
+				return 3;
+			}
+			return 2;
+		}
+		/* Object Push Profile */
+		if (strstr(name, "OPP") != NULL) {
+			if (strcmp(s->CurrentConfig->Model, "obexnone") == 0) {
+				return 3;
+			}
+			return 2;
+		}
 		/* Anything matching OBEX has lowest priority */
 		if (strstr(name, "OBEX") != NULL) return 1;
 
@@ -82,6 +93,8 @@ int bluetooth_checkservicename(GSM_StateMachine *s, const char *name)
 		/* Motorola */
 		if (strstr(name, "Dial-up networking Gateway") != NULL) return 3;
 		if (strstr(name, "Serial port") != NULL) return 2;
+		/* Samsung */
+		if (strstr(name, "Dial-up Networking") != NULL) return 3;
 		/* Siemens, Thomas Eitzenberger */
 		if (strstr(name, "SerialPort1") != NULL) return 3;
 		if (strstr(name, "SerialPort") != NULL) return 2;
@@ -98,34 +111,32 @@ int bluetooth_checkservicename(GSM_StateMachine *s, const char *name)
 
 #if defined (GSM_ENABLE_BLUETOOTHDEVICE) || defined (GSM_ENABLE_IRDADEVICE)
 
+/* Windows do not have this, but we don't seem to need it there */
+#ifndef MSG_DONTWAIT
+#define MSG_DONTWAIT 0
+#endif
+
 int socket_read(GSM_StateMachine *s UNUSED, void *buf, size_t nbytes, socket_type hPhone)
 {
 	fd_set 		readfds;
 	int result = 0;
-#ifdef WIN32
 	struct timeval 	timer;
-#else
-	int flags;
-#endif
 
 	FD_ZERO(&readfds);
 	FD_SET(hPhone, &readfds);
+
+	timer.tv_sec = 0;
+	timer.tv_usec = 0;
+
+	if (select(hPhone + 1, &readfds, NULL, NULL, &timer) > 0) {
+		result = recv(hPhone, buf, nbytes, MSG_DONTWAIT);
 #ifndef WIN32
-	if (select(hPhone+1, &readfds, NULL, NULL, 0) > 0) {
-		flags = fcntl(hPhone, F_GETFL);
-		fcntl(hPhone, F_SETFL, flags | O_NONBLOCK);
-		result = read(hPhone, buf, nbytes);
 		if (result < 0 && errno != EINTR) {
-			return 0;
-		}
-		fcntl(hPhone, F_SETFL, flags);
-	}
-#else
-	memset(&timer,0,sizeof(timer));
-	if (select(0, &readfds, NULL, NULL, &timer) > 0) {
-		result = recv(hPhone, buf, nbytes, 0);
-	}
+ 			return 0;
+ 		}
 #endif
+	}
+
 	return result;
 }
 
@@ -182,6 +193,9 @@ GSM_Error lock_device(const char* port, char **lock_name)
 	int 		fd, len;
 	GSM_Error	error = ERR_NONE;
 	size_t wrotebytes;
+	char 	buf[max_buf_len];
+	int 	pid, n = 0;
+
 
 	dbgprintf("Locking device\n");
 
@@ -210,45 +224,45 @@ GSM_Error lock_device(const char* port, char **lock_name)
 	/* Check for the stale lockfile.
 	 * The code taken from minicom by Miquel van Smoorenburg */
 	if ((fd = open(lock_file, O_RDONLY)) >= 0) {
-		char 	buf[max_buf_len];
-		int 	pid, n = 0;
-
 		n = read(fd, buf, sizeof(buf) - 1);
-		close(fd);
-		if (n > 0) {
-			pid = -1;
-			if (n == 4)
-				/* Kermit-style lockfile. */
-				pid = *(int *)buf;
-			else {
-				/* Ascii lockfile. */
-				buf[n] = 0;
-				sscanf(buf, "%d", &pid);
+		if (n <= 0) {
+			goto failread;
+		}
+		if (n == 4 && 4 == sizeof(int) &&
+			! (
+				isdigit(buf[0]) &&
+				isdigit(buf[1]) &&
+				isdigit(buf[2]) &&
+				isdigit(buf[3])
+				)) {
+			/* Rewind */
+			lseek(fd, 0, SEEK_SET);
+			/* Read PID */
+			/* We could make it from buf, but we would have to care about endians. */
+			n = read(fd, &pid, sizeof(int));
+			if (n != 4) {
+				dbgprintf("Reading lock for second time failed\n");
+				goto failread;
+
 			}
-			if (pid > 0 && kill((pid_t)pid, 0) < 0 && errno == ESRCH) {
-				dbgprintf("Lockfile %s is stale. Overriding it..\n", lock_file);
-				sleep(1);
-				if (unlink(lock_file) == -1) {
-					dbgprintf("Overriding failed, please check the permissions\n");
-					dbgprintf("Cannot lock device\n");
-					error = ERR_PERMISSION;
-					goto failed;
-				}
-			} else {
-				dbgprintf("Device already locked by PID %d.\n", pid);
-				error = ERR_DEVICELOCKED;
+		} else {
+			/* Ascii lockfile. */
+			buf[n] = 0;
+			sscanf(buf, "%d", &pid);
+		}
+
+
+		if (pid > 0 && kill((pid_t)pid, 0) < 0 && errno == ESRCH) {
+			dbgprintf("Lockfile %s is stale. Overriding it..\n", lock_file);
+			if (unlink(lock_file) != 0) {
+				dbgprintf("Overriding failed, please check the permissions\n");
+				dbgprintf("Cannot lock device\n");
+				error = ERR_PERMISSION;
 				goto failed;
 			}
-		}
-		/* this must not happen. because we could open the file   */
-		/* no wrong permissions are set. only reason could be     */
-		/* flock/lockf or a empty lockfile due to a broken binary */
-		/* which is more likely					  */
-		if (n == 0) {
-			dbgprintf("Unable to read lockfile %s.\n", lock_file);
-			dbgprintf("Please check for reason and remove the lockfile by hand.\n");
-			dbgprintf("Cannot lock device\n");
-			error = ERR_UNKNOWN;
+		} else {
+			dbgprintf("Device already locked by PID %d.\n", pid);
+			error = ERR_DEVICELOCKED;
 			goto failed;
 		}
 	}
@@ -279,6 +293,11 @@ GSM_Error lock_device(const char* port, char **lock_name)
 	close(fd);
 	*lock_name = lock_file;
 	return ERR_NONE;
+failread:
+	dbgprintf("Unable to read lockfile %s.\n", lock_file);
+	dbgprintf("Please check for reason and remove the lockfile by hand.\n");
+	dbgprintf("Cannot lock device\n");
+	error = ERR_UNKNOWN;
 failed:
 	free(lock_file);
 	*lock_name = NULL;
