@@ -84,11 +84,11 @@ typedef struct {
 	/**
 	 * Name of error.
 	 */
-	char *ErrorName;
+	const char *ErrorName;
 	/**
 	 * Textual representation, not localised, use gettext to get localised string.
 	 */
-	char *ErrorText;
+	const char *ErrorText;
 } PrintErrorEntry;
 
 /**
@@ -156,13 +156,15 @@ static PrintErrorEntry PrintErrorEntries[] = {
 	{ERR_USING_DEFAULTS, "USING_DEFAULTS",		N_("Using default values.")},
 	{ERR_CORRUPTED, "CORRUPTED",			N_("Corrupted data returned by phone.")},
 	{ERR_BADFEATURE, "BADFEATURE",		N_("Bad feature string in configuration.")},
+	{ERR_DISABLED, "DISABLED",		N_("Desired functionality has been disabled on compile time.")},
+	{ERR_SPECIFYCHANNEL, "SPECIFYCHANNEL", N_("Bluetooth configuration requires channel option.")},
 
 	{0,	"",				""}
 };
 
 const char *GSM_ErrorName(GSM_Error e)
 {
-	char	*def 	= NULL;
+	const char	*def 	= NULL;
 	int	i	= 0;
 
 	while (PrintErrorEntries[i].ErrorNum != 0) {
@@ -178,7 +180,7 @@ const char *GSM_ErrorName(GSM_Error e)
 
 const char *GSM_ErrorString(GSM_Error e)
 {
-	char	*def 	= NULL;
+	const char	*def 	= NULL;
 	int	i	= 0;
 
 	while (PrintErrorEntries[i].ErrorNum != 0) {
@@ -211,43 +213,35 @@ const char *GetGammuVersion(void)
 
 GSM_Debug_Info *GSM_GetGlobalDebug()
 {
-	return &di;
+	return &GSM_global_debug;
 }
 
-GSM_Error GSM_SetDebugFileDescriptor(FILE *fd, GSM_Debug_Info *privdi)
+GSM_Error GSM_SetDebugFileDescriptor(FILE *fd, bool closable, GSM_Debug_Info *privdi)
 {
 	privdi->was_lf = true;
 
-	/* If we should use global file descriptor, use it */
-	if (privdi->use_global) {
-		/* Aren't we the changing the global di? */
-		if (privdi != &di) {
-			if (privdi->df != di.df &&
-					privdi->dl!=0 &&
-					fileno(privdi->df) != 1 &&
-					fileno(privdi->df) != 2)
-				fclose(privdi->df);
-			privdi->df = di.df;
-			return ERR_NONE;
-        	}
-    	} else {
-        	/* If we should not use global file descriptor, don't even try use it */
-        	if (privdi->df == di.df) privdi->df = stdout;
-    	}
-
-	if (privdi->df && privdi->df != stdout) {
+	if (privdi->df != NULL
+			&& fileno(privdi->df) != fileno(stderr)
+			&& fileno(privdi->df) != fileno(stdout)
+			&& privdi->closable) {
 		fclose(privdi->df);
 	}
+
 	privdi->df = fd;
+	privdi->closable = closable;
+
 	return ERR_NONE;
 }
 
-GSM_Error GSM_SetDebugFile(char *info, GSM_Debug_Info *privdi)
+GSM_Error GSM_SetDebugFile(const char *info, GSM_Debug_Info *privdi)
 {
 	FILE *testfile;
 
-	if (info != NULL && info[0]!=0 && privdi->dl != 0) {
-		switch (privdi->dl) {
+	if (info == NULL || strlen(info) == 0) {
+		return GSM_SetDebugFileDescriptor(NULL, false, privdi);
+	}
+
+	switch (privdi->dl) {
 		case DL_BINARY:
 			testfile = fopen(info,"wb" COMMIT_FLAG);
 			break;
@@ -266,15 +260,14 @@ GSM_Error GSM_SetDebugFile(char *info, GSM_Debug_Info *privdi)
 			break;
 		default:
 			testfile = fopen(info,"w" COMMIT_FLAG);
-		}
-		if (testfile == NULL) {
-			dbgprintf("Can't open debug file\n");
-			return ERR_CANTOPENFILE;
-		} else {
-			GSM_SetDebugFileDescriptor(testfile, privdi);
-		}
 	}
-	return ERR_NONE;
+
+	if (testfile == NULL) {
+		dbgprintf("Can't open debug file\n");
+		return ERR_CANTOPENFILE;
+	} else {
+		return GSM_SetDebugFileDescriptor(testfile, true, privdi);
+	}
 }
 
 void GSM_LogError(GSM_StateMachine * s, const char * message, const GSM_Error err) {
