@@ -36,6 +36,18 @@
 #define PATH_MAX (MAX_PATH)
 #endif
 
+
+GSM_Debug_Info *GSM_GetDI(GSM_StateMachine *s)
+{
+	GSM_Debug_Info *curdi;
+
+	curdi = &GSM_global_debug;
+	if (s != NULL && s->di.use_global == false) {
+		curdi = &(s->di);
+	}
+	return curdi;
+}
+
 static void GSM_RegisterConnection(GSM_StateMachine *s, unsigned int connection,
 		GSM_Device_Functions *device, GSM_Protocol_Functions *protocol)
 {
@@ -155,7 +167,7 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 	/* Free allocated memory */
 	free(buff);
 
-	if (s->ConnectionType==0) {
+	if (s->ConnectionType == 0) {
 		return ERR_UNKNOWNCONNECTIONTYPESTRING;
 	}
 
@@ -224,8 +236,11 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 #ifdef GSM_ENABLE_BLUEOBEX
 	GSM_RegisterConnection(s, GCT_BLUEOBEX,   &BlueToothDevice,&OBEXProtocol);
 #endif
-	if (s->Device.Functions==NULL || s->Protocol.Functions==NULL)
-			return ERR_SOURCENOTAVAILABLE;
+	if (s->Device.Functions == NULL || s->Protocol.Functions == NULL) {
+		smprintf(s, "Connection %s is know but was disabled on compile time\n", connection);
+		return ERR_DISABLED;
+	}
+
 	return ERR_NONE;
 }
 
@@ -254,7 +269,7 @@ static void GSM_RegisterModule(GSM_StateMachine *s,GSM_Phone_Functions *phone)
  */
 GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 {
-	OnePhoneModel *model;
+	GSM_PhoneModel *model;
 
 	/* Auto model */
 	if (s->CurrentConfig->Model[0] == 0) {
@@ -300,6 +315,45 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 			return ERR_NONE;
 		}
 #endif
+
+#ifdef GSM_ENABLE_NOKIA6510
+		if ( s->ConnectionType ==  GCT_MBUS2 ||
+				s->ConnectionType ==  GCT_FBUS2 ||
+				s->ConnectionType ==  GCT_FBUS2DLR3 ||
+				s->ConnectionType ==  GCT_FBUS2PL2303 ||
+				s->ConnectionType ==  GCT_FBUS2BLUE ||
+				s->ConnectionType ==  GCT_FBUS2IRDA ||
+				s->ConnectionType ==  GCT_DKU5FBUS2 ||
+				s->ConnectionType ==  GCT_DKU2PHONET ||
+				s->ConnectionType ==  GCT_PHONETBLUE ||
+				s->ConnectionType ==  GCT_IRDAPHONET ||
+				s->ConnectionType ==  GCT_BLUEFBUS2 ||
+				s->ConnectionType ==  GCT_BLUEPHONET) {
+			/* Try to detect phone type */
+			if (strcmp(model->model, "unknown") == 0 && model->features[0] == 0) {
+				smprintf(s, "WARNING: phone not known, please report it to authors (see <http://cihar.com/gammu/report>). Thank you\n");
+				if (strncmp(s->Phone.Data.Model, "RM-", 3) == 0) {
+					/* 167 is really a wild guess */
+					if (atoi(s->Phone.Data.Model + 3) > 167) {
+						smprintf(s, "WARNING: Guessed phone as S40/30 compatible!\n");
+						GSM_AddPhoneFeature(model, F_SERIES40_30);
+						GSM_AddPhoneFeature(model, F_FILES2);
+						GSM_AddPhoneFeature(model, F_TODO66);
+						GSM_AddPhoneFeature(model, F_RADIO);
+						GSM_AddPhoneFeature(model, F_NOTES);
+						GSM_AddPhoneFeature(model, F_SMS_FILES);
+					}
+				}
+			}
+
+			/* If phone is S40, use 6510 */
+			if (GSM_IsPhoneFeatureAvailable(model, F_SERIES40_30)) {
+				smprintf(s,"[Module           - \"%s\"]\n", N6510Phone.models);
+				s->Phone.Functions = &N6510Phone;
+				return ERR_NONE;
+			}
+		}
+#endif
 		if (model->model[0] == 0) return ERR_UNKNOWNMODELSTRING;
 	}
 	s->Phone.Functions=NULL;
@@ -357,7 +411,7 @@ GSM_Error GSM_OpenConnection(GSM_StateMachine *s)
 {
 	GSM_Error error;
 
-	if (strcasecmp(s->CurrentConfig->LockDevice,"yes") == 0) {
+	if (s->CurrentConfig->LockDevice != NULL && strcasecmp(s->CurrentConfig->LockDevice,"yes") == 0) {
 		error = lock_device(s->CurrentConfig->Device, &(s->LockFile));
 		if (error != ERR_NONE) return error;
 	}
@@ -470,6 +524,9 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 
 		error=s->Phone.Functions->GetModel(s);
 		if (error != ERR_NONE) return error;
+
+		error=s->Phone.Functions->Terminate(s);
+		if (error != ERR_NONE) return error;
 	}
 	return ERR_NONE;
 }
@@ -516,13 +573,15 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 		s->opened			  = false;
 		s->Phone.Functions		  = NULL;
 
-		s->di 				  = di;
+		s->di 				  = GSM_none_debug;
 		s->di.use_global 		  = s->CurrentConfig->UseGlobalDebugFile;
-		GSM_SetDebugLevel(s->CurrentConfig->DebugLevel, &s->di);
-		error = GSM_SetDebugFile(s->CurrentConfig->DebugFile, &s->di);
-		if (error != ERR_NONE) {
-			GSM_LogError(s, "Init:GSM_SetDebugFile" , error);
-			return error;
+		if (!s->di.use_global) {
+			GSM_SetDebugLevel(s->CurrentConfig->DebugLevel, &s->di);
+			error = GSM_SetDebugFile(s->CurrentConfig->DebugFile, &s->di);
+			if (error != ERR_NONE) {
+				GSM_LogError(s, "Init:GSM_SetDebugFile" , error);
+				return error;
+			}
 		}
 
 		smprintf_level(s, D_ERROR, "[Gammu            - %s built %s %s using %s]\n",
@@ -531,6 +590,9 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 				__DATE__,
 				GetCompiler()
 				);
+		StripSpaces(s->CurrentConfig->Connection);
+		StripSpaces(s->CurrentConfig->Model);
+		StripSpaces(s->CurrentConfig->Device);
 		smprintf_level(s, D_ERROR, "[Connection       - \"%s\"]\n",
 				s->CurrentConfig->Connection);
 		smprintf_level(s, D_ERROR, "[Connection index - %d]\n", i);
@@ -543,7 +605,7 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 					GetOS());
 		}
 
-		if (s->di.dl==DL_BINARY) {
+		if (GSM_GetDI(s)->dl == DL_BINARY) {
 			smprintf(s,"%c",((unsigned char)strlen(VERSION)));
 			smprintf(s,"%s",VERSION);
 		}
@@ -673,7 +735,7 @@ int GSM_ReadDevice (GSM_StateMachine *s, bool waitforreply)
 		res = s->Device.Functions->ReadDevice(s, buff, 255);
 		if (!waitforreply) break;
 		if (res > 0) break;
-		my_sleep(5);
+		usleep(5000);
 		GSM_GetCurrentDateTime(&Date);
 	}
 
@@ -703,7 +765,7 @@ GSM_Error GSM_TerminateConnection(GSM_StateMachine *s)
 	error = GSM_CloseConnection(s);
 	if (error != ERR_NONE) return error;
 
-	if (!s->di.use_global && s->di.dl!=0 && fileno(s->di.df) != 1 && fileno(s->di.df) != 2) fclose(s->di.df);
+	GSM_SetDebugFileDescriptor(NULL, false, &(s->di));
 
 	s->opened = false;
 
@@ -735,7 +797,7 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 		if (GSM_ReadDevice(s, true) != 0) {
 			i = 0;
 		} else {
-			my_sleep(10);
+			usleep(10000);
 		}
 
 		if (length != 0) {
@@ -888,11 +950,11 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 		if (Phone->SentMsg != NULL) {
 			smprintf(s,"LAST SENT frame ");
 			smprintf(s, "type 0x%02X/length " SIZE_T_FORMAT, Phone->SentMsg->Type, Phone->SentMsg->Length);
-			DumpMessage(&s->di, Phone->SentMsg->Buffer, Phone->SentMsg->Length);
+			DumpMessage(GSM_GetDI(s), Phone->SentMsg->Buffer, Phone->SentMsg->Length);
 		}
 		smprintf(s, "RECEIVED frame ");
 		smprintf(s, "type 0x%02X/length 0x" SIZE_T_HEX_FORMAT "/" SIZE_T_FORMAT, msg->Type, msg->Length, msg->Length);
-		DumpMessage(&s->di, msg->Buffer, msg->Length);
+		DumpMessage(GSM_GetDI(s), msg->Buffer, msg->Length);
 		smprintf(s, "\n");
 	}
 
@@ -1203,12 +1265,18 @@ fail:
 
 void GSM_DumpMessageLevel2_Text(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, const char *text)
 {
-	if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-	    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
+	GSM_Debug_Info *curdi;
+
+	curdi = GSM_GetDI(s);
+
+	if (curdi->dl == DL_TEXT ||
+			curdi->dl == DL_TEXTALL ||
+			curdi->dl == DL_TEXTDATE ||
+			curdi->dl == DL_TEXTALLDATE) {
 		smprintf(s, "%s", text);
 		smprintf(s, "type 0x%02X/length 0x%02X/%i",
 				type, messagesize, messagesize);
-		DumpMessage(&s->di, message, messagesize);
+		DumpMessage(curdi, message, messagesize);
 		if (messagesize == 0)
 			smprintf(s,"\n");
 	}
@@ -1227,8 +1295,11 @@ void GSM_DumpMessageLevel2Recv(GSM_StateMachine *s, unsigned const char *message
 void GSM_DumpMessageLevel3_Custom(GSM_StateMachine *s, unsigned const char *message, int messagesize, int type, int direction)
 {
 	int i;
+	GSM_Debug_Info *curdi;
 
-	if (s->di.dl==DL_BINARY) {
+	curdi = GSM_GetDI(s);
+
+	if (curdi->dl == DL_BINARY) {
 		smprintf(s,"%c", direction);
 		smprintf(s,"%c",type);
 		smprintf(s,"%c",messagesize/256);
@@ -1251,14 +1322,16 @@ int smprintf(GSM_StateMachine *s, const char *format, ...)
 {
 	va_list		argp;
 	int 		result=0;
-	char		buffer[2000];
+	GSM_Debug_Info *curdi;
+
+	curdi = &GSM_global_debug;
+	if (s != NULL && s->di.use_global == false) {
+		curdi = &(s->di);
+	}
 
 	va_start(argp, format);
 
-	if ((s != NULL && s->di.df != 0) || (s == NULL && di.df != 0)) {
-		result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
-		result = smfprintf((s == NULL) ? &di : &(s->di), "%s", buffer);
-	}
+	result = dbg_vprintf(curdi, format, argp);
 
 	va_end(argp);
 	return result;
@@ -1269,46 +1342,54 @@ int smprintf_level(GSM_StateMachine * s, GSM_DebugSeverity severity, const char
 {
 	va_list		argp;
 	int 		result=0;
-	char		buffer[2000];
+	GSM_Debug_Info *curdi;
+
+	curdi = GSM_GetDI(s);
 
 	if (severity == D_TEXT) {
-		if (s->di.dl != DL_TEXT &&
-				s->di.dl != DL_TEXTALL &&
-				s->di.dl != DL_TEXTDATE &&
-				s->di.dl != DL_TEXTALLDATE) {
+		if (curdi->dl != DL_TEXT &&
+				curdi->dl != DL_TEXTALL &&
+				curdi->dl != DL_TEXTDATE &&
+				curdi->dl != DL_TEXTALLDATE) {
 			return 0;
 		}
 	} else if (severity == D_ERROR) {
-		if (s->di.dl != DL_TEXT &&
-				s->di.dl != DL_TEXTALL &&
-				s->di.dl != DL_TEXTDATE &&
-				s->di.dl != DL_TEXTALLDATE &&
-				s->di.dl != DL_TEXTERROR &&
-				s->di.dl != DL_TEXTERRORDATE) {
+		if (curdi->dl != DL_TEXT &&
+				curdi->dl != DL_TEXTALL &&
+				curdi->dl != DL_TEXTDATE &&
+				curdi->dl != DL_TEXTALLDATE &&
+				curdi->dl != DL_TEXTERROR &&
+				curdi->dl != DL_TEXTERRORDATE) {
 			return 0;
 		}
 	}
 	va_start(argp, format);
 
-	if ((s != NULL && s->di.df != 0) || (s == NULL && di.df != 0)) {
-		result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
-		result = smfprintf((s == NULL) ? &di : &(s->di), "%s", buffer);
-	}
+	result = dbg_vprintf(curdi, format, argp);
 
 	va_end(argp);
 	return result;
 }
 
-void GSM_OSErrorInfo(GSM_StateMachine *s, char *description)
+void GSM_OSErrorInfo(GSM_StateMachine *s, const char *description)
 {
 #ifdef WIN32
 	int 		i;
 	unsigned char 	*lpMsgBuf;
+#endif
+	GSM_Debug_Info *curdi;
 
+	curdi = GSM_GetDI(s);
+
+#ifdef WIN32
 	/* We don't use errno in win32 - GetLastError gives better info */
 	if (GetLastError() != 0) {
-		if (s->di.dl == DL_TEXTERROR || s->di.dl == DL_TEXT || s->di.dl == DL_TEXTALL ||
-		    s->di.dl == DL_TEXTERRORDATE || s->di.dl == DL_TEXTDATE || s->di.dl == DL_TEXTALLDATE) {
+		if (curdi->dl == DL_TEXTERROR ||
+				curdi->dl == DL_TEXT ||
+				curdi->dl == DL_TEXTALL ||
+				curdi->dl == DL_TEXTERRORDATE ||
+				curdi->dl == DL_TEXTDATE ||
+				curdi->dl == DL_TEXTALLDATE) {
 			FormatMessage(
 				FORMAT_MESSAGE_ALLOCATE_BUFFER |
 				FORMAT_MESSAGE_FROM_SYSTEM |
@@ -1329,15 +1410,19 @@ void GSM_OSErrorInfo(GSM_StateMachine *s, char *description)
 			LocalFree(lpMsgBuf);
 		}
 	}
-	return;
-#endif
+#else
 
 	if (errno!=-1) {
-		if (s->di.dl == DL_TEXTERROR || s->di.dl == DL_TEXT || s->di.dl == DL_TEXTALL ||
-		    s->di.dl == DL_TEXTERRORDATE || s->di.dl == DL_TEXTDATE || s->di.dl == DL_TEXTALLDATE) {
+		if (curdi->dl == DL_TEXTERROR ||
+				curdi->dl == DL_TEXT ||
+				curdi->dl == DL_TEXTALL ||
+				curdi->dl == DL_TEXTERRORDATE ||
+				curdi->dl == DL_TEXTDATE ||
+				curdi->dl == DL_TEXTALLDATE) {
 			smprintf(s,"[System error     - %s, %i, \"%s\"]\n",description,errno,strerror(errno));
 		}
 	}
+#endif
 }
 
 #ifdef GSM_ENABLE_BACKUP
@@ -1476,7 +1561,7 @@ GSM_ConnectionType GSM_GetUsedConnection(GSM_StateMachine *s)
 	return s->ConnectionType;
 }
 
-OnePhoneModel *GSM_GetModelInfo(GSM_StateMachine *s)
+GSM_PhoneModel *GSM_GetModelInfo(GSM_StateMachine *s)
 {
 	return s->Phone.Data.ModelInfo;
 }
