@@ -29,7 +29,7 @@
 #include "coding.h"
 
 /* function changes #10 #13 chars to \n \r */
-char *EncodeUnicodeSpecialChars(unsigned char *buffer)
+char *EncodeUnicodeSpecialChars(const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0;
 	static unsigned char	Buf[20000];
@@ -83,7 +83,7 @@ char *EncodeUnicodeSpecialChars(unsigned char *buffer)
 }
 
 /* function changes #10 #13 chars to \n \r */
-char *EncodeSpecialChars(unsigned char *buffer)
+char *EncodeSpecialChars(const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0;
 	static unsigned char	Buf[10000];
@@ -111,7 +111,7 @@ char *EncodeSpecialChars(unsigned char *buffer)
 	return Buf;
 }
 
-char *DecodeUnicodeSpecialChars(unsigned char *buffer)
+char *DecodeUnicodeSpecialChars(const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0, level=0;
 	static unsigned char	Buf[10000];
@@ -150,7 +150,7 @@ char *DecodeUnicodeSpecialChars(unsigned char *buffer)
 	return Buf;
 }
 
-char *DecodeSpecialChars(unsigned char *buffer)
+char *DecodeSpecialChars(const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0, level=0;
 	static unsigned char	Buf[10000];
@@ -199,7 +199,7 @@ char *strcasestr(const char *s, const char *find)
 		} while (strncasecmp(s, find, len) != 0);
 		s--;
 	}
-	return ((char *)s);
+	return (char *)s;
 }
 #endif
 
@@ -279,12 +279,12 @@ char *DecodeUnicodeConsole(const unsigned char *src)
 {
  	static char dest[500];
 
-	if (di.coding[0] != 0) {
-		if (!strcmp(di.coding,"utf8")) {
+	if (GSM_global_debug.coding[0] != 0) {
+		if (!strcmp(GSM_global_debug.coding,"utf8")) {
 			EncodeUTF8(dest, src);
 		} else {
 #ifdef WIN32
-			setlocale(LC_ALL, di.coding);
+			setlocale(LC_ALL, GSM_global_debug.coding);
 #endif
 			DecodeUnicode(src,dest);
 		}
@@ -520,7 +520,7 @@ void DecodeDefault (unsigned char *dest, const unsigned char *src, size_t len, b
 	bool	FoundSpecial = false;
 
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, src, len);
+	DumpMessageText(&GSM_global_debug, src, len);
 #endif
 
 	for (i = 0; i < len; i++) {
@@ -559,7 +559,7 @@ void DecodeDefault (unsigned char *dest, const unsigned char *src, size_t len, b
 	dest[current++]=0;
 	dest[current++]=0;
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, dest, UnicodeLength(dest)*2);
+	DumpMessageText(&GSM_global_debug, dest, UnicodeLength(dest)*2);
 #endif
 }
 
@@ -585,7 +585,7 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, size_t *len, b
 	bool	FoundSpecial,FoundNormal;
 
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, src, (*len)*2);
+	DumpMessageText(&GSM_global_debug, src, (*len)*2);
 #endif
 
 	for (i = 0; i < *len; i++) {
@@ -653,7 +653,7 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, size_t *len, b
 	}
 	dest[current]=0;
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, dest, current);
+	DumpMessageText(&GSM_global_debug, dest, current);
 #endif
 
 	*len = current;
@@ -952,17 +952,17 @@ void ReadUnicodeFile(unsigned char *Dest, const unsigned char *Source)
 
 INLINE int GetBit(unsigned char *Buffer, size_t BitNum)
 {
-	return Buffer[BitNum/8] & 1<<(7-(BitNum%8));
+	return Buffer[BitNum / 8] & (1 << (7 - (BitNum % 8)));
 }
 
 INLINE int SetBit(unsigned char *Buffer, size_t BitNum)
 {
-	return Buffer[BitNum/8] |= 1<<(7-(BitNum%8));
+	return Buffer[BitNum / 8] |= 1 << (7 - (BitNum % 8));
 }
 
 int ClearBit(unsigned char *Buffer, size_t BitNum)
 {
-	return Buffer[BitNum/8] &= 255 - (1 << (7-(BitNum%8)));
+	return Buffer[BitNum / 8] &= 255 - (1 << (7 - (BitNum % 8)));
 }
 
 void BufferAlign(unsigned char *Destination, size_t *CurrentBit)
@@ -1166,13 +1166,7 @@ int strncasecmp (const char *s1, const char *s2, size_t n)
 		++p2;
 	} while (c1 == c2);
 
-	if (UCHAR_MAX <= INT_MAX)
-		return c1 - c2;
-	else
-	/* On machines where 'char' and 'int' are types of the same size, the
-	difference of two 'unsigned char' values - including the sign bit -
-	doesn't fit in an 'int'.  */
-		return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
+	return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
 }
 #undef TOLOWER
 #endif
@@ -1203,13 +1197,7 @@ int strcasecmp (const char *s1, const char *s2)
 		++p2;
 	} while (c1 == c2);
 
-	if (UCHAR_MAX <= INT_MAX)
-		return c1 - c2;
-	else
-	/* On machines where 'char' and 'int' are types of the same size, the
-	difference of two 'unsigned char' values - including the sign bit -
-	doesn't fit in an 'int'.  */
-		return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
+	return (c1 > c2 ? 1 : c1 < c2 ? -1 : 0);
 }
 #undef TOLOWER
 #endif
@@ -1371,7 +1359,7 @@ ret0:
 #undef tolowerwchar
 }
 
-void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int MaxLen, bool MergeLines)
+GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, size_t MaxOutLen, bool MergeLines)
 {
 	bool skip = false;
 	bool quoted_printable = false;
@@ -1381,19 +1369,19 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 
 	OutBuffer[0] = 0;
 	pos = 0;
-	if (Buffer == NULL) return;
+	if (Buffer == NULL) return ERR_NONE;
 	while ((*Pos) < MaxLen) {
 		switch (Buffer[*Pos]) {
 		case 0x00:
-			return;
+			return ERR_NONE;
 		case 0x0A:
 		case 0x0D:
 			if (skip) {
 				if (Buffer[*Pos] == 0x0d) {
-					if (was_cr && skip) return;
+					if (was_cr && skip) return ERR_NONE;
 					was_cr = true;
 				} else {
-					if (was_lf && skip) return;
+					if (was_lf && skip) return ERR_NONE;
 					was_lf = true;
 				}
 			}
@@ -1418,7 +1406,7 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 						break;
 					}
 				}
-				return;
+				return ERR_NONE;
 			}
 			break;
 		default:
@@ -1431,11 +1419,88 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 			OutBuffer[pos]     = Buffer[*Pos];
 			pos++;
 			OutBuffer[pos] = 0;
+			if (pos + 1 >= MaxOutLen) return ERR_MOREMEMORY;
 		}
 		(*Pos)++;
 	}
+	return ERR_NONE;
 }
 
+GSM_Error GSM_GetVCSLine(char **OutBuffer, char *Buffer, size_t *Pos, size_t MaxLen, bool MergeLines)
+{
+	bool skip = false;
+	bool quoted_printable = false;
+	bool was_cr = false, was_lf = false;
+	size_t pos;
+	int tmp;
+	size_t OutLen = 200;
+
+	*OutBuffer = malloc(OutLen);
+	if (*OutBuffer == NULL) return ERR_MOREMEMORY;
+	(*OutBuffer)[0] = 0;
+	pos = 0;
+	if (Buffer == NULL) return ERR_NONE;
+	while ((*Pos) < MaxLen) {
+		switch (Buffer[*Pos]) {
+		case 0x00:
+			return ERR_NONE;
+		case 0x0A:
+		case 0x0D:
+			if (skip) {
+				if (Buffer[*Pos] == 0x0d) {
+					if (was_cr && skip) return ERR_NONE;
+					was_cr = true;
+				} else {
+					if (was_lf && skip) return ERR_NONE;
+					was_lf = true;
+				}
+			}
+			if (pos != 0 && !skip) {
+				if (MergeLines) {
+					/* (Quote printable new line) Does string end with = ? */
+					if ((*OutBuffer)[pos - 1] == '=' && quoted_printable) {
+						pos--;
+						(*OutBuffer)[pos] = 0;
+						skip = true;
+						was_cr = (Buffer[*Pos] == 0x0d);
+						was_lf = (Buffer[*Pos] == 0x0a);
+						break;
+					}
+					/* (vCard continuation) Next line start with space? */
+					tmp = *Pos + 1;
+					if (Buffer[*Pos + 1] == 0x0a || Buffer[*Pos + 1] == 0x0d) {
+						tmp += 1;
+					}
+					if (Buffer[tmp] == ' ') {
+						*Pos = tmp;
+						break;
+					}
+				}
+				return ERR_NONE;
+			}
+			break;
+		default:
+			/* Detect quoted printable for possible escaping */
+			if (Buffer[*Pos] == ':' &&
+					strstr(*OutBuffer, ";ENCODING=QUOTED-PRINTABLE") != NULL) {
+				quoted_printable = true;
+			}
+			skip = false;
+			(*OutBuffer)[pos]     = Buffer[*Pos];
+			pos++;
+			(*OutBuffer)[pos] = 0;
+			if (pos + 2 >= OutLen) {
+				OutLen += 100;
+				*OutBuffer = realloc(*OutBuffer, OutLen);
+				if (*OutBuffer == NULL) return ERR_MOREMEMORY;
+			}
+		}
+		(*Pos)++;
+	}
+	return ERR_NONE;
+}
+
+
 void StringToDouble(char *text, double *d)
 {
 	bool 		before=true;
@@ -1764,7 +1829,7 @@ OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
-static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], size_t len)
+static void EncodeBASE64Block(const unsigned char in[3], char out[4], const size_t len)
 {
 	/* BASE64 translation Table as described in RFC1113 */
 	unsigned char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -1775,7 +1840,7 @@ static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], size_t
 	out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
 }
 
-void EncodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
+void EncodeBASE64(const unsigned char *Input, char *Output, const size_t Length)
 {
 	unsigned char 	in[3], out[4];
 	size_t 		i, pos = 0, len, outpos = 0;
@@ -1799,14 +1864,14 @@ void EncodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Leng
 	Output[outpos++] = 0;
 }
 
-static void DecodeBASE64Block(unsigned char in[4], unsigned char out[3])
+static void DecodeBASE64Block(const char in[4], unsigned char out[3])
 {
-	out[0] = (unsigned char) (in[0] << 2 | in[1] >> 4);
-	out[1] = (unsigned char) (in[1] << 4 | in[2] >> 2);
+	out[0] = (unsigned char) ((in[0] << 2) | (in[1] >> 4));
+	out[1] = (unsigned char) ((in[1] << 4) | (in[2] >> 2));
 	out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
 }
 
-int DecodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
+int DecodeBASE64(const char *Input, unsigned char *Output, const size_t Length)
 {
 	unsigned char 	cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";
 	unsigned char 	in[4], out[3], v;
@@ -1851,7 +1916,9 @@ int DecodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Lengt
 bool IconvDecode(const char *charset, const char *input, const size_t inlen, unsigned char *output, size_t outlen)
 {
 	iconv_t ic;
-	size_t rest = inlen;
+	/* Add one to convert also trailing zero, this is broken for
+	 * multibyte input, but we don't use iconv for this so far */
+	size_t rest = inlen + 1;
 	SECOND_ICONV_ARG in;
 	char *out;
 
@@ -1893,4 +1960,3 @@ bool IconvEncode(const char *charset, const unsigned char *input, const size_t i
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
-
