@@ -16,6 +16,7 @@
 #endif
 
 #include <gammu-unicode.h>
+#include <gammu-error.h>
 
 #ifndef HAVE_WCHAR_T
 typedef		int wchar_t;
@@ -46,8 +47,8 @@ void 		ReverseUnicodeString		(unsigned char *String);
 void 		DecodeUnicodeSpecialNOKIAChars	(unsigned char *dest, const unsigned char *src, int len);
 void 		EncodeUnicodeSpecialNOKIAChars	(unsigned char *dest, const unsigned char *src, int len);
 
-char 		*EncodeUnicodeSpecialChars	(unsigned char *buffer);
-char 		*DecodeUnicodeSpecialChars	(unsigned char *buffer);
+char 		*EncodeUnicodeSpecialChars	(const unsigned char *buffer);
+char 		*DecodeUnicodeSpecialChars	(const unsigned char *buffer);
 
 /* ------------------------------- BCD ------------------------------------- */
 unsigned char	EncodeWithBCDAlphabet		(int value);
@@ -70,8 +71,8 @@ void		DecodeISO88591QuotedPrintable	(unsigned char *dest, const unsigned char *s
 void DecodeXMLUTF8(unsigned char *dest, const unsigned char *src, int len);
 
 /* ------------------------------- BASE64 ---------------------------------- */
-void 		EncodeBASE64			(const unsigned char *Input, unsigned char *Output, size_t Length);
-int 		DecodeBASE64			(const unsigned char *Input, unsigned char *Output, size_t Length);
+void 		EncodeBASE64			(const unsigned char *Input, char *Output, const size_t Length);
+int 		DecodeBASE64			(const char *Input, unsigned char *Output, const size_t Length);
 
 /* ---------------------- DefaultAlphabet for SMS -------------------------- */
 void 		EncodeDefault			(unsigned char *dest, const unsigned char *src, size_t *len, bool UseExtensions, unsigned char *ExtraAlphabet);
@@ -140,6 +141,20 @@ extern int strcasecmp (const char *s1, const char *s2);
 #endif
 
 
+/**
+ * Gets VCS line from buffer.
+ *
+ * @param MergeLines: Determine whether merge lines as vCard style
+ * continuation or quoted printable continutaion.
+ * @param Buffer: Data source to parse.
+ * @param Pos: Current position in data.
+ * @param OutBuffer: Pointer to buffer pointer, which will be allocated.
+ * @param MaxLen: Maximal length of data to process.
+ *
+ * \return ERR_NONE on success, ERR_MOREMEMORY if buffer is too small.
+ */
+GSM_Error GSM_GetVCSLine(char **OutBuffer, char *Buffer, size_t *Pos, size_t MaxLen, bool MergeLines);
+
 /**
  * Gets line from buffer.
  *
@@ -148,14 +163,18 @@ extern int strcasecmp (const char *s1, const char *s2);
  * @param Buffer: Data source to parse.
  * @param Pos: Current position in data.
  * @param OutBuffer: Buffer where line will be written.
- * @param MaxLen: Maximal length of data to write.
+ * @param MaxLen: Maximal length of data to process.
+ * @param MaxOutLen: Size of output buffer.
+ *
+ * \return ERR_NONE on success, ERR_MOREMEMORY if buffer is too small.
  */
-void MyGetLine		(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int MaxLen, bool MergeLines);
+GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, size_t MaxOutLen, bool MergeLines);
 
-char *EncodeSpecialChars(unsigned char *buffer);
-char *DecodeSpecialChars(unsigned char *buffer);
+char *EncodeSpecialChars(const unsigned char *buffer);
+char *DecodeSpecialChars(const unsigned char *buffer);
 
 #ifdef ICONV_FOUND
+
 bool IconvDecode(const char *charset, const char *input, const size_t inlen, unsigned char *output, size_t outlen);
 bool IconvEncode(const char *charset, const unsigned char *input, const size_t inlen, char *output, size_t outlen);
 #endif
