@@ -66,6 +66,8 @@ int RecalcDateTime(struct tm *st, const int year, const int month, const int day
 	st->tm_mon = month - 1;
 	st->tm_mday = day;
 
+	st->tm_isdst = -1; /* FIXME */
+
 	return 1;
 }
 
@@ -135,6 +137,15 @@ char *DayOfWeek (unsigned int year, unsigned int month, unsigned int day)
 	return DayOfWeekChar;
 }
 
+int GSM_GetLocalTimezoneOffset() {
+	struct tm *tg, *tl;
+	time_t now = time(NULL);
+	tg = gmtime(&now);
+	tl = localtime(&now);
+	// Returns offset including daylight saving (found as boolean in tl.tm_isdst).
+	return (tl->tm_hour - tg->tm_hour) * 3600 + (tl->tm_min - tg->tm_min) * 60 + (tl->tm_sec - tg->tm_sec);
+}
+
 void Fill_GSM_DateTime(GSM_DateTime *Date, time_t timet)
 {
 	struct tm *now;
@@ -146,7 +157,7 @@ void Fill_GSM_DateTime(GSM_DateTime *Date, time_t timet)
 	Date->Hour	= now->tm_hour;
 	Date->Minute	= now->tm_min;
 	Date->Second	= now->tm_sec;
-	Date->Timezone	= 0; /* FIXME: can we get it from the OS? */
+	Date->Timezone	= GSM_GetLocalTimezoneOffset();
 }
 
 void GSM_GetCurrentDateTime (GSM_DateTime *Date)
@@ -238,11 +249,8 @@ char *OSDateTime (GSM_DateTime dt, bool TimeZone)
 	/* This is not Y2K safe */
 	strftime(retval2, 200, "%c", &timeptr);
 	if (TimeZone) {
-		if (dt.Timezone >= 0) {
-			snprintf(retval, sizeof(retval) - 1, " +%02i00",dt.Timezone);
-		} else {
-			snprintf(retval, sizeof(retval) - 1, " -%02i00",dt.Timezone);
-		}
+		snprintf(retval, sizeof(retval) - 1, " %+03i%02i",
+			dt.Timezone / 3600, abs((dt.Timezone % 3600) / 60));
 		strcat(retval2,retval);
 	}
 	/* If don't have weekday name, include it */
@@ -282,7 +290,7 @@ char *OSDate (GSM_DateTime dt)
 	timeptr.tm_min  	= dt.Minute;
 	timeptr.tm_sec  	= dt.Second;
 	timeptr.tm_wday 	= GetDayOfWeek(dt.Year, dt.Month, dt.Day);
-#ifdef _BSD_SOURCE
+#ifdef HAVE_STRUCT_TM_TM_ZONE
 	timeptr.tm_zone		= NULL;
 #endif
 
@@ -357,7 +365,7 @@ void FreeLines(GSM_CutLines *lines)
 	lines->allocated = 0;
 }
 
-void SplitLines(const char *message, int messagesize, GSM_CutLines *lines, unsigned char *whitespaces, int spaceslen, bool eot)
+void SplitLines(const char *message, int messagesize, GSM_CutLines *lines, const char *whitespaces, int spaceslen, bool eot)
 {
 	int 	i,number=0,j;
 	bool 	whitespace=true, nowwhite;
@@ -401,11 +409,17 @@ void SplitLines(const char *message, int messagesize, GSM_CutLines *lines, unsig
 		lines->numbers[number] = messagesize;
 }
 
-char *GetLineString(const char *message, const GSM_CutLines *lines, int start)
+const char *GetLineString(const char *message, const GSM_CutLines *lines, int start)
 {
 	static char *retval = NULL;
 	int len;
 
+	if (message == NULL) {
+		free(retval);
+		retval = NULL;
+		return NULL;
+	}
+
 	len = GetLineLength(message, lines, start);
 	retval = realloc(retval, len + 1);
 	if (retval == NULL) {
@@ -432,48 +446,27 @@ void CopyLineString(char *dest, const char *src, const GSM_CutLines *lines, int
 	dest[len] = 0;
 }
 
-GSM_Debug_Info di = {0,NULL,false,"",false};
+GSM_Debug_Info GSM_none_debug = {
+	0,
+	NULL,
+	false,
+	"",
+	false,
+	false
+	};
 
-#ifdef DEBUG
-PRINTF_STYLE(1, 2)
-int dbgprintf(const char *format, ...)
+GSM_Debug_Info GSM_global_debug = {
+	0,
+	NULL,
+	false,
+	"",
+	false,
+	false
+	};
+
+PRINTF_STYLE(2, 0)
+int dbg_vprintf(GSM_Debug_Info *d, const char *format, va_list argp)
 {
-	va_list			argp;
-	int 			result;
-	static unsigned char 	nextline[2000]="";
-	unsigned char		buffer[2000];
-	GSM_DateTime 		date_time;
-
-	va_start(argp, format);
-	result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
-	va_end(argp);
-
-	if (di.df != NULL && (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE)) {
-		strcat(nextline, buffer);
-
-		if (strstr(buffer, "\n") != NULL) {
-			if (di.dl == DL_TEXTALLDATE) {
-				GSM_GetCurrentDateTime(&date_time);
-				fprintf(di.df,"%s %4d/%02d/%02d %02d:%02d:%02d: %s",
-		                	DayOfWeek(date_time.Year, date_time.Month, date_time.Day),
-		                	date_time.Year, date_time.Month, date_time.Day,
-		                	date_time.Hour, date_time.Minute, date_time.Second,nextline);
-			} else {
-				fprintf(di.df, "%s", nextline);
-			}
-			strcpy(nextline, "");
-		}
-
-		return result;
-	}
-	return 0;
-}
-#endif
-
-PRINTF_STYLE(2, 3)
-int smfprintf(GSM_Debug_Info *d, const char *format, ...)
-{
-        va_list 		argp;
 	int 			result=0;
 	char			buffer[3000];
 	char			*pos, *end;
@@ -482,19 +475,12 @@ int smfprintf(GSM_Debug_Info *d, const char *format, ...)
 	FILE			*f;
 	Debug_Level		l;
 
-	if (d->use_global) {
-		f = di.df;
-		l = di.dl;
-	} else {
-		f = d->df;
-		l = d->dl;
-	}
+	f = d->df;
+	l = d->dl;
 
 	if (l == DL_NONE || f == NULL) return 0;
 
-	va_start(argp, format);
 	result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
-	va_end(argp);
 	pos = buffer;
 
 	while (*pos != 0) {
@@ -546,6 +532,41 @@ int smfprintf(GSM_Debug_Info *d, const char *format, ...)
 	return result;
 }
 
+#ifdef DEBUG
+PRINTF_STYLE(1, 2)
+int dbgprintf(const char *format, ...)
+{
+        va_list 		argp;
+	int 			result;
+
+	va_start(argp, format);
+	result = dbg_vprintf(&GSM_global_debug, format, argp);
+	va_end(argp);
+
+	return result;
+}
+#endif
+
+PRINTF_STYLE(2, 3)
+int smfprintf(GSM_Debug_Info *d, const char *format, ...)
+{
+        va_list 		argp;
+	int 			result;
+	GSM_Debug_Info		*tmpdi;
+
+	if (d->use_global) {
+		tmpdi = &GSM_global_debug;
+	} else {
+		tmpdi = d;
+	}
+
+	va_start(argp, format);
+	result = dbg_vprintf(tmpdi, format, argp);
+	va_end(argp);
+
+	return result;
+}
+
 bool GSM_SetDebugLevel(const char *info, GSM_Debug_Info *privdi)
 {
 	if (info == NULL)			{privdi->dl = DL_NONE;	 	return true;}
@@ -568,52 +589,75 @@ bool GSM_SetDebugCoding(const char *info, GSM_Debug_Info *privdi)
 
 bool GSM_SetDebugGlobal(bool info, GSM_Debug_Info *privdi)
 {
-	if (privdi->use_global && info) {
-		GSM_SetDebugFileDescriptor(NULL, privdi);
-	}
 	privdi->use_global = info;
 	return true;
 }
 
+#define CHARS_PER_LINE (16)
+
 /* Dumps a message */
-void DumpMessage(GSM_Debug_Info *d, const unsigned char *message, int messagesize)
+void DumpMessage(GSM_Debug_Info *d, const unsigned char *message, const int messagesize)
 {
-	int 		i,j=0,len=16;
-	unsigned char	buffer[200];
+	int i, j = 0;
+	char buffer[(CHARS_PER_LINE * 5) + 1];
 
 	if (d->df == NULL || messagesize == 0) return;
 
 	smfprintf(d, "\n");
 
-	memset(buffer,0x20,sizeof(buffer));
-	buffer[len*5-1]=0;
+	memset(buffer, ' ', CHARS_PER_LINE * 5);
+	buffer[CHARS_PER_LINE * 5] = 0;
 
 	for (i = 0; i < messagesize; i++) {
-		snprintf(buffer+j*4, sizeof(buffer) - j*4 - 1, "%02X",message[i]);
-		buffer[j*4+2] = 0x20;
-		/* 9 = tab */
-		if (isprint(message[i]) && message[i]!=0x09
-			/* 0x01 = beep in windows xp */
-			&& message[i]!=0x01
-			/* these are empty in windows xp */
-			&& message[i]!=0x85 && message[i]!=0x95
-  			&& message[i]!=0xA6 && message[i]!=0xB7) {
-			if (j != len-1) buffer[j*4+2] 	= message[i];
-			buffer[(len-1)*4+j+3]		= message[i];
+		/* Write hex number */
+		snprintf(buffer + (j * 4), 3, "%02X", message[i]);
+		buffer[(j * 4) + 2] = ' '; /* wipe snprintf's \0 */
+
+		/* Write char if possible */
+		if (isprint(message[i])
+				/* 0x09 = tab */
+				&& message[i] != 0x09
+				/* 0x01 = beep in windows xp */
+				&& message[i] != 0x01
+				/* these are empty in windows xp */
+				&& message[i] != 0x85
+				&& message[i] != 0x95
+				&& message[i] != 0xA6
+				&& message[i] != 0xB7) {
+			buffer[(j * 4) + 2] = message[i];
+			buffer[(CHARS_PER_LINE - 1) * 4 + j + 4] = message[i];
 		} else {
-			buffer[(len-1)*4+j+3]		= '.';
+			buffer[(CHARS_PER_LINE - 1) * 4 + j + 4] = '.';
 		}
-		if (j != len-1 && i != messagesize-1) buffer[j*4+3] = '|';
-		if (j == len-1) {
+
+		/* Write char separator */
+		if (j != CHARS_PER_LINE - 1 && i != messagesize - 1) {
+			buffer[j * 4 + 3] = '|';
+		}
+
+		/* Print out buffer */
+		if (j == CHARS_PER_LINE - 1) {
 			smfprintf(d, "%s\n", buffer);
-			memset(buffer,0x20,sizeof(buffer));
-			buffer[len*5-1]=0;
+			memset(buffer, ' ', CHARS_PER_LINE * 5);
 			j = 0;
 		} else {
 			j++;
 		}
 	}
-	if (j != 0) smfprintf(d, "%s\n", buffer);
+
+	/* Anything remains to be printed? */
+	if (j != 0) {
+		smfprintf(d, "%s\n", buffer);
+	}
+}
+
+#undef CHARS_PER_LINE
+
+void DumpMessageText(GSM_Debug_Info *d, const unsigned char *message, const int messagesize)
+{
+	if (d->dl != DL_TEXTALL && d->dl == DL_TEXTALLDATE) return;
+	DumpMessage(d, message, messagesize);
+
 }
 
 const char *GetOS(void)
@@ -788,6 +832,25 @@ bool GSM_IsNewerVersion(const char *latest_version, const char *current_version)
 	return false;
 }
 
+void StripSpaces(char *buff)
+{
+	ssize_t i;
+
+	i = 0;
+	while(iswspace(buff[i])) {
+		i++;
+	}
+	if (i > 0) {
+		memmove(buff, buff + i, strlen(buff + i));
+	}
+	i = strlen(buff) - 1;
+	while(iswspace(buff[i]) && i >= 0) {
+		buff[i] = 0;
+		i--;
+	}
+
+}
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
