@@ -282,6 +282,7 @@ typedef struct {
 	size_t			NumberLength;
 	size_t			TextLength;
 	int			MemorySize;
+	int			MemoryUsed;
 
 	GSM_SMSMemoryStatus	LastSMSStatus;
 	int			LastSMSRead;
@@ -361,6 +362,10 @@ typedef struct {
 	 * Which folder do we read SMS from.
 	 */
 	int			SMSReadFolder;
+	/**
+	 * Mode of SQWE (Siemens phones and switching to OBEX).
+	 */
+	int			SQWEMode;
 } GSM_Phone_ATGENData;
 
 /**
