@@ -63,6 +63,13 @@ GSM_Error ATOBEX_SetATMode(GSM_StateMachine *s)
 	s->Phone.Functions->ReplyFunctions	= ATGENReplyFunctions;
 	Priv->Mode				= ATOBEX_ModeAT;
 
+	/* Terminate SQWE Obex mode */
+	if (Priv->HasOBEX == ATOBEX_OBEX_SQWE) {
+		sleep(1);
+		error = GSM_WaitFor (s, "+++", 3, 0x00, 100, ID_IncomingFrame);
+		if (error != ERR_NONE) return error;
+	}
+
 	/* Initialise AT protocol */
 	error = s->Protocol.Functions->Initialise(s);
 	if (error != ERR_NONE) return error;
@@ -103,23 +110,23 @@ GSM_Error ATOBEX_SetOBEXMode(GSM_StateMachine *s, OBEX_Service service)
 	switch (Priv->HasOBEX) {
 		case ATOBEX_OBEX_CPROT0:
 			/* 3GPP TS 27.007 standard */
-			error = GSM_WaitFor (s, "AT+CPROT=0\r", 11, 0x00, 4, ID_SetOBEX);
+			error = GSM_WaitFor (s, "AT+CPROT=0\r", 11, 0x00, 20, ID_SetOBEX);
 			break;
 		case ATOBEX_OBEX_EOBEX:
 			/* Sony-Ericsson extension */
-			error = GSM_WaitFor (s, "AT*EOBEX\r", 9, 0x00, 4, ID_SetOBEX);
+			error = GSM_WaitFor (s, "AT*EOBEX\r", 9, 0x00, 20, ID_SetOBEX);
 			break;
 		case ATOBEX_OBEX_MODE22:
 			/* Motorola extension */
-			error = GSM_WaitFor (s, "AT+MODE=22\r", 11, 0x00, 4, ID_SetOBEX);
+			error = GSM_WaitFor (s, "AT+MODE=22\r", 11, 0x00, 20, ID_SetOBEX);
 			break;
 		case ATOBEX_OBEX_XLNK:
 			/* Sharp extension */
-			error = GSM_WaitFor (s, "AT+XLNK\r", 8, 0x00, 4, ID_SetOBEX);
+			error = GSM_WaitFor (s, "AT+XLNK\r", 8, 0x00, 20, ID_SetOBEX);
 			break;
 		case ATOBEX_OBEX_SQWE:
 			/* Siemens extension */
-			error = GSM_WaitFor (s, "AT^SQWE=3\r", 10, 0x00, 4, ID_SetOBEX);
+			error = GSM_WaitFor (s, "AT^SQWE=3\r", 10, 0x00, 20, ID_SetOBEX);
 			break;
 		case ATOBEX_OBEX_None:
 			break;
@@ -136,7 +143,7 @@ GSM_Error ATOBEX_SetOBEXMode(GSM_StateMachine *s, OBEX_Service service)
 	if (error != ERR_NONE) return error;
 
 	/* Need some sleep before starting talk in OBEX */
-	my_sleep(100);
+	sleep(1);
 
 	/* Switch to OBEX protocol and initialise it */
 	s->Protocol.Functions = &OBEXProtocol;
@@ -224,6 +231,7 @@ GSM_Error ATOBEX_Terminate(GSM_StateMachine *s)
 	GSM_Error 		error;
 
 	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) return error;
 	OBEXGEN_FreeVars(s);
 	return ATGEN_Terminate(s);
 }
