@@ -105,7 +105,7 @@ static GSM_Error N7110_GetMemoryStatus(GSM_StateMachine *s, GSM_MemoryStatus *St
 	return GSM_WaitFor (s, req, 6, 0x03, 4, ID_GetMemoryStatus);
 }
 
-static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char *folderid, int *location)
+static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char *folderid, unsigned int *location)
 {
 	int ifolderid;
 
@@ -358,7 +358,7 @@ static GSM_Error N7110_PrivGetSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessag
 {
 	GSM_Error		error;
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	int			i;
         unsigned char 		req[] = {N6110_FRAME_HEADER, 0x07,
 					 0x08,			/* folder ID */
@@ -420,7 +420,7 @@ static GSM_Error N7110_GetSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 	int			i;
 	bool			found = false;
 
-	N7110_GetSMSLocation(s, &sms->SMS[0], &folderid, &location);
+	N7110_GetSMSLocation(s, &(sms->SMS[0]), &folderid, &location);
 	error=N7110_GetSMSFolderStatus(s, folderid);
 	if (error!=ERR_NONE) return error;
 	for (i=0;i<Priv->LastSMSFolder.Number;i++) {
@@ -448,7 +448,7 @@ static GSM_Error N7110_GetNextSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessag
 		error=N7110_GetSMSFolders(s,&Priv->LastSMSFolders);
 		if (error!=ERR_NONE) return error;
 	} else {
-		N7110_GetSMSLocation(s, &sms->SMS[0], &folderid, &location);
+		N7110_GetSMSLocation(s, &(sms->SMS[0]), &folderid, &location);
 		for (i=0;i<Priv->LastSMSFolder.Number;i++) {
 			if (Priv->LastSMSFolder.Location[i]==location) break;
 		}
@@ -684,7 +684,7 @@ static GSM_Error N7110_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone,
 			req[10] = size;
 			error = s->Protocol.Functions->WriteMessage(s, req, size+11, 0x00);
 			if (error!=ERR_NONE) return error;
-			my_sleep(1000);
+			sleep(1);
 			/* We have to make something (not important, what) now */
 			/* no answer from phone*/
 			return DCT3_GetNetworkInfo(s,&NetInfo);
@@ -702,7 +702,7 @@ static GSM_Error N7110_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone,
 	memcpy(req2+37,dest.NokiaBinary.Frame,dest.NokiaBinary.Length);
 	error = s->Protocol.Functions->WriteMessage(s, req2, 37+dest.NokiaBinary.Length, 0x1F);
 	if (error!=ERR_NONE) return error;
-	my_sleep(1000);
+	sleep(1);
 	/* We have to make something (not important, what) now */
 	/* no answer from phone*/
 	return DCT3_GetNetworkInfo(s,&NetInfo);
@@ -751,7 +751,8 @@ static GSM_Error N7110_ReplySaveSMSMessage(GSM_Protocol_Message msg, GSM_StateMa
 
 static GSM_Error N7110_PrivSetSMSMessage(GSM_StateMachine *s, GSM_SMSMessage *sms)
 {
-	int			length, location;
+	int			length;
+	unsigned int		location;
 	unsigned char		folderid, folder;
 	GSM_Error		error;
 	unsigned char 		req[256] = {N6110_FRAME_HEADER, 0x04,
@@ -822,7 +823,7 @@ static GSM_Error N7110_PrivSetSMSMessage(GSM_StateMachine *s, GSM_SMSMessage *sm
 
 static GSM_Error N7110_SetSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 {
-	int			location;
+	unsigned int		location;
 	unsigned char		folderid;
 
 	N7110_GetSMSLocation(s, sms, &folderid, &location);
@@ -832,7 +833,7 @@ static GSM_Error N7110_SetSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 
 static GSM_Error N7110_AddSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 {
-	int			location;
+	unsigned int		location;
 	unsigned char		folderid;
 
 	N7110_GetSMSLocation(s, sms, &folderid, &location);
@@ -1131,7 +1132,7 @@ static GSM_Error N7110_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 static GSM_Error N7110_DeleteSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 {
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	unsigned char 		req[] = {N7110_FRAME_HEADER, 0x0a,
 			       		 0x00, 		/* folder   */
 			       		 0x00, 0x00, 	/* location */
