@@ -27,14 +27,15 @@ GSM_Error N6510_ReplyGetCalendarInfo3(GSM_Protocol_Message msg, GSM_StateMachine
 		return ERR_MOREMEMORY;
 	}
 	if (j == 0) {
-		Last->Number=msg.Buffer[8]*256+msg.Buffer[9];
-		smprintf(s, "Number of Entries: %i\n",Last->Number);
+		Last->Number = msg.Buffer[8]*256 + msg.Buffer[9];
+		smprintf(s, "Number of Entries: %i\n", Last->Number);
 	}
 	smprintf(s, "Locations: ");
 	while (14+(i*4) <= msg.Length) {
-		Last->Location[j++]=msg.Buffer[12+i*4]*256+msg.Buffer[13+i*4];
-		smprintf(s, "%i ",Last->Location[j-1]);
+		Last->Location[j] = msg.Buffer[12+i*4]*256 + msg.Buffer[13+i*4];
+		smprintf(s, "%i ", Last->Location[j]);
 		i++;
+		j++;
 	}
 	smprintf(s, "\nNumber of Entries in frame: " SIZE_T_FORMAT "\n",i);
 	Last->Location[j] = 0;
@@ -100,11 +101,11 @@ GSM_Error N6510_GetCalendarInfo3(GSM_StateMachine *s, GSM_NOKIACalToDoLocations
 GSM_Error N6510_ReplyGetCalendar3(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
 	GSM_CalendarEntry 		*entry = s->Phone.Data.Cal;
-	GSM_DateTime			Date;
 	unsigned long			diff;
 	int				i;
 	bool				found = false;
 	GSM_Phone_N6510Data		*Priv = &s->Phone.Data.Priv.N6510;
+	int len;
 
 	smprintf(s, "Calendar note received method 3\n");
 
@@ -122,20 +123,24 @@ GSM_Error N6510_ReplyGetCalendar3(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		msg.Buffer[28]*256+msg.Buffer[29],
 		msg.Buffer[30],msg.Buffer[31],msg.Buffer[32],
 		msg.Buffer[33]);
-	Date.Year 	= msg.Buffer[28]*256+msg.Buffer[29];
+	GSM_GetCurrentDateTime(&entry->Entries[0].Date);
+	entry->Entries[0].Date.Year 	= msg.Buffer[28]*256+msg.Buffer[29];
 	if (entry->Type == GSM_CAL_BIRTHDAY) {
-		Date.Year = entry->Entries[0].Date.Year;
-		smprintf(s,"%i\n",Date.Year);
+		entry->Entries[0].Date.Year = entry->Entries[0].Date.Year;
+		smprintf(s,"%i\n",entry->Entries[0].Date.Year);
 	}
-	Date.Month 	= msg.Buffer[30];
-	Date.Day 	= msg.Buffer[31];
-	Date.Hour 	= msg.Buffer[32];
-	Date.Minute 	= msg.Buffer[33];
+	entry->Entries[0].Date.Month 	= msg.Buffer[30];
+	entry->Entries[0].Date.Day 	= msg.Buffer[31];
+	entry->Entries[0].Date.Hour 	= msg.Buffer[32];
+	entry->Entries[0].Date.Minute 	= msg.Buffer[33];
 	/* Garbage seen with 3510i 3.51 */
-	if (Date.Month == 0 && Date.Day == 0 && Date.Hour == 0 && Date.Minute == 0) return ERR_EMPTY;
-	Date.Second	= 0;
+	if (entry->Entries[0].Date.Month == 0 &&
+			entry->Entries[0].Date.Day == 0 &&
+			entry->Entries[0].Date.Hour == 0 &&
+			entry->Entries[0].Date.Minute == 0)
+		return ERR_EMPTY;
+	entry->Entries[0].Date.Second	= 0;
 	entry->Entries[0].EntryType = CAL_START_DATETIME;
-	memcpy(&entry->Entries[0].Date,&Date,sizeof(GSM_DateTime));
 	entry->EntriesNum++;
 
 	GSM_GetCalendarRecurranceRepeat(msg.Buffer+40, msg.Buffer+46, entry);
@@ -145,14 +150,13 @@ GSM_Error N6510_ReplyGetCalendar3(GSM_Protocol_Message msg, GSM_StateMachine *s)
 			msg.Buffer[34]*256+msg.Buffer[35],
 			msg.Buffer[36],msg.Buffer[37],msg.Buffer[38],
 			msg.Buffer[39]);
-		Date.Year 	= msg.Buffer[34]*256+msg.Buffer[35];
-		Date.Month 	= msg.Buffer[36];
-		Date.Day 	= msg.Buffer[37];
-		Date.Hour 	= msg.Buffer[38];
-		Date.Minute 	= msg.Buffer[39];
-		Date.Second	= 0;
+		entry->Entries[entry->EntriesNum].Date.Year 	= msg.Buffer[34]*256+msg.Buffer[35];
+		entry->Entries[entry->EntriesNum].Date.Month 	= msg.Buffer[36];
+		entry->Entries[entry->EntriesNum].Date.Day 	= msg.Buffer[37];
+		entry->Entries[entry->EntriesNum].Date.Hour 	= msg.Buffer[38];
+		entry->Entries[entry->EntriesNum].Date.Minute 	= msg.Buffer[39];
+		entry->Entries[entry->EntriesNum].Date.Second	= 0;
 		entry->Entries[entry->EntriesNum].EntryType = CAL_END_DATETIME;
-		memcpy(&entry->Entries[entry->EntriesNum].Date,&Date,sizeof(GSM_DateTime));
 		entry->EntriesNum++;
 	}
 
@@ -200,10 +204,17 @@ GSM_Error N6510_ReplyGetCalendar3(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		}
 	}
 
-	memcpy(entry->Entries[entry->EntriesNum].Text, msg.Buffer+54, msg.Buffer[51]*2);
-	entry->Entries[entry->EntriesNum].Text[msg.Buffer[51]*2]   = 0;
-	entry->Entries[entry->EntriesNum].Text[msg.Buffer[51]*2+1] = 0;
-	entry->Entries[entry->EntriesNum].EntryType		   = CAL_TEXT;
+	len = msg.Buffer[50] * 256 + msg.Buffer[51];
+	if (len > GSM_MAX_CALENDAR_TEXT_LENGTH) {
+		smprintf(s, "Calendar text too long (%d), truncating to %d\n", len, GSM_MAX_CALENDAR_TEXT_LENGTH);
+		len = GSM_MAX_CALENDAR_TEXT_LENGTH;
+	}
+	memcpy(entry->Entries[entry->EntriesNum].Text,
+		msg.Buffer + 54,
+		len * 2);
+	entry->Entries[entry->EntriesNum].Text[len * 2]  = 0;
+	entry->Entries[entry->EntriesNum].Text[len * 2 + 1] = 0;
+	entry->Entries[entry->EntriesNum].EntryType = CAL_TEXT;
 	entry->EntriesNum++;
 	smprintf(s, "Note text: \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum-1].Text));
 
@@ -462,6 +473,7 @@ static GSM_Error N6510_AddCalendar3(GSM_StateMachine *s, GSM_CalendarEntry *Note
 
 	error=N6510_GetCalendarNotePos3(s);
 	if (error!=ERR_NONE) return error;
+	Note->Location = *FirstCalendarPos;
 	req[8] = *FirstCalendarPos/256;
 	req[9] = *FirstCalendarPos%256;
 
@@ -553,9 +565,10 @@ static GSM_Error N6510_AddCalendar3(GSM_StateMachine *s, GSM_CalendarEntry *Note
 	}
 
 	if (Text != -1) {
-		req[49] = UnicodeLength(Note->Entries[Text].Text);
-		CopyUnicodeString(req+54,Note->Entries[Text].Text);
-		count+= req[49]*2;
+		req[48] = UnicodeLength(Note->Entries[Text].Text) / 256;
+		req[49] = UnicodeLength(Note->Entries[Text].Text) % 256;
+		CopyUnicodeString(req + 54, Note->Entries[Text].Text);
+		count += req[49] * 2 + (req[48] * 256) * 2;
 	}
 
 	if (Phone != -1 && NoteType == GSM_CAL_CALL) {
@@ -657,10 +670,19 @@ GSM_Error N6510_ReplyGetNoteInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 GSM_Error N6510_ReplyGetNote(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
+	int len;
+
 	smprintf(s, "Note received\n");
-	memcpy(s->Phone.Data.Note->Text,msg.Buffer+54,(msg.Buffer[50]*256+msg.Buffer[51])*2);
-	s->Phone.Data.Note->Text[(msg.Buffer[50]*256+msg.Buffer[51])*2] = 0;
-	s->Phone.Data.Note->Text[(msg.Buffer[50]*256+msg.Buffer[51])*2+1] = 0;
+	len = msg.Buffer[50] * 256 + msg.Buffer[51];
+	if (len > GSM_MAX_NOTE_TEXT_LENGTH) {
+		smprintf(s, "Note too long (%d), truncating to %d\n", len, GSM_MAX_NOTE_TEXT_LENGTH);
+		len = GSM_MAX_NOTE_TEXT_LENGTH;
+	}
+	memcpy(s->Phone.Data.Note->Text,
+		msg.Buffer + 54,
+		len * 2);
+	s->Phone.Data.Note->Text[len * 2] = 0;
+	s->Phone.Data.Note->Text[(len * 2) + 1] = 0;
 	return ERR_NONE;
 }
 
@@ -714,7 +736,7 @@ GSM_Error N6510_ReplyGetNoteFirstLoc(GSM_Protocol_Message msg, GSM_StateMachine
 
 GSM_Error N6510_ReplyAddNote(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
-	smprintf(s, "ToDo added\n");
+	smprintf(s, "Note added\n");
 	return ERR_NONE;
 }
 
@@ -724,7 +746,7 @@ GSM_Error N6510_AddNote(GSM_StateMachine *s, GSM_NoteEntry *Not)
 	int 			count=54;
 	unsigned char 		reqLoc[] = {N6110_FRAME_HEADER, 0x95,
 					    0x02};	/* 1 = todo, 2 = note */
-	unsigned char 		req[5000] = {
+	unsigned char 		req[GSM_MAX_NOTE_TEXT_LENGTH + 500] = {
 		N6110_FRAME_HEADER, 0x65,
 		0x02,					/* 0 = calendar, 1 = todo, 2 = note 	*/
 		0x00, 0x00, 0x00,
@@ -746,18 +768,21 @@ GSM_Error N6510_AddNote(GSM_StateMachine *s, GSM_NoteEntry *Not)
 		0x00,					/* note text length 			*/
 		0x00,					/* phone length/meeting place		*/
 		0x00, 0x00, 0x00};
+	size_t length;
 
 	s->Phone.Data.Note = Not;
 
 	smprintf(s, "Getting first free Note location\n");
 	error = GSM_WaitFor (s, reqLoc, 5, 0x13, 4, ID_SetNote);
 	if (error!=ERR_NONE) return error;
-	req[8] = Not->Location/256;
-	req[9] = Not->Location%256;
+	req[8] = Not->Location / 256;
+	req[9] = Not->Location % 256;
 
-	req[49] = UnicodeLength(Not->Text);
-	CopyUnicodeString(req+54,Not->Text);
-	count+= req[49]*2;
+	length = UnicodeLength(Not->Text);
+	req[48] = length / 256;
+	req[49] = length % 256;
+	CopyUnicodeString(req + 54, Not->Text);
+	count += length * 2;
 
 	req[count++] = 0x00;
 
@@ -912,6 +937,7 @@ GSM_Error N6510_ReplyGetToDo2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	GSM_ToDoEntry 		*Last = s->Phone.Data.ToDo;
 	GSM_DateTime		Date;
 	unsigned long		diff;
+	int len;
 
 	smprintf(s, "ToDo received method 2\n");
 
@@ -927,9 +953,16 @@ GSM_Error N6510_ReplyGetToDo2(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		default	 : return ERR_UNKNOWN;
 	}
 
-	memcpy(Last->Entries[0].Text,msg.Buffer+54,msg.Buffer[51]*2);
-	Last->Entries[0].Text[msg.Buffer[51]*2]   = 0;
-	Last->Entries[0].Text[msg.Buffer[51]*2+1] = 0;
+	len = msg.Buffer[50] * 256 + msg.Buffer[51];
+	if (len > GSM_MAX_TODO_TEXT_LENGTH) {
+		smprintf(s, "Todo text too long (%d), truncating to %d\n", len, GSM_MAX_TODO_TEXT_LENGTH);
+		len = GSM_MAX_TODO_TEXT_LENGTH;
+	}
+	memcpy(Last->Entries[0].Text,
+		msg.Buffer + 54,
+		len * 2);
+	Last->Entries[0].Text[len * 2] = 0;
+	Last->Entries[0].Text[(len * 2) + 1] = 0;
     	Last->Entries[0].EntryType = TODO_TEXT;
 	smprintf(s, "Text: \"%s\"\n",DecodeUnicodeString(Last->Entries[0].Text));
 
@@ -1144,7 +1177,7 @@ static GSM_Error N6510_AddToDo2(GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
  	int 			Text, Alarm, EndTime, Completed, count=54, Phone;
 	unsigned char 		reqLoc[] = {N6110_FRAME_HEADER, 0x95,
 					    0x01};	/* 1 = todo, 2 = note */
-	unsigned char 		req[5000] = {
+	unsigned char 		req[GSM_MAX_TODO_TEXT_LENGTH + 500] = {
 		N6110_FRAME_HEADER, 0x65,
 		0x01,					/* 0 = calendar, 1 = todo 		*/
 		0x00, 0x00, 0x00,
@@ -1237,9 +1270,10 @@ static GSM_Error N6510_AddToDo2(GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 	}
 
 	if (Text != -1) {
-		req[49] = UnicodeLength(ToDo->Entries[Text].Text);
-		CopyUnicodeString(req+54,ToDo->Entries[Text].Text);
-		count+= req[49]*2;
+		req[48] = UnicodeLength(ToDo->Entries[Text].Text) / 256;
+		req[49] = UnicodeLength(ToDo->Entries[Text].Text) % 256;
+		CopyUnicodeString(req + 54, ToDo->Entries[Text].Text);
+		count += req[49] * 2 + (req[48] * 256) * 2;
 	}
 
 	req[count++] = 0x00;
