@@ -27,7 +27,7 @@ static int N6510_FindFileCheckSum12(unsigned char *ptr, int len)
 	acc  = 0xffff;
 	while (len--) {
 		accx = (accx & 0xffff00ff) | (acc & 0xff00);
-		acc  = (acc  & 0xffff00ff) | *ptr++ << 8;
+		acc  = (acc  & 0xffff00ff) | (*ptr++ << 8);
 		for (i = 0; i < 8; i++) {
 			acc <<= 1;
 			if (acc & 0x10000)     acc ^= 0x1021;
@@ -179,6 +179,12 @@ GSM_Error N6510_ReplyGetFileFolderInfo1(GSM_Protocol_Message msg, GSM_StateMachi
 				memcpy(&Priv->Files[i+(msg.Buffer[8]*256+msg.Buffer[9])],&Priv->Files[i],sizeof(GSM_File));
 				i--;
 			}
+			if (Priv->FilesLocationsUsed + (msg.Buffer[8]*256+msg.Buffer[9]) >= GSM_PHONE_MAXSMSINFOLDER) {
+				smprintf(s,"Too small buffer for folder data\n");
+				Priv->filesystem2error  = ERR_MOREMEMORY;
+				Priv->FilesEnd 		= true;
+				return ERR_MOREMEMORY;
+			}
 			Priv->FilesLocationsUsed += (msg.Buffer[8]*256+msg.Buffer[9]);
 			for (i=0;i<(msg.Buffer[8]*256+msg.Buffer[9]);i++) {
 				sprintf(buffer,"%i",msg.Buffer[13+i*4-1]*256 + msg.Buffer[13+i*4]);
@@ -316,6 +322,7 @@ static GSM_Error N6510_GetFileSystemStatus1(GSM_StateMachine *s, GSM_FileSystemS
 	req[4] = 0x01;
 	smprintf(s, "Getting used/total memory in filesystem\n");
 	error = GSM_WaitFor (s, req, 10, 0x6D, 4, ID_FileSystemStatus);
+	if (error != ERR_NONE) return error;
 
 	req[3] = 0x22;
 	req[4] = 0x01;
@@ -900,7 +907,7 @@ GSM_Error N6510_ReplyGetFileFolderInfo2(GSM_Protocol_Message msg, GSM_StateMachi
 			smprintf(s,"File or folder details received\n");
 
 			if (msg.Buffer[3] == 0x69) {
-				if (Priv->FilesLocationsUsed==1000) {
+				if (Priv->FilesLocationsUsed + 1 >= GSM_PHONE_MAXSMSINFOLDER) {
 					smprintf(s,"Too small buffer for folder data\n");
 					Priv->filesystem2error  = ERR_MOREMEMORY;
 					Priv->FilesEnd 		= true;
@@ -1035,7 +1042,7 @@ static GSM_Error N6510_PrivGetFolderListing2(GSM_StateMachine *s, GSM_File *File
 	if (error!=ERR_NONE) return error;
 
 	while (!Priv->FilesEnd) {
-		my_sleep(100);
+		usleep(100000);
 		if (GSM_ReadDevice(s,true)==0) i++; else i=0;
 		if (i==3) {
 			smprintf(s,"Connection broken or WELL KNOWN phone firmware problem (which makes, that not all files are reported)\n");
@@ -2001,7 +2008,7 @@ GSM_Error N6510_GetMMSFolders(GSM_StateMachine *s, GSM_MMSFolders *folders)
 		Start 		= true;
 		while (1) {
 			error = N6510_GetFolderListing(s,&Files,Start);
-			if (error == ERR_EMPTY) return ERR_NONE;
+			if (error == ERR_EMPTY) break;
 			if (error != ERR_NONE) return error;
 			Start = false;
 			if (!Files.Folder) continue;
@@ -2434,10 +2441,10 @@ GSM_Error N6510_DecodeFilesystemSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sm
 		i+=FFF->Buffer[i+1];
 
 		CopyUnicodeString(sms->SMS[0].Number,FFF->Buffer+94);
-		Layout.TPDCS=i+2;
-		smprintf(s,"TPDCS %02x\n",FFF->Buffer[i+2]);
-		smprintf(s,"TPUDL %02x\n",FFF->Buffer[i+11]);
+		Layout.TPDCS=i+3;
 		Layout.TPUDL = i+11;
+		smprintf(s,"TPDCS %02x\n",FFF->Buffer[Layout.TPDCS]);
+		smprintf(s,"TPUDL %02x\n",FFF->Buffer[Layout.TPUDL]);
 		Layout.Text = i+12;
 		sms->SMS[0].Coding = GSM_GetMessageCoding(FFF->Buffer[Layout.TPDCS]);
 		GSM_DecodeSMSFrameText(&sms->SMS[0], FFF->Buffer, Layout);
