@@ -182,7 +182,12 @@ size_t N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_
 		}
 		dbgprintf("entry num %i %i\n",i,entry->EntriesNum);
 		if (entry->Entries[i].EntryType == PBK_Text_Note)   type = N7110_PBK_NOTE;
-		if (entry->Entries[i].EntryType == PBK_Text_Postal) type = N7110_PBK_POSTAL;
+		if (entry->Entries[i].EntryType == PBK_Text_Postal) {
+			if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKNOPOSTAL)) {
+				continue;
+			}
+			type = N7110_PBK_POSTAL;
+		}
 		if (entry->Entries[i].EntryType == PBK_Text_Email)  type = N7110_PBK_EMAIL;
 		if (entry->Entries[i].EntryType == PBK_Text_Email2) type = N7110_PBK_EMAIL;
 		if (entry->Entries[i].EntryType == PBK_Text_URL) {
@@ -287,6 +292,29 @@ size_t N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_
 			req[count-1]--;
 			continue;
 		}
+		if (entry->Entries[i].EntryType == PBK_Number_Messaging) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKFAVORITEMESSAGE)) {
+				entry->Entries[i].AddError = ERR_INVALIDDATA;
+				/* The favorite messaging number is stored as a phone number,
+				 * the phone wants an id to a previously supplied entry, so we search for that.
+				 * In case there was an error in the previous entries, we stop.
+				 * Otherwise we would point past the supplied entries. */
+				for (j = 0; j < i && entry->Entries[j].AddError == ERR_NONE; j++) {
+					if (mywstrncmp(entry->Entries[i].Text, entry->Entries[j].Text, -1)) {
+						string[0] = j + 1;
+						string[1] = 0x00;
+						string[2] = 0x00;
+						count += N71_65_PackPBKBlock(s, N2630_PBK_FAVMESSAGING, 3, block++, string, req + count);
+						count --;
+						req[count-5] = 8;
+						entry->Entries[i].AddError = ERR_NONE;
+						break;
+					}
+
+				}
+			}
+			continue;
+		}
 	}
 
 	*block2=block;
@@ -482,6 +510,10 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			if (Block[5] == 0x55) {
 				Type = PBK_Number_General;    smprintf(s,"General number ");
 			}
+			/* Yet another unknown General number */
+			if (Block[5] == 0x08) {
+				Type = PBK_Number_General;    smprintf(s,"General number ");
+			}
 			if (Block[5] == N7110_PBK_NUMBER_GENERAL) {
 				Type = PBK_Number_General;    smprintf(s,"General number ");
 			}
@@ -864,6 +896,21 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
+		if (Block[0] == N2630_PBK_FAVMESSAGING) {
+			/* The phone sends an entry id. We store it as phone number because
+			 * entry->Entries doesn't retain order. */
+			int num = (int)Block[5];
+			entry->Entries[entry->EntriesNum].EntryType=PBK_Number_Messaging;
+			if (num >= entry->EntriesNum) {
+				smprintf(s, "ERROR: Number_Messaging references future entry\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			CopyUnicodeString(entry->Entries[entry->EntriesNum].Text,entry->Entries[num-1].Text);
+			smprintf(s,"Marked entry #%i (%s) as favorite messaging number\n",
+				num,DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
 		smprintf(s, "ERROR: unknown pbk entry 0x%02x\n",Block[0]);
 		return ERR_UNKNOWNRESPONSE;
 	}
@@ -936,7 +983,7 @@ GSM_Error NOKIA_ReplyGetPhoneString(GSM_Protocol_Message msg, GSM_StateMachine *
  * type of message (type), pointer for buffer for response (*value), request
  * type (request) and what is start byte in response for our string
  */
-GSM_Error NOKIA_GetPhoneString(GSM_StateMachine *s, unsigned char *msgframe, int msglen, unsigned char msgtype, char *retvalue, GSM_Phone_RequestID request, int startresponse)
+GSM_Error NOKIA_GetPhoneString(GSM_StateMachine *s, const unsigned char *msgframe, int msglen, unsigned char msgtype, char *retvalue, GSM_Phone_RequestID request, int startresponse)
 {
 	retvalue[0] = 0;
 	s->Phone.Data.StartPhoneString = startresponse;
@@ -1107,7 +1154,6 @@ GSM_Error DCT3DCT4_ReplyCallDivert(GSM_Protocol_Message msg, GSM_StateMachine *s
 				/* 6310i */
 				cd->Response.EntriesNum = 0;
 				return ERR_NONE;
-				break;
 	    		}
 		    	smprintf(s,"\n");
 			j = pos + 2;
@@ -1282,7 +1328,6 @@ GSM_Error DCT3DCT4_ReplyGetWAPBookmark(GSM_Protocol_Message msg, GSM_StateMachin
 			smprintf(s, "ERROR: unknown %i\n",msg.Buffer[4]);
 			return ERR_UNKNOWNRESPONSE;
 		}
-		break;
 	}
 	return ERR_UNKNOWNRESPONSE;
 }
@@ -1501,6 +1546,9 @@ GSM_Error DCT3DCT4_GetFirmware (GSM_StateMachine *s)
 GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 {
 	switch (error) {
+	case 0x21:
+		smprintf(s, "Wait for synchronisation???\n");
+		return ERR_WORKINPROGRESS;
 	case 0x27:
 		smprintf(s, "No PIN\n");
 		return ERR_SECURITYERROR;
@@ -1532,7 +1580,7 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 		 */
 		return ERR_NONE;
 	default:
-		smprintf(s, "ERROR: unknown %i\n",error);
+		smprintf(s, "ERROR: unknown status code 0x%x\n", error);
 		return ERR_UNKNOWNRESPONSE;
 	}
 }
@@ -1543,12 +1591,21 @@ GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine
 	case 0x0f:
 		smprintf(s, "Phonebook entry writing failed\n");
 		switch (msg.Buffer[10]) {
+		case 0xf:
+			smprintf(s, "Invalid block sent\n");
+			return ERR_BUG;
+		case 0x23:
+			smprintf(s, "Block size does not match a definition\n");
+			return ERR_BUG;
 		case 0x25:
 			smprintf(s, "when you try to save into entry with caller group assigment in phone with caller groups standard 2 (like in 6230i)\n");
 			return ERR_PERMISSION;
 		case 0x29:
 			smprintf(s, "no caller group with given number (6230i)\n");
 			return ERR_MEMORY;
+		case 0x32:
+			smprintf(s, "Ignoring ERROR: unknown 50 (probably group contains 50 entries)\n");
+			return ERR_NONE;
 		case 0x36:
 			smprintf(s, "Too long name\n");
 			return ERR_NOTSUPPORTED;
@@ -1561,9 +1618,9 @@ GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine
 		case 0x3e:
 			smprintf(s, "Too much entries\n");
 			return ERR_NOTSUPPORTED;
-		case 50:
-			smprintf(s, "Ignoring ERROR: unknown 50 (probably group contains 50 entries)\n");
-			return ERR_NONE;
+		case 0x43:
+			smprintf(s, "Incorrect characters\n");
+			return ERR_NOTSUPPORTED;
 		default:
 			smprintf(s, "ERROR: unknown %i\n",msg.Buffer[10]);
 			return ERR_UNKNOWNRESPONSE;
@@ -1952,7 +2009,10 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
 	GSM_CalendarFindDefaultTextTimeAlarmPhone(Note, &Text, &Time, &Alarm, &Phone, &EndTime, &Location);
 
-	if (Time == -1) return ERR_UNKNOWN;
+	if (Time == -1) {
+		smprintf(s, "Can not save entry without time!\n");
+		return ERR_UNKNOWN;
+	}
 	if (NoteType != GSM_CAL_BIRTHDAY) {
 		Date.Year 	= 2030;	Date.Month 	= 01; Date.Day    = 01;
 		Date.Hour 	= 00;	Date.Minute 	= 00; Date.Second = 00;
@@ -2113,6 +2173,7 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 	error=N71_65_GetCalendarNotePos1(s);
 	if (error!=ERR_NONE) return error;
 	if (FirstCalendarPos != NULL) {
+		Note->Location = *FirstCalendarPos;
 		req[4] = *FirstCalendarPos/256;
 		req[5] = *FirstCalendarPos%256;
 	}
@@ -2127,7 +2188,10 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 
 	GSM_CalendarFindDefaultTextTimeAlarmPhone(Note, &Text, &Time, &Alarm, &Phone, &EndTime, &Location);
 
-	if (Time == -1) return ERR_UNKNOWN;
+	if (Time == -1) {
+		smprintf(s, "Can not save entry without time!\n");
+		return ERR_UNKNOWN;
+	}
 	memcpy(&DT,&Note->Entries[Time].Date,sizeof(GSM_DateTime));
 	req[8]	= DT.Year / 256;
 	req[9]	= DT.Year % 256;
@@ -2246,6 +2310,8 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 
 GSM_Error N71_65_ReplyDelCalendar(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
+	if (msg.Buffer[3] == 0xf0) return ERR_NOTSUPPORTED;
+
 	smprintf(s, "Deleted calendar note on location %d\n",msg.Buffer[4]*256+msg.Buffer[5]);
 	return ERR_NONE;
 }
@@ -2271,7 +2337,7 @@ GSM_Error N71_65_ReplyGetCalendarInfo1(GSM_Protocol_Message msg, GSM_StateMachin
 	while (LastCalendar->Location[j] != 0x00) j++;
 	if (j >= GSM_MAXCALENDARTODONOTES) {
 		smprintf(s, "Increase GSM_MAXCALENDARNOTES\n");
-		return ERR_UNKNOWN;
+		return ERR_MOREMEMORY;
 	}
 	if (j == 0) {
 		LastCalendar->Number=msg.Buffer[4]*256+msg.Buffer[5];
@@ -2514,7 +2580,7 @@ GSM_Error N71_65_GetNextCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note,
 	return GSM_WaitFor (s, req, 6, 0x13, 4, ID_GetCalendarNote);
 }
 
-GSM_Error N71_65_EnableFunctions(GSM_StateMachine *s,char *buff,int len)
+GSM_Error N71_65_EnableFunctions(GSM_StateMachine *s,const char *buff,int len)
 {
 	unsigned char buffer[50] = {N6110_FRAME_HEADER, 0x10,
 				    0x07};	/* Length */
@@ -2540,7 +2606,7 @@ GSM_Error N71_65_ReplySendDTMF(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	return ERR_UNKNOWNRESPONSE;
 }
 
-GSM_CalendarNoteType N71_65_FindCalendarType(GSM_CalendarNoteType Type, OnePhoneModel *model)
+GSM_CalendarNoteType N71_65_FindCalendarType(GSM_CalendarNoteType Type, GSM_PhoneModel *model)
 {
 	switch (Type) {
 	case GSM_CAL_CALL:
