@@ -70,14 +70,15 @@ static GSM_Error GSM_DecodeSMSDateTime(GSM_DateTime *DT, unsigned char *req)
 	DT->Second  = DecodeWithBCDAlphabet(req[5]);
 
 	/* Base for timezone is GMT. It's in quarters */
-	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))/4;
+	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))*3600/4;
 
 	if (req[6]&0x08) DT->Timezone = -DT->Timezone;
 
 	dbgprintf("Decoding date & time: ");
 	dbgprintf("%s %4d/%02d/%02d ", DayOfWeek(DT->Year, DT->Month, DT->Day),
 		  DT->Year, DT->Month, DT->Day);
-	dbgprintf("%02d:%02d:%02d %02d00\n", DT->Hour, DT->Minute, DT->Second, DT->Timezone);
+	dbgprintf("%02d:%02d:%02d%+03i%02i\n", DT->Hour, DT->Minute, DT->Second,
+		DT->Timezone / 3600, abs((DT->Timezone % 3600) / 60));
 
 	return ERR_NONE;
 }
@@ -413,15 +414,13 @@ static GSM_Error GNAPGEN_GetNextSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sm
 
 static GSM_Error GNAPGEN_EncodeSMSFrame(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char *req, GSM_SMSMessageLayout *Layout, int *length)
 {
-	int			start, count = 0;
+	int			count = 0;
 	GSM_Error		error;
 
 	memset(Layout,255,sizeof(GSM_SMSMessageLayout));
 
 	sms->Class = -1;
 
-	start = *length;
-
 	*Layout = PHONE_SMSSubmit;
 
 	/*  smsc number is semi-octet */
@@ -536,6 +535,7 @@ static GSM_Error GNAPGEN_PrivSetSMSMessage(GSM_StateMachine *s, GSM_SMSMessage *
 	memset(req+8,0x00,300-8);
 
 	error = GNAPGEN_EncodeSMSFrame( s, sms, req + 10, &Layout, &length );
+	if (error != ERR_NONE) return error;
 	req[9] = length;
 
 	s->Phone.Data.SaveSMSMessage=sms;
@@ -1064,11 +1064,9 @@ static GSM_Error GNAPGEN_ReplyDeleteSMSMessage ( GSM_Protocol_Message msg, GSM_S
 		case 16:
 			smprintf(s, "invalid location\n");
 			return ERR_UNKNOWN;
-			break;
 		case 17:
 			smprintf(s, "unknown memory type\n");
 			return ERR_UNKNOWN;
-			break;
 		case 0:
 			smprintf(s, "deleted");
 	}
