@@ -30,16 +30,16 @@ static GSM_Error AT_WriteMessage (GSM_StateMachine *s, unsigned const char *buff
 			/* For some phones like Siemens M20 we need to wait a little
 			 * after writing each char. Possible reason: these phones
 			 * can't receive so fast chars or there is bug here in Gammu */
-			my_sleep(1);
+			usleep(1000);
 		}
-		my_sleep(400);
+		usleep(400000);
 	}
 
 	return ERR_NONE;
 }
 
 typedef struct {
-	char	*text;
+	const char	*text;
 	int	lines;
 } SpecialAnswersStruct;
 
@@ -50,11 +50,12 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 	size_t			i;
 
 	/* These are lines with end of "normal" answers */
-	static char 		*StartStrings[] = {
+	static const char 		*StartStrings[] = {
 		"OK"		, 	"ERROR"	 	,
 		"+CME ERROR:"	,	"+CMS ERROR:"	,
 
 		"+CPIN: "	,	/*A2D issue*/
+		"COMMAND NOT SUPPORT" /* HUAWEI */	,
 
 		NULL};
 
@@ -71,6 +72,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 		{"RING"		,1},	{"NO CARRIER"	,1},
 		{"NO ANSWER"	,1},	{"+COLP"	,1},
 		{"+CLIP"	,1},	{"+CRING"	,1},
+		{"+CCWA"	,1},
 
 		{"*EBCA"	,1},
 
@@ -126,6 +128,11 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 						i++;
 						continue;
 					}
+				  	if (s->Phone.Data.RequestID == ID_SetOBEX &&
+							strcmp(SpecialAnswers[i].text, "NO CARRIER") == 0) {
+						i++;
+						continue;
+					}
 					d->SpecialAnswerStart 	= d->LineStart;
 					d->SpecialAnswerLines	= SpecialAnswers[i].lines;
 				}
@@ -203,6 +210,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 static GSM_Error AT_Initialise(GSM_StateMachine *s)
 {
 	GSM_Protocol_ATData *d = &s->Protocol.Data.AT;
+	GSM_Error		error;
 
 	d->Msg.Buffer 		= NULL;
 	d->Msg.BufferUsed	= 0;
@@ -216,7 +224,11 @@ static GSM_Error AT_Initialise(GSM_StateMachine *s)
 	d->EditMode		= false;
 	d->FastWrite		= false;
 
-	s->Device.Functions->DeviceSetDtrRts(s,true,true);
+	error = s->Device.Functions->DeviceSetParity(s, false);
+	if (error != ERR_NONE) return error;
+
+	error = s->Device.Functions->DeviceSetDtrRts(s,true,true);
+	if (error != ERR_NONE) return error;
 
 	return s->Device.Functions->DeviceSetSpeed(s,s->Speed);
 }
