@@ -25,6 +25,7 @@ GSM_Error SaveICS(char *FileName, GSM_Backup *backup)
 	size_t Length = 0;
 	unsigned char 	Buffer[1000];
 	FILE 		*file;
+	GSM_Error error;
 
 	file = fopen(FileName, "wb");
 	if (file == NULL) return ERR_CANTOPENFILE;
@@ -38,7 +39,8 @@ GSM_Error SaveICS(char *FileName, GSM_Backup *backup)
 		sprintf(Buffer, "%c%c",13,10);
 		chk_fwrite(Buffer,1,2,file);
 		Length = 0;
-		GSM_EncodeVCALENDAR(Buffer,&Length,backup->Calendar[i],false,Mozilla_iCalendar);
+		error = GSM_EncodeVCALENDAR(Buffer, sizeof(Buffer),&Length,backup->Calendar[i],false,Mozilla_iCalendar);
+		if (error != ERR_NONE) return error;
 		chk_fwrite(Buffer,1,Length,file);
 		i++;
 	}
@@ -47,7 +49,8 @@ GSM_Error SaveICS(char *FileName, GSM_Backup *backup)
 		sprintf(Buffer, "%c%c",13,10);
 		chk_fwrite(Buffer,1,2,file);
 		Length = 0;
-		GSM_EncodeVTODO(Buffer,&Length,backup->ToDo[i],false,Mozilla_VToDo);
+		error = GSM_EncodeVTODO(Buffer, sizeof(Buffer), &Length,backup->ToDo[i],false,Mozilla_VToDo);
+		if (error != ERR_NONE) return error;
 		chk_fwrite(Buffer,1,Length,file);
 		i++;
 	}
@@ -68,13 +71,13 @@ GSM_Error LoadICS(char *FileName, GSM_Backup *backup)
 	GSM_Error		error;
 	GSM_CalendarEntry	Calendar;
 	GSM_ToDoEntry		ToDo;
-	int			numCal = 0, numToDo = 0, Pos;
+	int			numCal = 0, numToDo = 0;
+	size_t Pos = 0;
 
 	File.Buffer = NULL;
 	error = GSM_ReadFile(FileName, &File);
 	if (error != ERR_NONE) return error;
 
-	Pos = 0;
 	while (1) {
 		error = GSM_DecodeVCALENDAR_VTODO(File.Buffer, &Pos, &Calendar, &ToDo, Mozilla_iCalendar, Mozilla_VToDo);
 		if (error == ERR_EMPTY) break;
