@@ -13,7 +13,7 @@
 
 #ifdef GSM_ENABLE_BACKUP
 
-static void SaveLDIFText(FILE *file, unsigned char *Name, unsigned char *Value)
+static void SaveLDIFText(FILE *file, const char *Name, const unsigned char *Value)
 {
 	unsigned char Buffer[1000],Buffer2[1000];
 
@@ -25,7 +25,7 @@ static void SaveLDIFText(FILE *file, unsigned char *Name, unsigned char *Value)
 	}
 }
 
-GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
+GSM_Error SaveLDIF(const char *FileName, GSM_Backup *backup)
 {
 	int 		i, j;
 	FILE 		*file;
@@ -77,6 +77,9 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 			case PBK_Number_Pager:
 				SaveLDIFText(file, "pager", backup->PhonePhonebook[i]->Entries[j].Text);
 				break;
+			case PBK_Number_Messaging:
+				SaveLDIFText(file, "messaging", backup->PhonePhonebook[i]->Entries[j].Text);
+				break;
 			case PBK_Number_Fax:
 				/* facsimileTelephoneNumber */
 				SaveLDIFText(file, "fax", backup->PhonePhonebook[i]->Entries[j].Text);
@@ -169,6 +172,7 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 			case PBK_CallLength:
 			case PBK_Text_PictureName:
 			case PBK_PushToTalkID:
+			case PBK_Photo:
 				dbgprintf("Feature missed\n");
 				break;
 			}
@@ -180,7 +184,7 @@ GSM_Error SaveLDIF(char *FileName, GSM_Backup *backup)
 	return ERR_NONE;
 }
 
-static bool ReadLDIFText(char *Buffer, char *Start, char *Value)
+static bool ReadLDIFText(const char *Buffer, const char *Start, char *Value)
 {
 	unsigned char 	Buffer2[1000],buff[200];
 	int 		i;
@@ -206,16 +210,18 @@ static bool ReadLDIFText(char *Buffer, char *Start, char *Value)
 	return false;
 }
 
-static GSM_Error GSM_DecodeLDIFEntry(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk)
+static GSM_Error GSM_DecodeLDIFEntry(char *Buffer, size_t *Pos, GSM_MemoryEntry *Pbk)
 {
 	unsigned char 	Line[2000],Buff[2000],Buff2[2000];
 	int		Level = 0;
+	GSM_Error	error;
 
 	Buff[0] 	= 0;
 	Pbk->EntriesNum = 0;
 
 	while (1) {
-		MyGetLine(Buffer, Pos, Line, strlen(Buffer), false);
+		error = MyGetLine(Buffer, Pos, Line, strlen(Buffer), sizeof(Line), false);
+		if (error != ERR_NONE) return error;
 		if (strlen(Line) == 0) break;
 		switch (Level) {
 		case 0:
@@ -248,6 +254,16 @@ static GSM_Error GSM_DecodeLDIFEntry(unsigned char *Buffer, int *Pos, GSM_Memory
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile;
 				Pbk->EntriesNum++;
 			}
+			if (ReadLDIFText(Line, "pager", Buff)) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Pager;
+				Pbk->EntriesNum++;
+			}
+			if (ReadLDIFText(Line, "messaging", Buff)) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Messaging;
+				Pbk->EntriesNum++;
+			}
 			if (ReadLDIFText(Line, "workPhone", Buff)) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Work;
@@ -303,18 +319,18 @@ static GSM_Error GSM_DecodeLDIFEntry(unsigned char *Buffer, int *Pos, GSM_Memory
 	return ERR_NONE;
 }
 
-GSM_Error LoadLDIF(char *FileName, GSM_Backup *backup)
+GSM_Error LoadLDIF(const char *FileName, GSM_Backup *backup)
 {
 	GSM_File 		File;
 	GSM_Error		error;
 	GSM_MemoryEntry	Pbk;
-	int			numPbk = 0, Pos;
+	int			numPbk = 0;
+	size_t Pos = 0;
 
 	File.Buffer = NULL;
 	error = GSM_ReadFile(FileName, &File);
 	if (error != ERR_NONE) return error;
 
-	Pos = 0;
 	while (1) {
 		error = GSM_DecodeLDIFEntry(File.Buffer, &Pos, &Pbk);
 		if (error == ERR_EMPTY) break;
