@@ -25,17 +25,19 @@ GSM_Error SaveVCard(char *FileName, GSM_Backup *backup)
 	size_t Length = 0;
 	unsigned char 	Buffer[1000];
 	FILE 		*file;
+	GSM_Error	error;
 
 	file = fopen(FileName, "wb");
 	if (file == NULL) return ERR_CANTOPENFILE;
 
 	i=0;
 	while (backup->PhonePhonebook[i]!=NULL) {
+		Length = 0;
+		error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,backup->PhonePhonebook[i],true,Nokia_VCard21);
+		if (error != ERR_NONE) return error;
+		chk_fwrite(Buffer,1,Length,file);
 		sprintf(Buffer, "%c%c",13,10);
 		chk_fwrite(Buffer,1,2,file);
-		Length = 0;
-		GSM_EncodeVCARD(Buffer,&Length,backup->PhonePhonebook[i],true,Nokia_VCard21);
-		chk_fwrite(Buffer,1,Length,file);
 		i++;
 	}
 
@@ -51,13 +53,13 @@ GSM_Error LoadVCard(char *FileName, GSM_Backup *backup)
 	GSM_File 		File;
 	GSM_Error		error;
 	GSM_MemoryEntry		Pbk;
-	int			numPbk = 0, Pos;
+	int			numPbk = 0;
+	size_t Pos = 0;
 
 	File.Buffer = NULL;
 	error = GSM_ReadFile(FileName, &File);
 	if (error != ERR_NONE) return error;
 
-	Pos = 0;
 	while (1) {
 		error = GSM_DecodeVCARD(File.Buffer, &Pos, &Pbk, Nokia_VCard21);
 		if (error == ERR_EMPTY) break;
