@@ -82,7 +82,7 @@ static bool PHONE_IsPointBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, int x
 	case GSM_Nokia6210StartupLogo:
 	case GSM_Nokia7110StartupLogo:
 	case GSM_Nokia6510OperatorLogo:
-		i=(buffer[(y/8*width) + x] & 1<<(y%8));
+		i=(buffer[(y / 8 * width) + x] & (1 << (y % 8)));
 		break;
 	case GSM_NokiaOperatorLogo:
 	case GSM_Nokia7110OperatorLogo:
@@ -92,10 +92,10 @@ static bool PHONE_IsPointBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, int x
 	case GSM_EMSMediumPicture:
 	case GSM_EMSBigPicture:
 		pixel=width*y + x;
-		i=(buffer[pixel/8] & 1<<(7-(pixel%8)));
+		i=(buffer[pixel / 8] & (1 << (7 - (pixel % 8))));
 		break;
 	case GSM_NokiaPictureImage:
-		i=(buffer[9*y + x/8] & 1<<(7-(x%8)));
+		i=(buffer[(9 * y) + (x / 8)] & (1 << (7 - (x % 8))));
 		break;
 	case GSM_AlcatelBMMIPicture:
 		break;
@@ -112,7 +112,7 @@ static void PHONE_SetPointBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, int
 	case GSM_Nokia6210StartupLogo:
 	case GSM_Nokia7110StartupLogo:
 	case GSM_Nokia6510OperatorLogo:
-		buffer[(y/8*width)+x] |= 1 << (y%8);
+		buffer[(y / 8 * width)+x] |= 1 << (y % 8);
 		break;
 	case GSM_NokiaOperatorLogo:
 	case GSM_Nokia7110OperatorLogo:
@@ -122,15 +122,15 @@ static void PHONE_SetPointBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, int
 	case GSM_EMSBigPicture:
 	case GSM_EMSVariablePicture:
 		pixel = width*y + x;
-		buffer[pixel/8] |= 1 << (7-(pixel%8));
+		buffer[pixel / 8] |= 1 << (7 - (pixel % 8));
 		break;
 	case GSM_NokiaPictureImage:
-		buffer[9*y + x/8] |= 1 << (7-(x%8));
+		buffer[(9 * y) + (x / 8)] |= 1 << (7 - (x % 8));
 		break;
 	case GSM_AlcatelBMMIPicture:
 		pixel = height / 8;
 		if ((height % 8) != 0) pixel++;
-		buffer[pixel*x + y/8] |= 1 << (7 - (y%8));
+		buffer[(pixel * x) + (y / 8)] |= 1 << (7 - (y % 8));
 		break;
 	}
 }
@@ -209,7 +209,7 @@ void PHONE_EncodeBitmap(GSM_Phone_Bitmap_Types Type, char *buffer, GSM_Bitmap *B
 	}
 }
 
-void GSM_GetMaxBitmapWidthHeight(GSM_Bitmap_Types Type, unsigned char *width, unsigned char *height)
+void GSM_GetMaxBitmapWidthHeight(GSM_Bitmap_Types Type, size_t *width, size_t *height)
 {
 	switch (Type) {
 		case GSM_CallerGroupLogo: *width=72; *height=14; break;
@@ -251,7 +251,7 @@ size_t GSM_GetBitmapSize(GSM_Bitmap *bmp)
 
 void GSM_PrintBitmap(FILE *file, GSM_Bitmap *bitmap)
 {
-	int x,y;
+	size_t x,y;
 
 	for (y=0;y<bitmap->BitmapHeight;y++) {
 		for (x=0;x<bitmap->BitmapWidth;x++) {
@@ -267,7 +267,7 @@ void GSM_PrintBitmap(FILE *file, GSM_Bitmap *bitmap)
 
 void GSM_ReverseBitmap(GSM_Bitmap *Bitmap)
 {
-	int x, y;
+	size_t x, y;
 
 	for (x=0;x<Bitmap->BitmapWidth;x++) {
 		for (y=0;y<Bitmap->BitmapHeight;y++) {
@@ -280,9 +280,9 @@ void GSM_ReverseBitmap(GSM_Bitmap *Bitmap)
 	}
 }
 
-void GSM_ResizeBitmap(GSM_Bitmap *dest, GSM_Bitmap *src, int width, int height)
+void GSM_ResizeBitmap(GSM_Bitmap *dest, GSM_Bitmap *src, size_t width, size_t height)
 {
-	int startx=0,endx=0,setx=0, starty=0,endy=0,sety=0, x, y;
+	size_t startx=0,endx=0,setx=0, starty=0,endy=0,sety=0, x, y;
 
 	if (src->BitmapWidth<=width) {
 		startx	= 0;
@@ -315,7 +315,8 @@ void GSM_ResizeBitmap(GSM_Bitmap *dest, GSM_Bitmap *src, int width, int height)
 
 GSM_Error Bitmap2BMP(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 {
-	int		x,y,pos,i,sizeimage,buffpos=0;
+	size_t		x,i,sizeimage,buffpos=0;
+	ssize_t		y, pos;
 	unsigned char	buff[1];
 	div_t		division;
 	bool		isfile=false;
@@ -352,7 +353,7 @@ GSM_Error Bitmap2BMP(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 	pos	  = 7;
 	sizeimage = 0;
 	/*lines are written from the last to the first*/
-	for (y=bitmap->BitmapHeight-1;y>=0;y--) {
+	for (y = bitmap->BitmapHeight - 1; y >= 0; y--) {
 		i=1;
 		for (x=0;x<bitmap->BitmapWidth;x++) {
 			/*new byte !*/
@@ -377,12 +378,12 @@ GSM_Error Bitmap2BMP(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 			}
 		}
 	}
-	dbgprintf("Data size in BMP file: %i\n",sizeimage);
+	dbgprintf("Data size in BMP file: " SIZE_T_FORMAT "\n",sizeimage);
 	division=div(sizeimage,256);
 	header[35]=division.quot;
 	header[34]=sizeimage-(division.quot*256);
   	sizeimage=sizeimage+sizeof(header);
-	dbgprintf("Size of BMP file: %i\n",sizeimage);
+	dbgprintf("Size of BMP file: " SIZE_T_FORMAT "\n",sizeimage);
 	division=div(sizeimage,256);
 	header[3]=division.quot;
 	header[2]=sizeimage-(division.quot*256);
@@ -457,7 +458,8 @@ static GSM_Error savebmp(FILE *file, GSM_MultiBitmap *bitmap)
 static GSM_Error PrivSaveNLMWBMP(FILE *file, GSM_Bitmap *Bitmap)
 {
 	unsigned char	buffer[1000];
-	int		x,y,pos,pos2;
+	size_t		x,y,pos;
+	ssize_t		pos2;
 	div_t		division;
 
 	pos=0;pos2=7;
@@ -523,7 +525,7 @@ fail:
 static GSM_Error PrivSaveNGGNOL(FILE *file, GSM_MultiBitmap *bitmap)
 {
 	char 	buffer[GSM_BITMAP_SIZE];
-	int	x,y;
+	size_t	x,y;
 	size_t	current=0;
 
 	for (y=0;y<bitmap->Bitmap[0].BitmapHeight;y++) {
@@ -594,11 +596,13 @@ fail:
 
 static GSM_Error savexpm(FILE *file, GSM_MultiBitmap *bitmap)
 {
-	int x,y;
+	size_t x,y;
 
 	fprintf(file,"/* XPM */\n");
 	fprintf(file,"static char * ala_xpm[] = {\n");
-	fprintf(file,"\"%i %i 2 1\",\n",bitmap->Bitmap[0].BitmapWidth,bitmap->Bitmap[0].BitmapHeight);
+	fprintf(file,"\"" SIZE_T_FORMAT " " SIZE_T_FORMAT " 2 1\",\n",
+		bitmap->Bitmap[0].BitmapWidth,
+		bitmap->Bitmap[0].BitmapHeight);
 	fprintf(file,"\".	s c	m #000000	g4 #000000	g #000000	c #000000\",\n");
 	fprintf(file,"\"#	s c	m #ffffff	g4 #ffffff	g #ffffff	c #ffffff\",\n");
 
@@ -686,7 +690,8 @@ GSM_Error BMP2Bitmap(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 {
 	bool		first_white,isfile=false;
 	unsigned char 	buff[34];
-	int		w,h,y,x,i,pos,buffpos=0;
+	size_t		w,h,x,i,buffpos=0;
+	ssize_t		y, pos;
 	size_t		readbytes;
 #ifdef DEBUG
 	int		sizeimage=0;
@@ -704,12 +709,12 @@ GSM_Error BMP2Bitmap(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 	/* height and width of image in the file */
 	h=buff[22]+256*buff[21];
 	w=buff[18]+256*buff[17];
-	dbgprintf("Image Size in BMP file: %dx%d\n",w,h);
+	dbgprintf("Image Size in BMP file: " SIZE_T_FORMAT "x" SIZE_T_FORMAT "\n",w,h);
 
 	GSM_GetMaxBitmapWidthHeight(bitmap->Type, &bitmap->BitmapWidth, &bitmap->BitmapHeight);
 	if (h<bitmap->BitmapHeight)	bitmap->BitmapHeight=h;
 	if (w<bitmap->BitmapWidth)	bitmap->BitmapWidth=w;
-	dbgprintf("Height %i %i, width %i %i\n",h,bitmap->BitmapHeight,w,bitmap->BitmapWidth);
+	dbgprintf("Height " SIZE_T_FORMAT " " SIZE_T_FORMAT ", width " SIZE_T_FORMAT " " SIZE_T_FORMAT "\n",h,bitmap->BitmapHeight,w,bitmap->BitmapWidth);
 
 	GSM_ClearBitmap(bitmap);
 
@@ -789,7 +794,7 @@ GSM_Error BMP2Bitmap(unsigned char *buffer, FILE *file,GSM_Bitmap *bitmap)
 				if(i==5) i=1;
 			}
 			/* we have top left corner ! */
-			if (x<=bitmap->BitmapWidth && y<=bitmap->BitmapHeight) {
+			if (x<=bitmap->BitmapWidth && (size_t)y<=bitmap->BitmapHeight) {
 				if (first_white) {
 					if ((buff[0]&(1<<pos))<=0) GSM_SetPointBitmap(bitmap,x,y);
 				} else {
@@ -837,7 +842,8 @@ static GSM_Error loadbmp(FILE *file, GSM_MultiBitmap *bitmap)
 static GSM_Error loadnlm (FILE *file, GSM_MultiBitmap *bitmap)
 {
 	unsigned char 	buffer[1000];
-	int 		pos,pos2,x,y,h,w,i,number;
+	size_t 		pos,x,y,h,w,i,number;
+	ssize_t		pos2;
 	div_t		division;
 	size_t		readbytes;
 
@@ -906,7 +912,7 @@ static GSM_Error loadnlm (FILE *file, GSM_MultiBitmap *bitmap)
 static GSM_Error loadnolngg(FILE *file, GSM_MultiBitmap *bitmap, bool nolformat)
 {
 	unsigned char 	buffer[2000];
-	int		i,h,w,x,y;
+	size_t		i,h,w,x,y;
 	size_t		readbytes;
 
 	readbytes = fread(buffer, 1, 6, file);
@@ -966,7 +972,7 @@ static GSM_Error loadnsl(FILE *file, GSM_MultiBitmap *bitmap)
 
 	while (fread(block,1,6,file)==6) {
 		block_size = block[4]*256 + block[5];
-		dbgprintf("Block %c%c%c%c, size %i\n",block[0],block[1],block[2],block[3],block_size);
+		dbgprintf("Block %c%c%c%c, size " SIZE_T_FORMAT "\n",block[0],block[1],block[2],block[3],block_size);
 		if (!strncmp(block, "FORM", 4)) {
 			dbgprintf("File ID\n");
 		} else {
@@ -987,7 +993,7 @@ static GSM_Error loadnsl(FILE *file, GSM_MultiBitmap *bitmap)
 					OldType = bitmap->Bitmap[0].Type;
 					PHONE_DecodeBitmap(GSM_NokiaStartupLogo, buffer, &bitmap->Bitmap[0]);
 					if (OldType != GSM_None) bitmap->Bitmap[0].Type = OldType;
-					dbgprintf("Startup logo (size %i)\n",block_size);
+					dbgprintf("Startup logo (size " SIZE_T_FORMAT  ")\n",block_size);
 				}
 			}
 		}
@@ -1031,7 +1037,7 @@ static GSM_Error loadgif(FILE *file, GSM_MultiBitmap *bitmap)
 
 	readbytes = fread(buffer, 1, length, file);
 	if (readbytes != length) return ERR_FILENOTSUPPORTED;
-        dbgprintf("Length %i name \"%s\"\n", length,
+        dbgprintf("Length " SIZE_T_FORMAT  " name \"%s\"\n", length,
 		DecodeUnicodeString(bmap->Name));
 
 	bmap->Type = GSM_PictureBinary;
