@@ -79,7 +79,7 @@ GSM_Error MakeKeySequence(char *text, GSM_KeyCode *KeyCode, size_t *Length)
 	return ERR_NONE;
 }
 
-GSM_Error GSM_ReadFile(char *FileName, GSM_File *File)
+GSM_Error GSM_ReadFile(const char *FileName, GSM_File *File)
 {
 	int 		i = 1000;
 	FILE		*file;
@@ -117,15 +117,15 @@ GSM_Error GSM_ReadFile(char *FileName, GSM_File *File)
 	return ERR_NONE;
 }
 
-static void GSM_JADFindLine(GSM_File File, char *Name, char *Value)
+static void GSM_JADFindLine(GSM_File File, const char *Name, char *Value)
 {
 	unsigned char 	Line[2000];
-	int		Pos = 0;
+	size_t		Pos = 0;
 
 	Value[0] = 0;
 
 	while (1) {
-		MyGetLine(File.Buffer, &Pos, Line, File.Used, false);
+		if (MyGetLine(File.Buffer, &Pos, Line, File.Used, sizeof(Line), false) != ERR_NONE) break;
 		if (strlen(Line) == 0) break;
 		if (!strncmp(Line,Name,strlen(Name))) {
 			Pos = strlen(Name);
@@ -186,25 +186,72 @@ void GSM_IdentifyFileFormat(GSM_File *File)
 	}
 }
 
-void SaveVCALDateTime(char *Buffer, size_t *Length, GSM_DateTime *Date, char *Start)
+PRINTF_STYLE(4, 5)
+GSM_Error VC_StoreLine(char *Buffer, const size_t buff_len, size_t *Pos, const char *format, ...)
 {
-	if (Start != NULL) {
-		*Length+=sprintf(Buffer+(*Length), "%s:",Start);
-	}
-	*Length+=sprintf(Buffer+(*Length), "%04d%02d%02dT%02d%02d%02d%s%c%c",
-			Date->Year, Date->Month, Date->Day,
-			Date->Hour, Date->Minute, Date->Second,
-			Date->Timezone == 0 ? "Z" : "",
-			13,10);
+        va_list 		argp;
+	int 			result;
+
+	va_start(argp, format);
+	result = vsnprintf(Buffer + (*Pos), buff_len - *Pos - 1, format, argp);
+	va_end(argp);
+
+	*Pos += result;
+	if (*Pos >= buff_len - 1) return ERR_MOREMEMORY;
+
+	result = snprintf(Buffer + (*Pos), buff_len - *Pos - 1, "%c%c", 13, 10);
+
+	*Pos += result;
+	if (*Pos >= buff_len - 1) return ERR_MOREMEMORY;
+
+	return ERR_NONE;
 }
 
-void SaveVCALDate(char *Buffer, size_t *Length, GSM_DateTime *Date, char *Start)
+PRINTF_STYLE(4, 5)
+GSM_Error VC_Store(char *Buffer, const size_t buff_len, size_t *Pos, const char *format, ...)
 {
+        va_list 		argp;
+	int 			result;
+
+	va_start(argp, format);
+	result = vsnprintf(Buffer + (*Pos), buff_len - *Pos - 1, format, argp);
+	va_end(argp);
+
+	*Pos += result;
+	if (*Pos >= buff_len - 1) return ERR_MOREMEMORY;
+
+	return ERR_NONE;
+}
+
+
+GSM_Error VC_StoreDateTime(char *Buffer, const size_t buff_len, size_t *Pos, const GSM_DateTime *Date, const char *Start)
+{
+	GSM_Error error;
+
 	if (Start != NULL) {
-		*Length+=sprintf(Buffer+(*Length), "%s:",Start);
+		error = VC_Store(Buffer, buff_len, Pos, "%s:", Start);
+		if (error != ERR_NONE) return error;
 	}
-	*Length+=sprintf(Buffer+(*Length), "%04d%02d%02d%C%C",
-			Date->Year, Date->Month, Date->Day,13,10);
+	error = VC_StoreLine(Buffer, buff_len, Pos,
+		"%04d%02d%02dT%02d%02d%02d%s",
+			Date->Year, Date->Month, Date->Day,
+			Date->Hour, Date->Minute, Date->Second,
+			Date->Timezone == 0 ? "Z" : "");
+	return error;
+}
+
+GSM_Error VC_StoreDate(char *Buffer, const size_t buff_len, size_t *Pos, const GSM_DateTime *Date, const char *Start)
+{
+	GSM_Error error;
+
+	if (Start != NULL) {
+		error = VC_Store(Buffer, buff_len, Pos, "%s:", Start);
+		if (error != ERR_NONE) return error;
+	}
+	error = VC_StoreLine(Buffer, buff_len, Pos,
+		"%04d%02d%02d",
+			Date->Year, Date->Month, Date->Day);
+	return error;
 }
 
 bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
@@ -214,6 +261,8 @@ bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 
 	memset(dt,0,sizeof(GSM_DateTime));
 
+	if (strlen(Buffer) < 8) return false;
+
 	strncpy(year, 	Buffer, 	4);
 	strncpy(month, 	Buffer+4, 	2);
 	strncpy(day, 	Buffer+6, 	2);
@@ -222,6 +271,8 @@ bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 	dt->Day		= atoi(day);
 
 	if (Buffer[8] == 'T') {
+		if (strlen(Buffer + 9) < 6) return false;
+
 		strncpy(hour, 	Buffer+9,	2);
 		strncpy(minute, Buffer+11,	2);
 		strncpy(second, Buffer+13,	2);
@@ -247,7 +298,7 @@ bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 	}
 
 	if (dt->Timezone != 0) {
-		timestamp = Fill_Time_T(*dt) + dt->Timezone*3600;
+		timestamp = Fill_Time_T(*dt) + dt->Timezone;
 		Fill_GSM_DateTime(dt, timestamp);
 		dt->Year +=1900;
 	}
@@ -255,7 +306,7 @@ bool ReadVCALDateTime(const char *Buffer, GSM_DateTime *dt)
 	return true;
 }
 
-bool ReadVCALInt(char *Buffer, char *Start, int *Value)
+bool ReadVCALInt(char *Buffer, const char *Start, int *Value)
 {
 	unsigned char buff[200];
 
@@ -277,7 +328,7 @@ bool ReadVCALInt(char *Buffer, char *Start, int *Value)
 }
 
 
-bool ReadVCALDate(char *Buffer, char *Start, GSM_DateTime *Date, bool *is_date_only)
+bool ReadVCALDate(char *Buffer, const char *Start, GSM_DateTime *Date, bool *is_date_only)
 {
 	char fullstart[200];
 	unsigned char datestring[200];
@@ -303,23 +354,77 @@ bool ReadVCALDate(char *Buffer, char *Start, GSM_DateTime *Date, bool *is_date_o
 }
 
 
-void SaveVCALText(char *Buffer, size_t *Length, unsigned char *Text, char *Start, bool UTF8)
+GSM_Error VC_StoreText(char *Buffer, const size_t buff_len, size_t *Pos, const unsigned char *Text, const char *Start, const bool UTF8)
 {
-	char buffer[1000];
+	char *buffer;
+	size_t len;
+	GSM_Error error;
 
-	if (UnicodeLength(Text) != 0) {
-		if (UTF8) {
-			EncodeUTF8(buffer, Text);
-			*Length += sprintf(Buffer+(*Length), "%s:%s%c%c", Start, DecodeUnicodeString(Text), 13, 10);
+	len = UnicodeLength(Text);
+
+	if (len == 0) return ERR_NONE;
+
+	/* Need to be big enough to store quoted printable */
+	buffer = (char *)malloc(len * 8);
+	if (buffer == NULL) return ERR_MOREMEMORY;
+
+	if (UTF8) {
+		EncodeUTF8(buffer, Text);
+		error =  VC_StoreLine(Buffer, buff_len, Pos, "%s:%s", Start, buffer);
+	} else {
+		EncodeUTF8QuotedPrintable(buffer,Text);
+		if (UnicodeLength(Text) == strlen(buffer)) {
+			/* Text is plain ASCII */
+			error =  VC_StoreLine(Buffer, buff_len, Pos, "%s:%s", Start, buffer);
 		} else {
-			EncodeUTF8QuotedPrintable(buffer,Text);
-			if (UnicodeLength(Text)==strlen(buffer)) {
-				*Length+=sprintf(Buffer+(*Length), "%s:%s%c%c",Start,DecodeUnicodeString(Text),13,10);
-			} else {
-				*Length+=sprintf(Buffer+(*Length), "%s;CHARSET=UTF-8;ENCODING=QUOTED-PRINTABLE:%s%c%c",Start,buffer,13,10);
-			}
+			error =  VC_StoreLine(Buffer, buff_len, Pos, "%s;CHARSET=UTF-8;ENCODING=QUOTED-PRINTABLE:%s", Start, buffer);
 		}
 	}
+
+	free(buffer);
+	return error;
+}
+
+GSM_Error VC_StoreBase64(char *Buffer, const size_t buff_len, size_t *Pos, const unsigned char *data, const size_t length)
+{
+	char *buffer, *pos, linebuffer[80];
+	size_t len, current;
+	char spacer[2];
+	GSM_Error error;
+
+	/*
+	 * Need to be big enough to store base64 (what is *4/3, but *2 is safer
+	 * and we don't have to care about rounding and padding).
+	 */
+	buffer = (char *)malloc(length * 2);
+	if (buffer == NULL) return ERR_MOREMEMORY;
+
+	spacer[0] = 0;
+	spacer[1] = 0;
+
+
+	EncodeBASE64(data, buffer, length);
+
+	len = strlen(buffer);
+	pos = buffer;
+
+	/* Write at most 76 chars per line */
+	while (len > 0) {
+		current = MIN(len, 76);
+		strncpy(linebuffer, pos, current);
+		linebuffer[current] = 0;
+		error =  VC_StoreLine(Buffer, buff_len, Pos, "%s%s", spacer, linebuffer);
+		if (error != ERR_NONE) {
+			free(buffer);
+			return error;
+		}
+		spacer[0] = ' ';
+		len -= current;
+		pos += current;
+	}
+
+	free(buffer);
+	return ERR_NONE;
 }
 
 unsigned char *VCALGetTextPart(unsigned char *Buff, int *pos)
@@ -349,7 +454,7 @@ unsigned char *VCALGetTextPart(unsigned char *Buff, int *pos)
  *
  * When all tokens are matched we found matching line.
  */
-bool ReadVCALText(char *Buffer, char *Start, unsigned char *Value, bool UTF8)
+bool ReadVCALText(char *Buffer, const char *Start, unsigned char *Value, const bool UTF8)
 {
 	char *line = NULL;
 	char **tokens = NULL;
@@ -584,65 +689,6 @@ fail:
 	return ret;
 }
 
-bool GSM_ReadHTTPFile(unsigned char *server, unsigned char *filename, GSM_File *file)
-{
-	socket_type s;
-	int len;
-	struct sockaddr_in 	address;
-	struct hostent 		*address2;
-	unsigned char 		buff[200];
-#ifdef WIN32
-	WSADATA			wsaData;
-
-	if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) return false;
-#endif
-
-	s = socket(AF_INET,SOCK_STREAM,0);
-	if (s == socket_invalid) return false;
-
-	address2 = gethostbyname(server);
-	if (address2 == NULL) return false;
-
-	memset((char *) &address, 0, sizeof(address));
-	address.sin_family 	= AF_INET;
-	address.sin_port 	= htons(80);
-	address.sin_addr.s_addr = *(u_long *) *(address2->h_addr_list);
-
-	if (connect(s,(struct sockaddr *)&address,sizeof(address))<0) return false;
-
-	sprintf(buff,"GET /%s HTTP/1.1\x0d\x0aHost: %s\x0d\x0aUser-Agent: Gammu/%s\x0d\x0a\x0d\x0a", filename, server, VERSION);
-	if (send(s,buff,strlen(buff),0)<0) return false;
-
-	free(file->Buffer);
-	file->Buffer 	= NULL;
-	file->Used 	= 0;
-
-#ifdef WIN32
-	while ((len=recv(s,buff,200,0))>0) {
-#else
-	while ((len=read(s,buff,200))>0) {
-#endif
-		file->Buffer = realloc(file->Buffer,file->Used + len);
-		memcpy(file->Buffer+file->Used,buff,len);
-		file->Used += len;
-	}
-#ifdef WIN32
-	closesocket(s);
-#else
-	close(s);
-#endif
-
-	if (file->Buffer == NULL) return false;
-	if (strstr(file->Buffer,"HTTP/1.1 200 OK")==NULL) {
-		free(file->Buffer);
-		file->Buffer = NULL;
-		file->Used   = 0;
-		return false;
-	}
-
-	return true;
-}
-
 void GSM_ClearBatteryCharge(GSM_BatteryCharge *bat)
 {
     bat->BatteryPercent = -1;
