@@ -2,14 +2,34 @@
 
 #include <string.h>
 #include <stdio.h>
+#include <stdarg.h>
+#include <ctype.h>
 
 #include <gammu-misc.h>
+#include <gammu-bitmap.h>
 
 #include "../misc/coding/coding.h"
+#include "../misc/misc.h"
 #include "gsmpbk.h"
 #include "gsmmisc.h"
 
-unsigned char *GSM_PhonebookGetEntryName (GSM_MemoryEntry *entry)
+GSM_MemoryType GSM_StringToMemoryType(const char *s) {
+    if (strcmp(s, "ME") == 0)      return MEM_ME;
+    else if (strcmp(s, "SM") == 0) return MEM_SM;
+    else if (strcmp(s, "ON") == 0) return MEM_ON;
+    else if (strcmp(s, "DC") == 0) return MEM_DC;
+    else if (strcmp(s, "RC") == 0) return MEM_RC;
+    else if (strcmp(s, "MC") == 0) return MEM_MC;
+    else if (strcmp(s, "MT") == 0) return MEM_MT;
+    else if (strcmp(s, "FD") == 0) return MEM_FD;
+    else if (strcmp(s, "VM") == 0) return MEM_VM;
+    else if (strcmp(s, "SL") == 0) return MEM_SL;
+    else {
+		return 0;
+    }
+}
+
+unsigned char *GSM_PhonebookGetEntryName (const GSM_MemoryEntry *entry)
 {
 	/* We possibly store here "LastName, FirstName" so allocate enough memory */
 	static char     dest[(GSM_PHONEBOOK_TEXT_LENGTH*2+2+1)*2];
@@ -54,7 +74,7 @@ unsigned char *GSM_PhonebookGetEntryName (GSM_MemoryEntry *entry)
 	return dest;
 }
 
-void GSM_PhonebookFindDefaultNameNumberGroup(GSM_MemoryEntry *entry, int *Name, int *Number, int *Group)
+void GSM_PhonebookFindDefaultNameNumberGroup(const GSM_MemoryEntry *entry, int *Name, int *Number, int *Group)
 {
 	int i;
 
@@ -102,7 +122,8 @@ void GSM_PhonebookFindDefaultNameNumberGroup(GSM_MemoryEntry *entry, int *Name,
 	}
 }
 
-void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool header, GSM_VCardVersion Version)
+
+GSM_Error GSM_EncodeVCARD(char *Buffer, const size_t buff_len, size_t *Length, GSM_MemoryEntry *pbk, const bool header, const GSM_VCardVersion Version)
 {
 	int Name, Number, Group, i;
 	int firstname = -1, lastname = -1;
@@ -111,32 +132,53 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 	unsigned char buffer[1024];
 	int pos;
 	bool ignore;
+	GSM_Error error;
+	GSM_BinaryPicture *bitmap;
 
 	GSM_PhonebookFindDefaultNameNumberGroup(pbk, &Name, &Number, &Group);
 
-	if (Version == Nokia_VCard10) {
-		if (header) *Length+=sprintf(Buffer+(*Length),"BEGIN:VCARD%c%c",13,10);
+	if (header) {
+		error = VC_StoreLine(Buffer, buff_len, Length, "BEGIN:VCARD");
+		if (error != ERR_NONE) return error;
+	}
+	if (Version == Nokia_VCard10 || Version == SonyEricsson_VCard10) {
 		if (Name != -1) {
-			*Length+=sprintf(Buffer+(*Length),"N:%s%c%c",DecodeUnicodeString(pbk->Entries[Name].Text),13,10);
+			error = VC_StoreLine(Buffer, buff_len, Length, "N:%s",
+				DecodeUnicodeString(pbk->Entries[Name].Text));
+			if (error != ERR_NONE) return error;
 		}
 		if (Number != -1) {
-			*Length +=sprintf(Buffer+(*Length),"TEL:%s%c%c",DecodeUnicodeString(pbk->Entries[Number].Text),13,10);
+			error = VC_StoreLine(Buffer, buff_len, Length, "TEL:%s",
+				DecodeUnicodeString(pbk->Entries[Number].Text));
+			if (error != ERR_NONE) return error;
 		}
-		if (header) *Length+=sprintf(Buffer+(*Length),"END:VCARD%c%c",13,10);
 	} else if (Version == Nokia_VCard21 || Version == SonyEricsson_VCard21) {
-		if (header) *Length+=sprintf(Buffer+(*Length),"BEGIN:VCARD%c%cVERSION:2.1%c%c",13,10,13,10);
+		if (header) {
+			error = VC_StoreLine(Buffer, buff_len, Length, "VERSION:2.1");
+			if (error != ERR_NONE) return error;
+		}
+
+		error = VC_StoreLine(Buffer, buff_len, Length, "X-GAMMU-LOCATION:%d", pbk->Location);
+		if (error != ERR_NONE) return error;
+
+		error = VC_StoreLine(Buffer, buff_len, Length, "X-GAMMU-MEMORY:%s", GSM_MemoryTypeToString(pbk->MemoryType));
+		if (error != ERR_NONE) return error;
+
 		for (i=0; i < pbk->EntriesNum; i++) {
 			ignore = false;
 			pbk->Entries[i].AddError = ERR_NONE;
 			switch(pbk->Entries[i].EntryType) {
 				case PBK_Text_Name:
-					*Length+=sprintf(Buffer+(*Length),"N");
+					error = VC_Store(Buffer, buff_len, Length, "N");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_NickName:
-					*Length+=sprintf(Buffer+(*Length),"NICKNAME");
+					error = VC_Store(Buffer, buff_len, Length, "NICKNAME");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_FormalName:
-					*Length+=sprintf(Buffer+(*Length),"FN");
+					error = VC_Store(Buffer, buff_len, Length, "FN");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_FirstName:
 					firstname = i;
@@ -187,81 +229,74 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 					ignore = true;
 					break;
 				case PBK_Date:
-					SaveVCALDate(Buffer, Length, &(pbk->Entries[i].Date), "BDAY");
+					error = VC_StoreDate(Buffer, buff_len, Length, &(pbk->Entries[i].Date), "BDAY");
+					if (error != ERR_NONE) return error;
 					ignore = true;
 					break;
 				case PBK_LastModified:
-					SaveVCALDateTime(Buffer, Length, &(pbk->Entries[i].Date), "LAST-MODIFIED");
+					error = VC_StoreDateTime(Buffer, buff_len, Length, &(pbk->Entries[i].Date), "LAST-MODIFIED");
+					if (error != ERR_NONE) return error;
 					ignore = true;
 					break;
 				case PBK_Number_General:
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					break;
 				case PBK_Number_Other:
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";OTHER");
-					break;
 				case PBK_Number_Pager:
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";PAGER");
-					break;
 				case PBK_Number_Mobile  :
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";CELL");
-					break;
 				case PBK_Number_Work    :
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";WORK");
-					break;
 				case PBK_Number_Fax     :
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = true;
-						break;
-					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";FAX");
-					break;
 				case PBK_Number_Home    :
+				case PBK_Number_Messaging    :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"TEL");
-					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
-					*Length+=sprintf(Buffer+(*Length),";HOME");
+					error = VC_Store(Buffer, buff_len, Length, "TEL");
+					if (error != ERR_NONE) return error;
+
+					if (Version != SonyEricsson_VCard21 && Number == i) {
+						error = VC_Store(Buffer, buff_len, Length, ";PREF");
+						if (error != ERR_NONE) return error;
+					}
+					switch (pbk->Entries[i].EntryType) {
+						case PBK_Number_Other:
+							error = VC_Store(Buffer, buff_len, Length, ";OTHER");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Pager:
+							error = VC_Store(Buffer, buff_len, Length, ";PAGER");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Mobile:
+							error = VC_Store(Buffer, buff_len, Length, ";CELL");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Work:
+							error = VC_Store(Buffer, buff_len, Length, ";WORK");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Fax:
+							error = VC_Store(Buffer, buff_len, Length, ";FAX");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Home:
+							error = VC_Store(Buffer, buff_len, Length, ";HOME");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Number_Messaging:
+							error = VC_Store(Buffer, buff_len, Length, ";MSG");
+							if (error != ERR_NONE) return error;
+							break;
+						default:
+							break;
+					}
 					break;
 				case PBK_Text_Note      :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"NOTE");
+					error = VC_Store(Buffer, buff_len, Length, "NOTE");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_Postal    :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
@@ -271,15 +306,18 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 					/* Don't ask why. Nokia phones save postal address
 					 * double - once like LABEL, second like ADR
 					 */
-					SaveVCALText(Buffer, Length, pbk->Entries[i].Text, "LABEL", false);
-					*Length+=sprintf(Buffer+(*Length),"ADR;HOME");
+					error = VC_StoreText(Buffer, buff_len, Length, pbk->Entries[i].Text, "LABEL", false);
+					if (error != ERR_NONE) return error;
+					error = VC_Store(Buffer, buff_len, Length, "ADR;HOME");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_WorkPostal    :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"ADR;WORK");
+					error = VC_Store(Buffer, buff_len, Length, "ADR;WORK");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_Email     :
 				case PBK_Text_Email2    :
@@ -287,40 +325,85 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"EMAIL");
+					error = VC_Store(Buffer, buff_len, Length, "EMAIL");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_URL       :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"URL");
+					error = VC_Store(Buffer, buff_len, Length, "URL");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_LUID      :
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"X-IRMC-LUID");
+					error = VC_Store(Buffer, buff_len, Length, "X-IRMC-LUID");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_JobTitle:
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"TITLE");
+					error = VC_Store(Buffer, buff_len, Length, "TITLE");
+					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_Company:
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = true;
 						break;
 					}
-					*Length+=sprintf(Buffer+(*Length),"ORG");
+					error = VC_Store(Buffer, buff_len, Length, "ORG");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Photo:
+					bitmap = &(pbk->Entries[i].Picture);
+					error = VC_Store(Buffer, buff_len, Length, "PHOTO;TYPE=");
+					if (error != ERR_NONE) return error;
+					switch (bitmap->Type) {
+						case PICTURE_BMP:
+							error = VC_Store(Buffer, buff_len, Length, "BMP;");
+							break;
+						case PICTURE_GIF:
+							error = VC_Store(Buffer, buff_len, Length, "GIF;");
+							break;
+						case PICTURE_JPG:
+							error = VC_Store(Buffer, buff_len, Length, "JPEG;");
+							break;
+						case PICTURE_ICN:
+							error = VC_Store(Buffer, buff_len, Length, "ICO;");
+							break;
+						case PICTURE_PNG:
+							error = VC_Store(Buffer, buff_len, Length, "PNG;");
+							break;
+						default:
+							dbgprintf("Unknown picture format: %d\n", bitmap->Type);
+							error = VC_Store(Buffer, buff_len, Length, "UNKNOWN;");
+							break;
+					}
+					if (error != ERR_NONE) return error;
+					error = VC_Store(Buffer, buff_len, Length, "ENCODING=BASE64:");
+					if (error != ERR_NONE) return error;
+					error = VC_StoreBase64(Buffer, buff_len, Length, bitmap->Buffer, bitmap->Length);
+					if (error != ERR_NONE) return error;
+					ignore = true;
+					break;
+				case PBK_Caller_Group:
+					error = VC_StoreLine(Buffer, buff_len, Length, "X-CALLER-GROUP:%d", pbk->Entries[i].Number);
+					if (error != ERR_NONE) return error;
+					ignore = true;
+					break;
+				case PBK_Private:
+					error = VC_StoreLine(Buffer, buff_len, Length, "X-PRIVATE:%d", pbk->Entries[i].Number);
+					if (error != ERR_NONE) return error;
+					ignore = true;
 					break;
 				/* Not supported fields */
-				case PBK_Caller_Group:
 				case PBK_Category:
-				case PBK_Private:
 				case PBK_RingtoneID:
 				case PBK_PictureID:
 				case PBK_Text_UserID:
@@ -336,7 +419,8 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 					break;
 			}
 			if (!ignore) {
-				SaveVCALText(Buffer, Length, pbk->Entries[i].Text, "", false);
+				error = VC_StoreText(Buffer, buff_len, Length, pbk->Entries[i].Text, "", false);
+				if (error != ERR_NONE) return error;
 			}
 		}
 		/* Save name if it is composed from parts */
@@ -355,7 +439,8 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 			}
 			buffer[2*pos] = 0;
 			buffer[2*pos + 1] = 0;
-			SaveVCALText(Buffer, Length, buffer, "N", false);
+			error = VC_StoreText(Buffer, buff_len, Length, buffer, "N", false);
+			if (error != ERR_NONE) return error;
 		}
 		/* Save workaddress if it is composed from parts */
 		if (workaddress != -1 || workcity != -1 || workstate != -1 || workzip != -1 || workcountry != -1) {
@@ -400,7 +485,8 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 			}
 			buffer[2*pos] = 0;
 			buffer[2*pos + 1] = 0;
-			SaveVCALText(Buffer, Length, buffer, "ADR;WORK", false);
+			error = VC_StoreText(Buffer, buff_len, Length, buffer, "ADR;WORK", false);
+			if (error != ERR_NONE) return error;
 		}
 		/* Save address if it is composed from parts */
 		if (address != -1 || city != -1 || state != -1 || zip != -1 || country != -1) {
@@ -445,10 +531,17 @@ void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool he
 			}
 			buffer[2*pos] = 0;
 			buffer[2*pos + 1] = 0;
-			SaveVCALText(Buffer, Length, buffer, "ADR;HOME", false);
+			error = VC_StoreText(Buffer, buff_len, Length, buffer, "ADR;HOME", false);
+			if (error != ERR_NONE) return error;
 		}
-		if (header) *Length+=sprintf(Buffer+(*Length),"END:VCARD%c%c",13,10);
+	} else {
+		return ERR_NOTSUPPORTED;
 	}
+	if (header) {
+		error = VC_StoreLine(Buffer, buff_len, Length, "END:VCARD");
+		if (error != ERR_NONE) return error;
+	}
+	return ERR_NONE;
 }
 
 void GSM_TweakInternationalNumber(unsigned char *Number, const GSM_NumberType numType)
@@ -500,32 +593,43 @@ void GSM_TweakInternationalNumber(unsigned char *Number, const GSM_NumberType nu
 }
 
 
+#define CHECK_NUM_ENTRIES if (Pbk->EntriesNum >= GSM_PHONEBOOK_ENTRIES) { error = ERR_MOREMEMORY; goto vcard_done; }
+
 /**
  * \bug We should avoid using static buffers here.
  */
-GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk, GSM_VCardVersion Version UNUSED)
+GSM_Error GSM_DecodeVCARD(char *Buffer, size_t *Pos, GSM_MemoryEntry *Pbk, GSM_VCardVersion Version)
 {
-	unsigned char   Line[20000],Buff[20000];
+	char   Buff[20000];
 	int	     Level = 0;
-	unsigned char   *s;
+	char   *s;
 	int		pos;
+	GSM_Error	error;
+	char	*Line = NULL;
 
 	Buff[0]	 = 0;
 	Pbk->EntriesNum = 0;
+	if (Version != SonyEricsson_VCard21_Phone) {
+		Pbk->Location = 0;
+		Pbk->MemoryType = MEM_MT;
+	}
 
 	while (1) {
-		MyGetLine(Buffer, Pos, Line, strlen(Buffer), true);
+		free(Line);
+		Line = NULL;
+		error = GSM_GetVCSLine(&Line, Buffer, Pos, strlen(Buffer), true);
+		if (error != ERR_NONE) goto vcard_done;
 		if (strlen(Line) == 0) break;
 		switch (Level) {
 		case 0:
 			if (strstr(Line,"BEGIN:VCARD")) Level = 1;
 			break;
 		case 1:
-			if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+			CHECK_NUM_ENTRIES;
 			if (strstr(Line,"END:VCARD")) {
-				if (Pbk->EntriesNum == 0) return ERR_EMPTY;
-				return ERR_NONE;
+				goto vcard_complete;
 			}
+			Pbk->Entries[Pbk->EntriesNum].AddError = ERR_NONE;
 			if (ReadVCALText(Line, "N", Buff, false)) {
 				pos = 0;
 				s = VCALGetTextPart(Buff, &pos);
@@ -543,12 +647,39 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_FirstName;
 					Pbk->EntriesNum++;
 				}
 			}
+			if (strncmp(Line, "PHOTO;JPEG;BASE64:", 18) == 0 ||
+				strncmp(Line, "PHOTO;BASE64;JPEG:", 18) == 0 ||
+				strncmp(Line, "PHOTO;TYPE=JPEG;BASE64:", 23) == 0 ||
+				strncmp(Line, "PHOTO;BASE64;TYPE=JPEG:", 23) == 0 ||
+				strncmp(Line, "PHOTO;TYPE=JPEG;ENCODING=BASE64:", 32) == 0 ||
+				strncmp(Line, "PHOTO;ENCODING=BASE64;TYPE=JPEG:", 32) == 0 ||
+				strncmp(Line, "PHOTO;JPEG;ENCODING=BASE64:", 27) == 0 ||
+				strncmp(Line, "PHOTO;ENCODING=BASE64;JPEG:", 27) == 0) {
+				/* Find : (it should be there we matched it above) */
+				s = strchr(Line, ':');
+				s++;
+				/* Skip whitespace */
+				while (isspace(*s) && *s) s++;
+
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Photo;
+				Pbk->Entries[Pbk->EntriesNum].Picture.Type = PICTURE_JPG;
+
+				/* We allocate here more memory than is actually required */
+				Pbk->Entries[Pbk->EntriesNum].Picture.Buffer = malloc(strlen(s));
+				if (Pbk->Entries[Pbk->EntriesNum].Picture.Buffer == NULL)
+					return ERR_MOREMEMORY;
+
+				Pbk->Entries[Pbk->EntriesNum].Picture.Length =
+					DecodeBASE64(s, Pbk->Entries[Pbk->EntriesNum].Picture.Buffer, strlen(s));
+				Pbk->EntriesNum++;
+			}
+
 			if (ReadVCALText(Line, "TEL",		   Buff, false) ||
 			    ReadVCALText(Line, "TEL;VOICE",	     Buff, false) ||
 			    ReadVCALText(Line, "TEL;TYPE=VOICE",	     Buff, false)) {
@@ -606,6 +737,36 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
 			}
+			if (ReadVCALText(Line, "TEL;PAGER",	      Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;PAGER;VOICE",	Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER,VOICE",   Buff, false)) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "TEL;MSG",	      Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;MSG;VOICE",	Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG,VOICE",   Buff, false)) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+			}
 			/* FAX + VOICE looks like nonsense */
 			if (ReadVCALText(Line, "TEL;FAX",	       Buff, false) ||
 			    ReadVCALText(Line, "TEL;TYPE=FAX",	  Buff, false) ||
@@ -664,28 +825,28 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_StreetAddress;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_City;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_State;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Zip;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
@@ -709,28 +870,28 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkStreetAddress;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkCity;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkState;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkZip;
 					Pbk->EntriesNum++;
-					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
@@ -789,14 +950,35 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 					Pbk->EntriesNum++;
 				}
 			}
+			if (ReadVCALText(Line, "X-PRIVATE", Buff, false)) {
+				Pbk->Entries[Pbk->EntriesNum].Number = atoi(DecodeUnicodeString(Buff));
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Private;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "X-CALLER-GROUP", Buff, false)) {
+				Pbk->Entries[Pbk->EntriesNum].Number = atoi(DecodeUnicodeString(Buff));
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Caller_Group;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "X-GAMMU-LOCATION", Buff, false)) {
+				Pbk->Location = atoi(DecodeUnicodeString(Buff));
+			}
+			if (ReadVCALText(Line, "X-GAMMU-MEMORY", Buff, false)) {
+				Pbk->MemoryType = GSM_StringToMemoryType(DecodeUnicodeString(Buff));
+			}
 			break;
 		}
 	}
 
-	if (Pbk->EntriesNum == 0) return ERR_EMPTY;
-	return ERR_NONE;
+vcard_complete:
+	if (Pbk->EntriesNum == 0) error = ERR_EMPTY;
+	else error = ERR_NONE;
+
+vcard_done:
+	free(Line);
+	return error;
 }
 
 /* How should editor hadle tabs in this file? Add editor commands here.
- * vim: noexpandtab sw=8 ts=8 sts=8:
+ * vim: noexpandtab sw=4 ts=4 sts=4 tw=79:
  */
