@@ -22,7 +22,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 {
 	unsigned char		Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
 	int			i,j,z;
-	long long			Length;
+	ssize_t			Length;
 	size_t EncodeLength;
 	unsigned int		Len;
 	size_t FreeText,FreeBytes,UsedText,Width,Height,x,y,Width2;
@@ -33,7 +33,6 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	GSM_Coding_Type 	Coding 	= SMS_Coding_Default_No_Compression;
 	GSM_Phone_Bitmap_Types	BitmapType;
 	GSM_MultiPartSMSEntry 	*Entry;
-	bool			start;
 	GSM_DateTime		Date;
 
 #ifdef DEBUG
@@ -191,7 +190,6 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			/* Checking number of SMS */
 			Used 		= 0;
 			FreeBytes 	= 0;
-			start		= true;
 			EncodeLength = Length;
 			while (1) {
 				if (FreeBytes != 0) {
@@ -217,7 +215,6 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				default:
 					break;
 				}
-				start = false;
 				Used++;
 			}
 			dbgprintf("Used SMS: " SIZE_T_FORMAT "\n",Used);
@@ -246,7 +243,6 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 
 			/* Saving */
 			FreeBytes = 0;
-			start	  = true;
 			EncodeLength = Length;
 			while (1) {
 				if (FreeBytes != 0) {
@@ -277,7 +273,6 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				Buffer[2] = 0x00; 	/* Position in EMS msg 	*/
 				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,EncodeLength+3,true,&UsedText,&CopiedText,&CopiedSMSText);
 				SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-EncodeLength-1] = UsedText;
-				start = false;
 			}
 
 			Entry->RingtoneNotes = Entry->Ringtone->NoteTone.NrCommands;
@@ -368,7 +363,8 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 
 				GSM_ResizeBitmap(&Bitmap, &Entry->Bitmap->Bitmap[0], Width, Height);
 #ifdef DEBUG
-				if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Bitmap);
+				if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+					GSM_PrintBitmap(GSM_global_debug.df,&Bitmap);
 #endif
 				PHONE_EncodeBitmap(BitmapType,Buffer+5, &Bitmap);
 				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,5+Length,true,&UsedText,&CopiedText,&CopiedSMSText);
@@ -420,7 +416,8 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			if (Length > 128) 	Length = 128;
 
 #ifdef DEBUG
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Bitmap);
+			if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+				GSM_PrintBitmap(GSM_global_debug.df,&Bitmap);
 #endif
 
 			/* Saving SMS */
@@ -445,7 +442,8 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 					}
 				}
 #ifdef DEBUG
-				if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Bitmap2);
+				if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+					GSM_PrintBitmap(GSM_global_debug.df,&Bitmap2);
 #endif
 
 				/* Adding new bitmap to SMS */
@@ -496,7 +494,8 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 
 			GSM_ResizeBitmap(&Bitmap, &Entry->Bitmap->Bitmap[0], Width, Height);
 #ifdef DEBUG
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Bitmap);
+			if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+				GSM_PrintBitmap(GSM_global_debug.df,&Bitmap);
 #endif
 			PHONE_EncodeBitmap(BitmapType,Buffer+5, &Bitmap);
 			GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,5+Length,true,&UsedText,&CopiedText,&CopiedSMSText);
@@ -532,9 +531,9 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	dbgprintf("SMS number is %i\n",SMS->Number);
 	for (i=0;i<SMS->Number;i++) {
 		dbgprintf("UDH length %i\n",SMS->SMS[i].UDH.Length);
-		DumpMessage(&di, SMS->SMS[i].UDH.Text, SMS->SMS[i].UDH.Length);
+		DumpMessage(&GSM_global_debug, SMS->SMS[i].UDH.Text, SMS->SMS[i].UDH.Length);
 		dbgprintf("SMS length " SIZE_T_FORMAT "\n",UnicodeLength(SMS->SMS[i].Text)*2);
-		DumpMessage(&di, SMS->SMS[i].Text, UnicodeLength(SMS->SMS[i].Text)*2);
+		DumpMessage(&GSM_global_debug, SMS->SMS[i].Text, UnicodeLength(SMS->SMS[i].Text)*2);
 	}
 #endif
 	return ERR_NONE;
@@ -583,7 +582,8 @@ static bool AddEMSText(GSM_SMSMessage *SMS, GSM_MultiPartSMSInfo *Info, int *Pos
 bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			       GSM_MultiSMSMessage 	*SMS)
 {
-	int  			i, w, Pos, UPI = 1, width, height;
+	int  			i, w, Pos, UPI = 1;
+	size_t			width, height;
 	size_t			z;
  	bool 			RetVal = false, NewPicture = true;
 	GSM_Phone_Bitmap_Types 	BitmapType;
@@ -724,17 +724,12 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 					z = Pos;
 					if (!AddEMSText(&SMS->SMS[i], Info, &Pos, SMS->SMS[i].UDH.Text[w+2]-z)) return false;
 				}
-				if (NewPicture) {
-					(Info->EntriesNum)++;
-					Info->Entries[Info->EntriesNum].Bitmap->Number = 0;
-					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapWidth  = 0;
-					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapHeight = 0;
-				}
 				Bitmap.BitmapWidth  = SMS->SMS[i].UDH.Text[w+3]*8;
 				Bitmap.BitmapHeight = SMS->SMS[i].UDH.Text[w+4];
 				Info->Entries[Info->EntriesNum].Bitmap = (GSM_MultiBitmap *)malloc(sizeof(GSM_MultiBitmap));
 				if (Info->Entries[Info->EntriesNum].Bitmap == NULL) return false;
 				if (NewPicture) {
+					Info->Entries[Info->EntriesNum].Bitmap->Number = 0;
 					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapWidth  = Bitmap.BitmapWidth;
 					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapHeight = Bitmap.BitmapHeight;
 					PHONE_DecodeBitmap(GSM_EMSVariablePicture,
