@@ -149,7 +149,8 @@ void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 			  int			Class,
 			  unsigned char		ReplaceMessage)
 {
-	size_t 		j,Len,UsedText,CopiedText,CopiedSMSText;
+	size_t 		Len,UsedText,CopiedText,CopiedSMSText;
+	int		j;
 	unsigned char 	UDHID;
 	GSM_DateTime 	Date;
 
@@ -277,8 +278,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 					size_t			Type)
 {
 	unsigned char 	buff[100],UDHID;
-	size_t 		i;
-	size_t p;
+	size_t		i, p;
 	GSM_UDHHeader	MyUDH;
 
 	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
@@ -322,7 +322,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 	/* Linked sms UDH */
 	if (SMS->Number != 1) {
 		UDHID = GSM_MakeSMSIDFromTime();
-		for (i=0;i<SMS->Number;i++) {
+		for (i = 0; i < (size_t)SMS->Number; i++) {
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-3] = UDHID;
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-2] = SMS->Number;
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-1] = i+1;
@@ -562,12 +562,14 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		NOKIA_EncodeWAPMMSSettingsSMSText(Buffer,&Length,Info->Entries[0].Settings,true);
 		break;
 	case SMS_NokiaVCARD10Long:
-		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+		error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+		if (error != ERR_NONE) return error;
 		/* is 1 SMS ? 8 = length of ..SCKE2 */
 		if (Length<=GSM_MAX_SMS_LENGTH-8) {
 			sprintf(Buffer,"//SCKE2 ");
 			Length = 8;
-			GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+			if (error != ERR_NONE) return error;
 		} else {
 			/* FIXME: It wasn't checked */
 			UDH = UDH_NokiaPhonebookLong;
@@ -577,12 +579,14 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_NokiaVCARD21Long:
-		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+		error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+		if (error != ERR_NONE) return error;
 		/* Is 1 SMS ? 12 = length of ..SCKL23F4 */
 		if (Length<=GSM_MAX_SMS_LENGTH-12) {
 			sprintf(Buffer,"//SCKL23F4%c%c",13,10);
 			Length = 12;
-			GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+			if (error != ERR_NONE) return error;
 		} else {
 			UDH = UDH_NokiaPhonebookLong;
 			/* Here can be also 8 bit coding */
@@ -592,27 +596,29 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_VCARD10Long:
-		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+		error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
+		if (error != ERR_NONE) return error;
 		if (Length>GSM_MAX_SMS_LENGTH) UDH = UDH_ConcatenatedMessages;
 		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_VCARD21Long:
-		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+		error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
+		if (error != ERR_NONE) return error;
 		if (Length>GSM_MAX_SMS_LENGTH) UDH = UDH_ConcatenatedMessages;
 		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_NokiaVCALENDAR10Long:
-		error=GSM_EncodeVCALENDAR(Buffer,&Length,Info->Entries[0].Calendar,true,Nokia_VCalendar);
+		error=GSM_EncodeVCALENDAR(Buffer, sizeof(Buffer),&Length,Info->Entries[0].Calendar,true,Nokia_VCalendar);
 		if (error != ERR_NONE) return error;
 		/* Is 1 SMS ? 8 = length of ..SCKE4 */
 		if (Length<=GSM_MAX_SMS_LENGTH-8) {
 			sprintf(Buffer,"//SCKE4 ");
 			Length = 8;
-			GSM_EncodeVCALENDAR(Buffer,&Length,Info->Entries[0].Calendar,true,Nokia_VCalendar);
+			GSM_EncodeVCALENDAR(Buffer, sizeof(Buffer),&Length,Info->Entries[0].Calendar,true,Nokia_VCalendar);
 		} else {
 			UDH = UDH_NokiaCalendarLong;
 			/* can be here 8 bit coding ? */
@@ -622,7 +628,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_NokiaVTODOLong:
-		error=GSM_EncodeVTODO(Buffer,&Length,Info->Entries[0].ToDo,true,Nokia_VToDo);
+		error=GSM_EncodeVTODO(Buffer, sizeof(Buffer),&Length,Info->Entries[0].ToDo,true,Nokia_VToDo);
 		if (error != ERR_NONE) return error;
 		UDH = UDH_NokiaCalendarLong;
 		Coding = SMS_Coding_Default_No_Compression;
@@ -670,11 +676,11 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		EncodeDefault(Buffer2, Buffer, &smslen, true, NULL);
 		DecodeDefault(Buffer,  Buffer2, smslen, true, NULL);
 #ifdef DEBUG
-		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) {
+		if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE) {
 			dbgprintf("Info->Entries[0].Buffer:\n");
-			DumpMessage(&di, Info->Entries[0].Buffer, UnicodeLength(Info->Entries[0].Buffer)*2);
+			DumpMessage(&GSM_global_debug, Info->Entries[0].Buffer, UnicodeLength(Info->Entries[0].Buffer)*2);
 			dbgprintf("Buffer:\n");
-			DumpMessage(&di, Buffer, UnicodeLength(Buffer)*2);
+			DumpMessage(&GSM_global_debug, Buffer, UnicodeLength(Buffer)*2);
 		}
 #endif
 		Info->UnicodeCoding = false;
@@ -847,7 +853,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 		Info->Entries[0].Bitmap->Number = 1;
 		PHONE_DecodeBitmap(GSM_NokiaCallerLogo, SMS->SMS[0].Text+4, &Info->Entries[0].Bitmap->Bitmap[0]);
 #ifdef DEBUG
-		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Info->Entries[0].Bitmap->Bitmap[0]);
+		if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+			GSM_PrintBitmap(GSM_global_debug.df,&Info->Entries[0].Bitmap->Bitmap[0]);
 #endif
 		Info->Entries[0].ID 	= SMS_NokiaCallerLogo;
 		Info->EntriesNum	= 1;
@@ -860,7 +867,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 		PHONE_DecodeBitmap(GSM_NokiaOperatorLogo, SMS->SMS[0].Text+7, &Info->Entries[0].Bitmap->Bitmap[0]);
 		NOKIA_DecodeNetworkCode(SMS->SMS[0].Text, Info->Entries[0].Bitmap->Bitmap[0].NetworkCode);
 #ifdef DEBUG
-		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Info->Entries[0].Bitmap->Bitmap[0]);
+		if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+			GSM_PrintBitmap(GSM_global_debug.df,&Info->Entries[0].Bitmap->Bitmap[0]);
 #endif
 		Info->Entries[0].ID 	= SMS_NokiaOperatorLogo;
 		Info->EntriesNum	= 1;
@@ -896,7 +904,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 				dbgprintf("OTA bitmap as Picture Image\n");
 				PHONE_DecodeBitmap(GSM_NokiaPictureImage, Buffer + i + 7, &Info->Entries[0].Bitmap->Bitmap[0]);
 #ifdef DEBUG
-				if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Info->Entries[0].Bitmap->Bitmap[0]);
+				if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+					GSM_PrintBitmap(GSM_global_debug.df,&Info->Entries[0].Bitmap->Bitmap[0]);
 #endif
 				break;
 			case SM30_RINGTONE:
@@ -912,7 +921,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 				dbgprintf("OTA bitmap as Screen Saver\n");
 				PHONE_DecodeBitmap(GSM_NokiaPictureImage, Buffer + i + 7, &Info->Entries[0].Bitmap->Bitmap[0]);
 #ifdef DEBUG
-				if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) GSM_PrintBitmap(di.df,&Info->Entries[0].Bitmap->Bitmap[0]);
+				if (GSM_global_debug.dl == DL_TEXTALL || GSM_global_debug.dl == DL_TEXTALLDATE)
+					GSM_PrintBitmap(GSM_global_debug.df,&Info->Entries[0].Bitmap->Bitmap[0]);
 #endif
 				Info->Entries[0].ID = SMS_NokiaScreenSaverLong;
 				break;
@@ -1002,7 +1012,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 {
 	bool			*INPUTSorted, copyit,OtherNumbers[GSM_SMS_OTHER_NUMBERS+1],wrong=false;
 	int			i,OUTPUTNum,z,w,m,p;
-	unsigned int		j;
+	int			j;
 	GSM_SiemensOTASMSInfo	SiemensOTA,SiemensOTA2;
 
 	i = 0;
@@ -1074,7 +1084,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 			INPUTSorted[i]	= true;
 			j		= 1;
 			/* We're searching for other parts in sequence */
-			while (j!=SiemensOTA.PacketsNum) {
+			while (j!=(int)SiemensOTA.PacketsNum) {
 				z=0;
 				while(INPUT[z]!=NULL) {
 					/* This was sorted earlier or is not single */
@@ -1087,7 +1097,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 						continue;
 					}
 					if (SiemensOTA2.SequenceID != SiemensOTA.SequenceID ||
-					    SiemensOTA2.PacketNum != j+1 ||
+					    (int)SiemensOTA2.PacketNum != j+1 ||
 					    SiemensOTA2.PacketsNum != SiemensOTA.PacketsNum ||
 					    strcmp(SiemensOTA2.DataType,SiemensOTA.DataType) ||
 					    strcmp(SiemensOTA2.DataName,SiemensOTA.DataName)) {
@@ -1251,8 +1261,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 			INPUTSorted[i]	= true;
 			j		= 1;
 			/* We're searching for other parts in sequence */
-			/* Typecasting to silent GCC warning, we should fit */
-			while (j != (unsigned int)INPUT[i]->SMS[0].UDH.AllParts) {
+			while (j != INPUT[i]->SMS[0].UDH.AllParts) {
 				z=0;
 				while(INPUT[z]!=NULL) {
 					/* This was sorted earlier or is not single */
@@ -1289,8 +1298,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 					if (INPUT[z]->SMS[0].UDH.ID8bit      != INPUT[i]->SMS[0].UDH.ID8bit	||
 							INPUT[z]->SMS[0].UDH.ID16bit     != INPUT[i]->SMS[0].UDH.ID16bit	||
 							INPUT[z]->SMS[0].UDH.AllParts    != INPUT[i]->SMS[0].UDH.AllParts 	||
-							/* Typecasting to silent GCC warning, we should fit */
-							(unsigned int)(INPUT[z]->SMS[0].UDH.PartNumber) != j + 1) {
+							(INPUT[z]->SMS[0].UDH.PartNumber) != j + 1) {
 						z++;
 						continue;
 					}
