@@ -97,14 +97,15 @@ static GSM_Error GSM_DecodeSMSDateTime(GSM_DateTime *DT, unsigned char *req)
 	DT->Second  = DecodeWithBCDAlphabet(req[5]);
 
 	/* Base for timezone is GMT. It's in quarters */
-	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))/4;
+	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))*3600/4;
 
 	if (req[6]&0x08) DT->Timezone = -DT->Timezone;
 
 	dbgprintf("Decoding date & time: ");
 	dbgprintf("%s %4d/%02d/%02d ", DayOfWeek(DT->Year, DT->Month, DT->Day),
 		DT->Year, DT->Month, DT->Day);
-	dbgprintf("%02d:%02d:%02d %02d00\n", DT->Hour, DT->Minute, DT->Second, DT->Timezone);
+	dbgprintf("%02d:%02d:%02d%+03i%02i\n", DT->Hour, DT->Minute, DT->Second,
+		DT->Timezone / 3600, abs((DT->Timezone % 3600) / 60));
 
 	return ERR_NONE;
 }
@@ -182,7 +183,7 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 		dbgprintf(", part %i of %i",UDH->PartNumber,UDH->AllParts);
 	}
 	dbgprintf("\n");
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, UDH->Text, UDH->Length);
+	DumpMessageText(&GSM_global_debug, UDH->Text, UDH->Length);
 #endif
 }
 
@@ -233,7 +234,7 @@ GSM_Coding_Type GSM_GetMessageCoding(const char TPDCS) {
 GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
 	int		off=0;	 	/* length of the User Data Header */
-	int 		w,i,tmp=0;
+	int 		w,i;
 	unsigned char	output[161];
 
 	SMS->UDH.Length = 0;
@@ -263,7 +264,7 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 				SMS->Length = 0;
 				break;
 			}
-			tmp=GSM_UnpackEightBitsToSeven(w, buffer[Layout.TPUDL]-off, SMS->Length, buffer+(Layout.Text+off), output);
+			GSM_UnpackEightBitsToSeven(w, buffer[Layout.TPUDL]-off, SMS->Length, buffer+(Layout.Text+off), output);
 			dbgprintf("7 bit SMS, length %i\n",SMS->Length);
 			DecodeDefault (SMS->Text, output, SMS->Length, true, NULL);
 			dbgprintf("%s\n",DecodeUnicodeString(SMS->Text));
@@ -273,7 +274,7 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 			memcpy(SMS->Text,buffer+(Layout.Text+off),SMS->Length);
 #ifdef DEBUG
 			dbgprintf("8 bit SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->Text, SMS->Length);
+			DumpMessageText(&GSM_global_debug, SMS->Text, SMS->Length);
 #endif
 			break;
 		case SMS_Coding_Unicode_No_Compression:
@@ -281,7 +282,7 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 			DecodeUnicodeSpecialNOKIAChars(SMS->Text,buffer+(Layout.Text+off), SMS->Length);
 #ifdef DEBUG
 			dbgprintf("Unicode SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, buffer+(Layout.Text+off), SMS->Length*2);
+			DumpMessageText(&GSM_global_debug, buffer+(Layout.Text+off), SMS->Length*2);
 			dbgprintf("%s\n",DecodeUnicodeString(SMS->Text));
 #endif
 			break;
@@ -490,7 +491,7 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 		memcpy(buffer+Layout.Text, SMS->UDH.Text, off);		/* we copy the udh */
 #ifdef DEBUG
 		dbgprintf("UDH, length %i\n",off);
-		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->UDH.Text, off);
+		DumpMessageText(&GSM_global_debug, SMS->UDH.Text, off);
 #endif
 	}
 	switch (SMS->Coding) {
@@ -498,12 +499,12 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 			/* the mask for the 8-bit data */
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
-			buffer[Layout.TPDCS] |= 0xf4;
+			buffer[Layout.TPDCS] |= (1 << 2);
 			memcpy(buffer+(Layout.Text+off), SMS->Text, SMS->Length);
 			size2 = size = SMS->Length+off;
 #ifdef DEBUG
 			dbgprintf("8 bit SMS, length %i\n",SMS->Length);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, SMS->Text, SMS->Length);
+			DumpMessageText(&GSM_global_debug, SMS->Text, SMS->Length);
 #endif
 			break;
 		case SMS_Coding_Default_No_Compression:
@@ -526,12 +527,12 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 		case SMS_Coding_Unicode_No_Compression:
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
-			buffer[Layout.TPDCS] |= 0x08;
+			buffer[Layout.TPDCS] |= (1 << 3);
 			EncodeUnicodeSpecialNOKIAChars(buffer+(Layout.Text+off), SMS->Text, UnicodeLength(SMS->Text));
 			size=size2=UnicodeLength(buffer+(Layout.Text+off))*2+off;
 #ifdef DEBUG
 			dbgprintf("Unicode SMS, length %i\n",(size2-off)/2);
-			if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, buffer+(Layout.Text+off), size2-off);
+			DumpMessageText(&GSM_global_debug, buffer+(Layout.Text+off), size2-off);
 			dbgprintf("%s\n",DecodeUnicodeString(buffer+(Layout.Text+off)));
 #endif
 			break;
@@ -591,7 +592,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	/* Message Class*/
 	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
 	if (Layout.TPDCS != 255) {
-		if (SMS->Class>=0 && SMS->Class<5) buffer[Layout.TPDCS] |= (240+SMS->Class);
+		if (SMS->Class >= 0 && SMS->Class <= 3) buffer[Layout.TPDCS] |= SMS->Class | (1 << 4);
 		dbgprintf("SMS class %i\n",SMS->Class);
 	}
 
