@@ -8,41 +8,58 @@ GSM_Error error;
 char buffer[100];
 
 /* Function to handle errors */
-void error_handler()
+void error_handler(void)
 {
 	if (error != ERR_NONE) {
-		printf("%s\n", GSM_ErrorString(error));
+		printf("ERROR: %s\n", GSM_ErrorString(error));
 		if (GSM_IsConnected(s))
 			GSM_TerminateConnection(s);
 		exit(error);
 	}
 }
 
-int main(int argc, char **argv)
+int main(int argc UNUSED, char **argv UNUSED)
 {
+	GSM_Debug_Info *debug_info;
+
+	/* Enable global debugging to stderr */
+	debug_info = GSM_GetGlobalDebug();
+	GSM_SetDebugFileDescriptor(stderr, false, debug_info);
+	GSM_SetDebugLevel("textall", debug_info);
+
 	/* Allocates state machine */
 	s = GSM_AllocStateMachine();
 	if (s == NULL)
 		return 3;
 
+	/*
+	 * Enable state machine debugging to stderr
+	 * Same could be achieved by just using global debug config.
+	 */
+	debug_info = GSM_GetDebug(s);
+	GSM_SetDebugGlobal(false, debug_info);
+	GSM_SetDebugFileDescriptor(stderr, false, debug_info);
+	GSM_SetDebugLevel("textall", debug_info);
+
 	/* Find configuration file */
 	error = GSM_FindGammuRC(&cfg);
 	error_handler();
 
 	/* Read it */
-	if (!GSM_ReadConfig(cfg, GSM_GetConfig(s, 0), 0))
-		return 2;
+	error = GSM_ReadConfig(cfg, GSM_GetConfig(s, 0), 0);
+	error_handler();
 
 	/* We have one valid configuration */
 	GSM_SetConfigNum(s, 1);
 
 	/* Connect to phone */
-	error = GSM_InitConnection(s, 3);	/* 3 means number of replies you want to wait for */
+	/* 3 means number of replies you want to wait for */
+	error = GSM_InitConnection(s, 3);
 	error_handler();
 
 	/* Here you can do some stuff with phone... */
 
-	/* As example we read some information about phone: */
+	/* As an example we read some information about phone: */
 
 	/* Manufacturer name */
 	error = GSM_GetManufacturer(s, buffer);
@@ -52,7 +69,9 @@ int main(int argc, char **argv)
 	/* Model name */
 	error = GSM_GetModel(s, buffer);
 	error_handler();
-	printf("Model         : %s (%s)\n", GSM_GetModelInfo(s)->model, buffer);
+	printf("Model         : %s (%s)\n",
+		GSM_GetModelInfo(s)->model,
+		buffer);
 
 	/* Terminate connection */
 	error = GSM_TerminateConnection(s);
