@@ -39,7 +39,8 @@ void SaveFile(int argc, char *argv[])
 			exit(-1);
 		}
 		j = 0;
-		GSM_EncodeVCALENDAR(Buffer, &j, Backup.Calendar[i],true,Nokia_VCalendar);
+		error = GSM_EncodeVCALENDAR(Buffer, sizeof(Buffer), &j, Backup.Calendar[i],true,Nokia_VCalendar);
+		Print_Error(error);
 	} else if (strcasecmp(argv[2],"BOOKMARK") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
@@ -75,7 +76,8 @@ void SaveFile(int argc, char *argv[])
 			exit(-1);
 		}
 		j = 0;
-		GSM_EncodeVNTFile(Buffer, &j, Backup.Note[i]);
+		error = GSM_EncodeVNTFile(Buffer, sizeof(Buffer), &j, Backup.Note[i]);
+		Print_Error(error);
 	} else if (strcasecmp(argv[2],"TODO") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("Where is backup filename and location?"));
@@ -93,7 +95,8 @@ void SaveFile(int argc, char *argv[])
 			exit(-1);
 		}
 		j = 0;
-		GSM_EncodeVTODO(Buffer, &j, Backup.ToDo[i], true, Nokia_VToDo);
+		error = GSM_EncodeVTODO(Buffer, sizeof(Buffer), &j, Backup.ToDo[i], true, Nokia_VToDo);
+		Print_Error(error);
 	} else if (strcasecmp(argv[2],"VCARD10") == 0 || strcasecmp(argv[2],"VCARD21") == 0) {
 		if (argc<6) {
 			printf("%s\n", _("Where is backup filename and location and memory type?"));
@@ -128,9 +131,11 @@ void SaveFile(int argc, char *argv[])
 		}
 		j = 0;
 		if (strcasecmp(argv[2],"VCARD10") == 0) {
-			GSM_EncodeVCARD(Buffer,&j,pbk,true,Nokia_VCard10);
+			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard10);
+			Print_Error(error);
 		} else {
-			GSM_EncodeVCARD(Buffer,&j,pbk,true,Nokia_VCard21);
+			error = GSM_EncodeVCARD(Buffer, sizeof(Buffer), &j, pbk, true, Nokia_VCard21);
+			Print_Error(error);
 		}
 	} else {
 		printf(_("Unknown backup format: \"%s\"\n"), argv[2]);
@@ -171,6 +176,7 @@ void DoBackup(int argc, char *argv[])
  	GSM_FMStation		FMStation;
  	GSM_GPRSAccessPoint	GPRSPoint;
 	bool			DoBackupPart;
+	bool			UseNext;
 	char buffer[GSM_MAX_INFO_LENGTH];
 
 	if (argc == 4 && strcasecmp(argv[3],"-yes") == 0) always_answer_yes = true;
@@ -229,39 +235,84 @@ void DoBackup(int argc, char *argv[])
 	if (Info.PhonePhonebook) {
 		printf("%s\n", _("Checking phone phonebook"));
 		MemStatus.MemoryType = MEM_ME;
-		error=GSM_GetMemoryStatus(gsm, &MemStatus);
-		if (error==ERR_NONE && MemStatus.MemoryUsed != 0) {
+		Pbk.MemoryType  = MEM_ME;
+		Pbk.Location = 0;
+		UseNext = false;
+		error = GSM_GetMemoryStatus(gsm, &MemStatus);
+		if (error == ERR_NONE) {
+			if (MemStatus.MemoryUsed != 0) {
+				DoBackupPart = true;
+			}
+		} else {
+			UseNext = true;
+			error = GSM_GetNextMemory(gsm, &Pbk, true);
+			if (error == ERR_NONE) {
+				DoBackupPart = true;
+			}
+
+		}
+		if (DoBackupPart) {
 			if (answer_yes("   %s", _("Backup phone phonebook?"))) DoBackupPart = true;
+			else DoBackupPart = false;
 		}
 	}
 	if (DoBackupPart) {
-		Pbk.MemoryType  = MEM_ME;
-		i		= 1;
-		used 		= 0;
-		while (used != MemStatus.MemoryUsed) {
-			Pbk.Location = i;
-			error=GSM_GetMemory(gsm, &Pbk);
-			if (error != ERR_EMPTY) {
-				Print_Error(error);
+		if (UseNext) {
+			used 		= 0;
+			fprintf(stderr, LISTFORMAT, _("Reading"));
+			while (error == ERR_NONE) {
 				if (used < GSM_BACKUP_MAX_PHONEPHONEBOOK) {
 					Backup.PhonePhonebook[used] = malloc(sizeof(GSM_MemoryEntry));
-				        if (Backup.PhonePhonebook[used] == NULL) Print_Error(ERR_MOREMEMORY);
+					if (Backup.PhonePhonebook[used] == NULL) Print_Error(ERR_MOREMEMORY);
 					Backup.PhonePhonebook[used+1] = NULL;
 				} else {
-					printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_PHONEPHONEBOOK");
+					printf("\n   ");
+					printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_PHONEPHONEBOOK");
+					printf("\n");
 					break;
 				}
 				*Backup.PhonePhonebook[used]=Pbk;
 				used++;
+				error = GSM_GetNextMemory(gsm, &Pbk, false);
+				fprintf(stderr, "*");
+				if (gshutdown) {
+					GSM_Terminate();
+					exit(0);
+				}
 			}
-			fprintf(stderr, _("%c   Reading: %i percent"),13,used*100/MemStatus.MemoryUsed);
-			i++;
-			if (gshutdown) {
-				GSM_Terminate();
-				exit(0);
+			fprintf(stderr, "\n");
+		} else {
+			Pbk.MemoryType  = MEM_ME;
+			i		= 1;
+			used 		= 0;
+			while (used != MemStatus.MemoryUsed) {
+				Pbk.Location = i;
+				error=GSM_GetMemory(gsm, &Pbk);
+				if (error != ERR_EMPTY) {
+					Print_Error(error);
+					if (used < GSM_BACKUP_MAX_PHONEPHONEBOOK) {
+						Backup.PhonePhonebook[used] = malloc(sizeof(GSM_MemoryEntry));
+						if (Backup.PhonePhonebook[used] == NULL) Print_Error(ERR_MOREMEMORY);
+						Backup.PhonePhonebook[used+1] = NULL;
+					} else {
+						printf("\n   ");
+						printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_PHONEPHONEBOOK");
+						printf("\n");
+						break;
+					}
+					*Backup.PhonePhonebook[used]=Pbk;
+					used++;
+				}
+				fprintf(stderr, "\r   ");
+				fprintf(stderr, _("Reading: %i percent"), used*100/MemStatus.MemoryUsed);
+				i++;
+				if (gshutdown) {
+					GSM_Terminate();
+					exit(0);
+				}
 			}
+			fprintf(stderr, "\n");
 		}
-		fprintf(stderr, "\n");
 	}
 	DoBackupPart = false;
 	if (Info.SIMPhonebook) {
@@ -286,13 +337,16 @@ void DoBackup(int argc, char *argv[])
 				        if (Backup.SIMPhonebook[used] == NULL) Print_Error(ERR_MOREMEMORY);
 					Backup.SIMPhonebook[used + 1] = NULL;
 				} else {
-					printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_SIMPHONEBOOK");
+					printf("\n   ");
+					printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_SIMPHONEBOOK");
+					printf("\n");
 					break;
 				}
 				*Backup.SIMPhonebook[used]=Pbk;
 				used++;
 			}
-			fprintf(stderr, _("%c   Reading: %i percent"),13,used*100/MemStatus.MemoryUsed);
+			fprintf(stderr, "\r   ");
+			fprintf(stderr, _("Reading: %i percent"), used*100/MemStatus.MemoryUsed);
 			i++;
 			if (gshutdown) {
 				GSM_Terminate();
@@ -318,7 +372,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.Calendar[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.Calendar[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_MAXCALENDARTODONOTES");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_MAXCALENDARTODONOTES");
+				printf("\n");
 				break;
 			}
 			*Backup.Calendar[used]=Calendar;
@@ -349,13 +405,16 @@ void DoBackup(int argc, char *argv[])
 				if (Backup.ToDo[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.ToDo[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_MAXCALENDARTODONOTES");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_MAXCALENDARTODONOTES");
+				printf("\n");
 				break;
 			}
 			*Backup.ToDo[used]=ToDo;
 			used ++;
 			error=GSM_GetNextToDo(gsm,&ToDo,false);
-			fprintf(stderr, _("%c   Reading: %i percent"),13,used*100/ToDoStatus.Used);
+			fprintf(stderr, "\r   ");
+			fprintf(stderr, _("Reading: %i percent"), used*100/ToDoStatus.Used);
 			if (gshutdown) {
 				GSM_Terminate();
 				exit(0);
@@ -380,7 +439,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.Note[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.Note[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_NOTE");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_NOTE");
+				printf("\n");
 				break;
 			}
 			*Backup.Note[used]=Note;
@@ -414,7 +475,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.CallerLogos[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.CallerLogos[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_CALLER");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_CALLER");
+				printf("\n");
 				break;
 			}
 			*Backup.CallerLogos[used] = Bitmap;
@@ -446,7 +509,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.SMSC[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.SMSC[used + 1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_SMSC");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_SMSC");
+				printf("\n");
 				break;
 			}
 			*Backup.SMSC[used]=SMSC;
@@ -508,7 +573,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.WAPBookmark[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.WAPBookmark[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_WAPBOOKMARK");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_WAPBOOKMARK");
+				printf("\n");
 				break;
 			}
 			*Backup.WAPBookmark[used]=Bookmark;
@@ -541,7 +608,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.WAPSettings[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.WAPSettings[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_WAPSETTINGS");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_WAPSETTINGS");
+				printf("\n");
 				break;
 			}
 			*Backup.WAPSettings[used]=Settings;
@@ -574,7 +643,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.MMSSettings[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.MMSSettings[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_MMSSETTINGS");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_MMSSETTINGS");
+				printf("\n");
 				break;
 			}
 			*Backup.MMSSettings[used]=Settings;
@@ -607,7 +678,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.ChatSettings[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.ChatSettings[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_CHATSETTINGS");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_CHATSETTINGS");
+				printf("\n");
 				break;
 			}
 			*Backup.ChatSettings[used]=Chat;
@@ -640,7 +713,9 @@ void DoBackup(int argc, char *argv[])
 			        if (Backup.SyncMLSettings[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.SyncMLSettings[used+1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_SYNCMLSETTINGS");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_SYNCMLSETTINGS");
+				printf("\n");
 				break;
 			}
 			*Backup.SyncMLSettings[used]=SyncML;
@@ -676,7 +751,9 @@ void DoBackup(int argc, char *argv[])
 				        if (Backup.Ringtone[used] == NULL) Print_Error(ERR_MOREMEMORY);
 					Backup.Ringtone[used+1] = NULL;
 				} else {
-					printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_RINGTONES");
+					printf("\n   ");
+					printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_RINGTONES");
+					printf("\n");
 					break;
 				}
 				*Backup.Ringtone[used]=Ringtone;
@@ -715,7 +792,9 @@ void DoBackup(int argc, char *argv[])
 				if (Backup.Profiles[used] == NULL) Print_Error(ERR_MOREMEMORY);
 				Backup.Profiles[used + 1] = NULL;
 			} else {
-				printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_PROFILES");
+				printf("\n   ");
+				printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_PROFILES");
+				printf("\n");
 				break;
 			}
 			*Backup.Profiles[used]=Profile;
@@ -745,7 +824,9 @@ void DoBackup(int argc, char *argv[])
 					if (Backup.FMStation[used] == NULL) Print_Error(ERR_MOREMEMORY);
  					Backup.FMStation[used + 1] = NULL;
  				} else {
-					printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_FMSTATIONS");
+					printf("\n   ");
+					printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_FMSTATIONS");
+					printf("\n");
 					break;
  				}
  				*Backup.FMStation[used]=FMStation;
@@ -778,7 +859,9 @@ void DoBackup(int argc, char *argv[])
 					if (Backup.GPRSPoint[used] == NULL) Print_Error(ERR_MOREMEMORY);
  					Backup.GPRSPoint[used + 1] = NULL;
  				} else {
-					printf(_("\n   Only part of data saved - increase %s") , "GSM_BACKUP_MAX_GPRSPOINT");
+					printf("\n   ");
+					printf(_("Only part of data saved, please increase %s.") , "GSM_BACKUP_MAX_GPRSPOINT");
+					printf("\n");
 					break;
  				}
  				*Backup.GPRSPoint[used]=GPRSPoint;
@@ -1649,30 +1732,29 @@ void RestoreSMS(int argc UNUSED, char *argv[])
 	GSM_SMS_Backup		Backup;
 	GSM_SMSFolders		folders;
 	int			smsnum = 0;
-	bool			restore8bit,doit;
+	bool			restore8bit;
 
-	error=GSM_ReadSMSBackupFile(argv[2], &Backup);
+	error = GSM_ReadSMSBackupFile(argv[2], &Backup);
 	Print_Error(error);
 
 	restore8bit = answer_yes(_("Do you want to restore binary SMS?"));
 
 	GSM_Init(true);
 
-	error=GSM_GetSMSFolders(gsm, &folders);
+	error = GSM_GetSMSFolders(gsm, &folders);
 	Print_Error(error);
 
 	while (Backup.SMS[smsnum] != NULL) {
-		doit = true;
-		if (!restore8bit && Backup.SMS[smsnum]->Coding == SMS_Coding_8bit) doit = false;
-		if (doit) {
+		if (restore8bit || Backup.SMS[smsnum]->Coding != SMS_Coding_8bit) {
 			SMS.Number = 1;
 			memcpy(&SMS.SMS[0],Backup.SMS[smsnum],sizeof(GSM_SMSMessage));
 			DisplayMultiSMSInfo(&SMS,false,false,NULL);
 			if (answer_yes(_("Restore %03i sms to folder \"%s\"%s?"),
-				smsnum+1,DecodeUnicodeConsole(folders.Folder[Backup.SMS[smsnum]->Folder-1].Name),
-				folders.Folder[Backup.SMS[smsnum]->Folder-1].Memory == MEM_SM ? _(" (SIM)") : "")) {
+					smsnum + 1,
+					DecodeUnicodeConsole(folders.Folder[Backup.SMS[smsnum]->Folder - 1].Name),
+					folders.Folder[Backup.SMS[smsnum]->Folder - 1].Memory == MEM_SM ? _(" (SIM)") : "")) {
 				smprintf(gsm, _("saving %i SMS\n"),smsnum);
-				error=GSM_AddSMS(gsm, Backup.SMS[smsnum]);
+				error = GSM_AddSMS(gsm, Backup.SMS[smsnum]);
 				Print_Error(error);
 			}
 		}
