@@ -8,6 +8,10 @@
 #include "common.h"
 #include "formats.h"
 
+#ifdef CURL_FOUND
+#include <curl/curl.h>
+#endif
+
 GSM_StateMachine *gsm;
 
 bool always_answer_yes = false;
@@ -71,7 +75,7 @@ void interrupt(int sign)
 	gshutdown = true;
 }
 
-void PrintSecurityStatus()
+void PrintSecurityStatus(void)
 {
 	GSM_SecurityCodeType Status;
 	GSM_Error error;
@@ -188,6 +192,28 @@ const char *GetDayName(const int day)
 	return result;
 }
 
+void Cleanup(void)
+{
+	GSM_Debug_Info *di;
+
+	/* Disconnect from phone */
+	if (GSM_IsConnected(gsm)) {
+		GSM_TerminateConnection(gsm);
+	}
+
+	/* Free state machine */
+	GSM_FreeStateMachine(gsm);
+
+	/* Close debug output if opened */
+	di = GSM_GetGlobalDebug();
+	GSM_SetDebugFileDescriptor(NULL, false, di);
+
+#ifdef CURL_FOUND
+	/* Free CURL memory */
+	curl_global_cleanup();
+#endif
+}
+
 void Print_Error(GSM_Error error)
 {
 	if (error != ERR_NONE) {
@@ -198,50 +224,68 @@ void Print_Error(GSM_Error error)
 			PrintSecurityStatus();
 		}
 
-		/* Disconnect from phone */
-		if (GSM_IsConnected(gsm)) {
-			GSM_TerminateConnection(gsm);
-		}
-
-		/* Free state machine */
-		GSM_FreeStateMachine(gsm);
+		Cleanup();
 
 		exit(-1);
 	}
 }
 
 /**
- * Converts memory type from string.
- *
- * \param type String with memory type.
- *
- * \return Parsed memory type or 0 on failure.
+ * Callback from CURL to get data.
  */
-GSM_MemoryType MemoryTypeFromString(const char *type)
-{
-	if (strcasecmp(type, "ME") == 0)
-		return MEM_ME;
-	if (strcasecmp(type, "SM") == 0)
-		return MEM_SM;
-	if (strcasecmp(type, "ON") == 0)
-		return MEM_ON;
-	if (strcasecmp(type, "DC") == 0)
-		return MEM_DC;
-	if (strcasecmp(type, "RC") == 0)
-		return MEM_RC;
-	if (strcasecmp(type, "MC") == 0)
-		return MEM_MC;
-	if (strcasecmp(type, "MT") == 0)
-		return MEM_MT;
-	if (strcasecmp(type, "FD") == 0)
-		return MEM_FD;
-	if (strcasecmp(type, "VM") == 0)
-		return MEM_VM;
-	if (strcasecmp(type, "SL") == 0)
-		return MEM_SL;
+size_t write_mem(void *ptr, size_t size, size_t nmemb, void *data) {
+	size_t realsize = size * nmemb;
+	GSM_File *file = (GSM_File *)data;
+
+	file->Buffer = realloc(file->Buffer,file->Used + realsize + 1);
+
+	if (file->Buffer) {
+		memcpy(file->Buffer + file->Used, ptr, realsize);
+		file->Used += realsize;
+		file->Buffer[file->Used] = 0;
+		return realsize;
+	}
 	return 0;
 }
 
+/**
+ * Downloads file from arbitrary URL.
+ *
+ * \param url URL to download.
+ * \param file Storage for data.
+ *
+ * \returns True on success.
+ */
+bool GSM_ReadHTTPFile(const char *url, GSM_File *file)
+{
+#ifdef CURL_FOUND
+	CURL *dl_handle = NULL;
+	CURLcode result;
+
+	dl_handle = curl_easy_init();
+	if (dl_handle == NULL) return false;
+
+	curl_easy_setopt(dl_handle, CURLOPT_URL, url);
+	curl_easy_setopt(dl_handle, CURLOPT_USERAGENT, "Gammu/" VERSION);
+	curl_easy_setopt(dl_handle, CURLOPT_WRITEFUNCTION, write_mem);
+	curl_easy_setopt(dl_handle, CURLOPT_WRITEDATA, file);
+	curl_easy_setopt(dl_handle, CURLOPT_FOLLOWLOCATION, 1);
+	curl_easy_setopt(dl_handle, CURLOPT_MAXREDIRS, 10);
+#if 0
+	/* Enable verbose outpuf from CURL */
+	curl_easy_setopt(dl_handle, CURLOPT_VERBOSE, 1);
+#endif
+
+	result = curl_easy_perform(dl_handle);
+
+	curl_easy_cleanup(dl_handle);
+
+	return (result == 0) ? true : false;
+#else
+	return false;
+#endif
+}
+
 /**
  * Initiates connection to phone.
  *
@@ -252,7 +296,7 @@ void GSM_Init(bool checkerror)
 	GSM_File PhoneDB;
 	char model[GSM_MAX_MODEL_LENGTH];
 	char current_version[GSM_MAX_VERSION_LENGTH];
-	char buff[50 + GSM_MAX_MODEL_LENGTH];
+	char url[70 + GSM_MAX_MODEL_LENGTH];
 	char latest_version[GSM_MAX_VERSION_LENGTH];
 	size_t pos = 0, oldpos = 0, i;
 	GSM_Error error;
@@ -285,9 +329,10 @@ void GSM_Init(bool checkerror)
 	latest_version[0] = 0;
 
 	/* Request information from phone db */
-	sprintf(buff, "support/phones/phonedbxml.php?model=%s", model);
+	sprintf(url, "http://www.gammu.org/support/phones/phonedbxml.php?model=%s", model);
 	PhoneDB.Buffer = NULL;
-	if (!GSM_ReadHTTPFile("www.gammu.org", buff, &PhoneDB))
+	PhoneDB.Used = 0;
+	if (!GSM_ReadHTTPFile(url, &PhoneDB))
 		return;
 
 	/* Parse reply */
