@@ -17,7 +17,7 @@ extern GSM_Reply_Function UserReplyFunctions3[];
 
 /* ------- some usefull functions ----------------------------------------- */
 
-GSM_Error CheckDCT3Only()
+GSM_Error CheckDCT3Only(void)
 {
 bool found = false;
 
@@ -42,7 +42,7 @@ if (strstr(N6110Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found =
 	return ERR_NONE;
 }
 
-static void CheckDCT3()
+static void CheckDCT3(void)
 {
 	GSM_Error error;
 
@@ -60,7 +60,7 @@ static void CheckDCT3()
 	}
 }
 
-static bool answer_yes3(char *text)
+static bool answer_yes3(const char *text)
 {
     	int         len;
     	char        ans[99];
@@ -206,7 +206,7 @@ void DCT3SelfTests(int argc, char *argv[])
 			GSM_Terminate();
 		}
 
-		my_sleep(400);
+		usleep(400000);
 	}
 
 	gsm->User.UserReplyFunctions=UserReplyFunctions3;
@@ -218,8 +218,8 @@ void DCT3SelfTests(int argc, char *argv[])
 }
 
 struct DCT3ADCInfo {
-	char 	*name;
-	char 	*unit;
+	const char 	*name;
+	const char 	*unit;
 	int 	x;
 	int	pos1;
 	int	pos2;
@@ -772,7 +772,7 @@ void DCT3DisplayOutput(int argc, char *argv[])
 
 	while (!gshutdown) {
 		GSM_ReadDevice(gsm,true);
-		my_sleep(10);
+		usleep(10000);
 	}
 
 	req[4] = 0x02;
