@@ -34,7 +34,7 @@ extern GSM_Reply_Function UserReplyFunctionsX[];
 GSMDecoder 		*gsmdec;
 struct wmx_tracestruct 	*traces;
 
-static GSM_Error DCT3_ReplySwitchDebug(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplySwitchDebug(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	switch(msg.Buffer[2]) {
 	case 0x70:
@@ -50,7 +50,7 @@ static GSM_Error DCT3_ReplySwitchDebug(GSM_Protocol_Message msg, GSM_StateMachin
 /**
  * RPC confirmation/reply
  */
-static GSM_Error DCT3_ReplyRPC(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyRPC(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("RPC Reply ");
 	printf("call=%02x rettype=%02x data=", msg.Buffer[2], msg.Buffer[3]);
@@ -155,12 +155,12 @@ static void mdircv_data(unsigned char type, unsigned char *buffer, size_t length
 	}
 }
 
-static GSM_Error DCT3_ReplyDebugTrace(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyDebugTrace(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-	int 			x;
+	size_t 			x;
 	int 			id,timestamp,number,length;
-	struct wmx_tracetype 	*minor;
-	char 			*desc;
+	struct wmx_tracetype 	*wmx_minor;
+	const char 			*desc;
 
 	//printf("Debug Trace Received\n");
 	/* parse frame
@@ -187,8 +187,8 @@ static GSM_Error DCT3_ReplyDebugTrace(GSM_Protocol_Message msg, GSM_StateMachine
 	/* Query trace type name */
 	desc = "Unknown";
 	if(traces != NULL) {
-		minor = wmx_tracestruct_queryminor(traces, id);
-		if(minor != NULL) desc = minor->desc;
+		wmx_minor = wmx_tracestruct_queryminor(traces, id);
+		if(wmx_minor != NULL) desc = wmx_minor->desc;
 	}
 	printf("<%04X> %s\n", id, desc);
 	printf("t=%04x nr=%02x: ", timestamp, number);
@@ -288,9 +288,9 @@ static GSM_Error DCT3_ReplyDebugTrace(GSM_Protocol_Message msg, GSM_StateMachine
 }
 
 
-static GSM_Error DCT3_ReplyMyPacket(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT3_ReplyMyPacket(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-	int x;
+	size_t x;
 
 	printf("MyPacket ");
 	for(x=0; x<msg.Length; x++) {
@@ -350,6 +350,7 @@ void DCT3SetDebug(int argc, char *argv[])
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		};
+	FILE *xout;
 
 	#define ENABLE_BIT(bit,verbose) reqEnable[3 + (bit>>3)] |= 1<<(7-(bit&7)); if(verbose){reqEnable[3 + 32 + (bit>>3)] |= 1<<(7-(bit&7));}
 
@@ -362,10 +363,12 @@ void DCT3SetDebug(int argc, char *argv[])
 
 	gsmdec = GSMDecoder_new();
 	/* Open XML file .. needs to be argument */
-	{
-		FILE *xout = fopen("out.xml", "w");
-		GSMDecoder_xmlout(gsmdec, xout);
+	xout = fopen("out.xml", "w");
+	if (xout == NULL) {
+		return;
 	}
+	GSMDecoder_xmlout(gsmdec, xout);
+
 	printf("Debug Trace Mode -- wumpus 2003\n");
 	traces = wmx_tracestruct_load(argv[2]);
 	if(traces == NULL)
@@ -416,17 +419,17 @@ void DCT3SetDebug(int argc, char *argv[])
 	/* We Need DCT3 */
 	if (CheckDCT3Only()!=ERR_NONE) return;
 
-	error=DCT3_EnableSecurity (s, 0x01);
+	error=DCT3_EnableSecurity (gsm, 0x01);
 	Print_Error(error);
 
-	s->User.UserReplyFunctions=UserReplyFunctionsX;
+	gsm->User.UserReplyFunctions=UserReplyFunctionsX;
 
-	//error=GSM_WaitFor (s, reqTest, sizeof(reqTest), 0x40, 1, ID_DebugSwitch);
+	//error=GSM_WaitFor (gsm, reqTest, sizeof(reqTest), 0x40, 1, ID_DebugSwitch);
 
-	//error=GSM_WaitFor (s, reqTest2, sizeof(reqTest2), 0xD1, 4, ID_RPC);
+	//error=GSM_WaitFor (gsm, reqTest2, sizeof(reqTest2), 0xD1, 4, ID_RPC);
 
 	/* Enable Debug Mode */
-	error=GSM_WaitFor (s, reqEnable, sizeof(reqEnable), 0x40, 4, ID_DebugSwitch);
+	error=GSM_WaitFor (gsm, reqEnable, sizeof(reqEnable), 0x40, 4, ID_DebugSwitch);
 
 	Print_Error(error);
 	signal(SIGINT, interrupt);
@@ -436,13 +439,13 @@ void DCT3SetDebug(int argc, char *argv[])
 	/*
 	while(x<100) {
 		//printf(": %02x\n",x);
-		s->Phone.Data.RequestID	= ID_DebugTrace;
-		res = s->Device.Functions->ReadDevice(s, buff, 255);
+		gsm->Phone.Data.RequestID	= ID_DebugTrace;
+		res = gsm->Device.Functions->ReadDevice(gsm, buff, 255);
 		if(res) {
 			printf("%02x\n",x);
 			for(y=0;y<res;y++) {
 				//printf("%02x\n",x,buff[y]&0xFF);
-				s->Protocol.Functions->StateMachine(s,buff[y]);
+				gsm->Protocol.Functions->StateMachine(gsm,buff[y]);
 				x++;
 			}
 		}
@@ -452,12 +455,12 @@ void DCT3SetDebug(int argc, char *argv[])
 
 	/* todo: wait and dump for some time */
 	while (!gshutdown) {
-		GSM_ReadDevice(s,true);
-		my_sleep(10);
+		GSM_ReadDevice(gsm,true);
+		usleep(10000);
 	}
 	signal(SIGINT, SIG_DFL);
 	printf("Disabling\n");
-	error=GSM_WaitFor (s, reqDisable, sizeof(reqDisable), 0x40, 10, ID_DebugSwitch);
+	error=GSM_WaitFor (gsm, reqDisable, sizeof(reqDisable), 0x40, 10, ID_DebugSwitch);
 	Print_Error(error);
 
 	GSMDecoder_free(gsmdec);
