@@ -24,7 +24,7 @@ extern GSM_Reply_Function UserReplyFunctions4[];
 
 /* ------- some usefull functions ----------------------------------------- */
 
-GSM_Error CheckDCT4Only()
+GSM_Error CheckDCT4Only(void)
 {
 	bool found = false;
 
@@ -49,7 +49,7 @@ GSM_Error CheckDCT4Only()
 	return ERR_NONE;
 }
 
-static void CheckDCT4()
+static void CheckDCT4(void)
 {
 	GSM_Error error;
 
@@ -67,7 +67,7 @@ static void CheckDCT4()
 	}
 }
 
-static bool answer_yes2(char *text)
+static bool answer_yes2(const char *text)
 {
     	int         len;
     	char        ans[99];
@@ -465,7 +465,7 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 		GSM_GetCurrentDateTime (&Date);
 		j=Date.Second;
 		while (j==Date.Second) {
-			my_sleep(10);
+			usleep(10000);
 			GSM_GetCurrentDateTime(&Date);
 		}
 	}
@@ -607,11 +607,10 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
  		sm->Phone.Data.PhoneString[1] = msg.Buffer[12];
  		sm->Phone.Data.PhoneString[2] = msg.Buffer[13];
 		return ERR_NONE;
-		break;
 	case 0x31:
 		dbgprintf("Names of voice records received\n");
 		j = 33;
-		for (i=0;i<msg.Buffer[9];i++) {
+		for (i = 0; i < (size_t)msg.Buffer[9]; i++) {
 			memcpy(Buffer,msg.Buffer+(j+1),msg.Buffer[j]);
 			Buffer[msg.Buffer[j]] 	= 0;
 			Buffer[msg.Buffer[j]+1] = 0;
@@ -1021,8 +1020,8 @@ static GSM_Error DCT4_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *sm
 }
 
 struct DCT4ADCInfo {
-	char 	*name;
-	char 	*unit;
+	const char 	*name;
+	const char 	*unit;
 	int 	x;
 };
 
