@@ -20,7 +20,7 @@ extern GSM_Error  ATGEN_SetMemory (GSM_StateMachine *sm, GSM_MemoryEntry *pbk);
 extern GSM_Reply_Function UserReplyFunctionsAtS[];
 
 bool 	new_variable;
-GSM_Error CheckSiemens()
+GSM_Error CheckSiemens(void)
 {
 	if (gsm->Phone.Data.Priv.ATGEN.Manufacturer != AT_Siemens) return ERR_NOTSUPPORTED;
 	return ERR_NONE;
@@ -232,9 +232,10 @@ GSM_Error ATSIEMENS_GetSAT(GSM_StateMachine *sm)
 {
     	GSM_Phone_ATGENData	*Priv = &(sm->Phone.Data.Priv.ATGEN);
     	GSM_Error		error;
-    	unsigned char		*reqSAT[]= {"D009810301260082028182",
+    	const char		*reqSAT[]= {"D009810301260082028182",
 				    	    "D009810301260282028182",
-				    	    "D009810301260582028182"},req[32];
+				    	    "D009810301260582028182"};
+	char req[32];
     	int 			i,len;
 
     	if (Priv->Manufacturer!=AT_Siemens) return ERR_NOTSUPPORTED;
