@@ -28,7 +28,7 @@
 /**
  * Displays status of filesystem (if available).
  */
-GSM_Error PrintFileSystemStatus()
+GSM_Error PrintFileSystemStatus(void)
 {
 	GSM_FileSystemStatus Status;
 	GSM_Error error;
@@ -142,7 +142,7 @@ void GetFileSystem(int argc, char *argv[])
 									  false));
 						} else
 							printf(" %30c", 0x20);
-						printf(" %9lli", (long long)Files.Used);
+						printf(" %9li", (long)Files.Used);
 						printf(" ");
 					} else
 						printf("|-- ");
@@ -557,7 +557,7 @@ void GetFileFolder(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-void AddOneFile(GSM_File * File, char *text, bool send)
+void AddOneFile(GSM_File * File, const char *text, const bool send)
 {
 	GSM_Error error;
 	int Pos, Handle, i, j, old1;
