@@ -10,7 +10,6 @@
 #include <signal.h>
 #include <ctype.h>
 #include <wchar.h>
-#include <unistd.h>
 
 #include <gammu.h>
 #include <gammu-config.h>
@@ -45,8 +44,16 @@
 #  include <sys/ioctl.h>
 #endif
 
+#ifdef CURL_FOUND
+#include <curl/curl.h>
+#endif
+
 #define ALL_MEMORY_TYPES "DC|MC|RC|ON|VM|SM|ME|MT|FD|SL"
 
+#define RSS_URL "http://blog.cihar.com/archives/gammu_releases/index-rss.xml"
+#define RSS_STABLE_STRING "Gammu stable version "
+#define RSS_TESTING_STRING "Gammu test version "
+
 #ifdef DEBUG
 static void MakeConvertTable(int argc UNUSED, char *argv[])
 {
@@ -121,7 +128,7 @@ static void ListNetworks(int argc, char *argv[])
 	}
 }
 
-static void PrintVersion()
+static void PrintVersion(void)
 {
 	printf(_("[Gammu version %s built %s on %s using %s]"),
 	       VERSION, __TIME__, __DATE__, GetCompiler());
@@ -256,6 +263,9 @@ static void Features(int argc UNUSED, char *argv[]UNUSED)
 #ifdef ICONV_FOUND
 	printf("  - %s\n", "ICONV");
 #endif
+#ifdef CURL_FOUND
+	printf("  - %s\n", "CURL");
+#endif
 #ifdef HAVE_MYSQL_MYSQL_H
 	printf("  - %s\n", "MYSQL");
 #endif
@@ -264,6 +274,93 @@ static void Features(int argc UNUSED, char *argv[]UNUSED)
 #endif
 }
 
+/**
+ * Converts version string to integer to allow comparing.
+ */
+int VersionToInt(const char *Buffer)
+{
+	size_t retval = 0, pos = 0;
+
+	retval = atoi(Buffer) * 10000;
+	while (Buffer[pos] != '.') {
+		pos++;
+		if (pos == strlen(Buffer))
+			return retval;
+	}
+	pos++;
+	retval += atoi(Buffer + pos) * 100;
+	while (Buffer[pos] != '.') {
+		pos++;
+		if (pos == strlen(Buffer))
+			return retval;
+	}
+	pos++;
+	return retval + atoi(Buffer + pos);
+}
+
+/**
+ * Check whether there is not newer version available online.
+ */
+static void CheckVersion(int argc, char *argv[])
+{
+	GSM_File RSS;
+	bool checkstable = true;
+	bool checktest = true;
+	const char *pos;
+	char new_version[20];
+	size_t i;
+
+	if (argc >= 3) {
+		if (strcasecmp(argv[2], "stable") == 0) {
+			checktest = false;
+		}
+	}
+
+	/* Read file */
+	RSS.Buffer = NULL;
+	RSS.Used = 0;
+	if (!GSM_ReadHTTPFile(RSS_URL, &RSS))
+		return;
+
+	if (checkstable) {
+		pos = strstr(RSS.Buffer, RSS_STABLE_STRING);
+		if (pos != NULL) {
+			pos += strlen(RSS_STABLE_STRING);
+
+			for (i = 0; i < sizeof(new_version) - 1 && (isdigit(pos[i]) || pos[i] == '.'); i++) {
+				new_version[i] = pos[i];
+			}
+
+			new_version[i] = 0;
+
+			if (VersionToInt(new_version) > VersionToInt(VERSION)) {
+				printf_info(_("There is newer stable Gammu version available! (%s instead of %s)\n"),
+				       new_version, VERSION);
+			}
+		}
+	}
+
+	if (checktest) {
+		pos = strstr(RSS.Buffer, RSS_TESTING_STRING);
+		if (pos != NULL) {
+			pos += strlen(RSS_TESTING_STRING);
+
+			for (i = 0; i < sizeof(new_version) - 1 && (isdigit(pos[i]) || pos[i] == '.'); i++) {
+				new_version[i] = pos[i];
+			}
+
+			new_version[i] = 0;
+
+			if (VersionToInt(new_version) > VersionToInt(VERSION)) {
+				printf_info(_("There is newer testing Gammu version available! (%s instead of %s)\n"),
+				       new_version, VERSION);
+			}
+		}
+	}
+
+	free(RSS.Buffer);
+}
+
 static void Version(int argc UNUSED, char *argv[]UNUSED)
 {
 	PrintVersion();
@@ -323,7 +420,10 @@ static void RunBatch(int argc, char *argv[])
 	batch = true;
 	while (!feof(bf)) {
 		ln[0] = 0;
-		fgets(ln, sizeof(ln) - 2, bf);
+		if (fgets(ln, sizeof(ln) - 2, bf) == NULL) {
+			printf_err(_("Error reading batch! Terminating.\n"));
+			break;
+		}
 		if (ln[strlen(ln) - 2] == 0x0D) {
 			/* reduce CRLF to LF so we have the same EOL on Windows and Linux */
 			ln[strlen(ln) - 2] = 0x0A;
@@ -334,7 +434,7 @@ static void RunBatch(int argc, char *argv[])
 			/* line is empty and is not a comment */
 			continue;
 		}
-			
+
 		/* split words into strings in the array argsv */
 		i = 0;
 		j = 0;
@@ -383,6 +483,7 @@ static GSM_Parameters Parameters[] = {
 	{"identify",			0, 0, Identify,			{H_Info,0},			""},
 	{"version",			0, 0, Version,			{H_Gammu,0},			""},
 	{"features",			0, 0, Features,			{H_Gammu,0},			""},
+	{"checkversion",		0, 1, CheckVersion,		{H_Gammu,0},			"[stable]"},
 	{"getdisplaystatus",		0, 0, GetDisplayStatus,		{H_Info,0},			""},
 	{"monitor",			0, 1, Monitor,			{H_Info,H_Network,H_Call,0},	"[times]"},
 	{"setautonetworklogin",	0, 0, SetAutoNetworkLogin,	{H_Network,0},			""},
@@ -402,7 +503,7 @@ static GSM_Parameters Parameters[] = {
 	{"deletefiles",		1,20, DeleteFiles,		{H_Filesystem,0},		"fileID"},
 #if defined(GSM_ENABLE_NOKIA_DCT3) || defined(GSM_ENABLE_NOKIA_DCT4)
 	{"nokiaaddplaylists",		0, 0, NokiaAddPlayLists,	{H_Filesystem,H_Nokia,0},	""},
-	{"nokiaaddfile",		2, 5, NokiaAddFile,		{H_Filesystem,H_Nokia,0},	"Application|Game file [-readonly][-overwrite]"},
+	{"nokiaaddfile",		2, 5, NokiaAddFile,		{H_Filesystem,H_Nokia,0},	"Application|Game file [-readonly][-overwrite][-overwriteall]"},
 	{"nokiaaddfile",		2, 5, NokiaAddFile,		{H_Filesystem,H_Nokia,0},	"Gallery|Gallery2|Camera|Tones|Tones2|Records|Video|Playlist|MemoryCard file [-name name][-protected][-readonly][-system][-hidden][-newtime]"},
 	{"playsavedringtone",		1, 1, DCT4PlaySavedRingtone, 	{H_Ringtone,0},			"number"},
 #endif
@@ -414,7 +515,8 @@ static GSM_Parameters Parameters[] = {
 	{"resetphonesettings",	1, 1, ResetPhoneSettings,	{H_Settings,0},			"PHONE|DEV|UIF|ALL|FACTORY"},
 	{"getmemory",			2, 4, GetMemory,		{H_Memory,0},			ALL_MEMORY_TYPES " start [stop [-nonempty]]"},
 	{"deletememory",		2, 3, DeleteMemory,		{H_Memory,0},			ALL_MEMORY_TYPES " start [stop]"},
-	{"getallmemory",		1, 2, GetAllMemory,		{H_Memory,0},			ALL_MEMORY_TYPES},
+	{"deleteallmemory",		1, 1, DeleteAllMemory,		{H_Memory,0},			ALL_MEMORY_TYPES},
+	{"getallmemory",		1, 1, GetAllMemory,		{H_Memory,0},			ALL_MEMORY_TYPES},
 	{"searchmemory",		1, 1, SearchMemory,		{H_Memory,0},			"text"},
 	{"listmemorycategory",	1, 1, ListMemoryCategory,	{H_Memory, H_Category,0},	"text|number"},
 	{"getfmstation",		1, 2, GetFMStation,		{H_FM,0},			"start [stop]"},
@@ -530,7 +632,7 @@ static GSM_Parameters Parameters[] = {
 	{"reset",			1, 1, Reset,			{H_Other,0},			"SOFT|HARD"},
 	{"getprofile",		1, 2, GetProfile,		{H_Settings,0},			"start [stop]"},
 	{"getsecuritystatus",		0, 0, GetSecurityStatus,	{H_Info,0},			""},
-	{"entersecuritycode",		2, 2, EnterSecurityCode,	{H_Other,0},			"PIN|PUK|PIN2|PUK2 code"},
+	{"entersecuritycode",		2, 2, EnterSecurityCode,	{H_Other,0},			"PIN|PUK|PIN2|PUK2 code|-"},
 	{"deletewapbookmark", 	1, 2, DeleteWAPBookmark, 	{H_WAP,0},			"start [stop]"},
 	{"getwapbookmark",		1, 2, GetWAPBookmark,		{H_WAP,0},			"start [stop]"},
 	{"getwapsettings",		1, 2, GetWAPMMSSettings,	{H_WAP,0},			"start [stop]"},
@@ -563,7 +665,7 @@ static GSM_Parameters Parameters[] = {
 	{"savefile",			4, 5, SaveFile,			{H_Backup,H_ToDo,0},		"TODO target.vcs file location"},
 	{"savefile",			4, 5, SaveFile,			{H_Backup,H_Memory,0},		"VCARD10|VCARD21 target.vcf file SM|ME location"},
 	{"savefile",			4, 5, SaveFile,			{H_Backup,H_WAP,0},		"BOOKMARK target.url file location"},
-	{"backup",			1, 2, Backup,			{H_Backup,H_Memory,H_Calendar,H_ToDo,H_Category,H_Ringtone,H_WAP,H_FM,0},			"file [-yes]"},
+	{"backup",			1, 2, DoBackup,			{H_Backup,H_Memory,H_Calendar,H_ToDo,H_Category,H_Ringtone,H_WAP,H_FM,0},			"file [-yes]"},
 	{"backupsms",			1, 1, BackupSMS,		{H_Backup,H_SMS,0},		"file"},
 	{"restore",			1, 2, Restore,			{H_Backup,H_Memory,H_Calendar,H_ToDo,H_Category,H_Ringtone,H_WAP,H_FM,0},			"file [-yes]"},
 	{"addnew",			1, 2, AddNew,			{H_Backup,H_Memory,H_Calendar,H_ToDo,H_Category,H_Ringtone,H_WAP,H_FM,0},			"file [-yes]"},
@@ -667,6 +769,7 @@ void HelpGeneral(void)
 
 	printf("%s\n\n", _("Usage: gammu [confign] [nothing|text|textall|binary|errors] <command> [options]"));
 	printf("%s\n", _("First parameter optionally specifies which config section to use (all are probed by default)."));
+	printf("%s\n", _("Use only number of config section, so if config section is [gammu42], use 42."));
 	printf("%s\n\n", _("Second parameter optionally controls debug level, next one specifies actions."));
 
 	printf("%s\n\n", _("Commands can be specified with or without leading --."));
@@ -682,11 +785,12 @@ void HelpGeneral(void)
 	printf("\n");
 }
 
-void HelpSplit(int cols, int len, unsigned char *buff)
+void HelpSplit(int cols, int len, const char *buff)
 {
 	int l, len2, pos, split;
 	bool in_opt, first = true;
-	char *remain, spaces[50], buffer[500];
+	const char *remain;
+	char spaces[50], buffer[500];
 
 	if (cols == 0) {
 		printf(" %s\n", buff);
@@ -707,7 +811,7 @@ void HelpSplit(int cols, int len, unsigned char *buff)
 				pos = 0;
 				in_opt = false;
 				if (!first)
-					printf(spaces);
+					printf("%s", spaces);
 				while (pos < cols - len && remain[pos] != 0) {
 					if (in_opt && remain[pos] == ']') {
 						in_opt = false;
@@ -848,27 +952,6 @@ void Help(int argc, char *argv[])
 	}
 }
 
-int FoundVersion(unsigned char *Buffer)
-{
-	size_t retval = 0, pos = 0;
-
-	retval = atoi(Buffer) * 10000;
-	while (Buffer[pos] != '.') {
-		pos++;
-		if (pos == strlen(Buffer))
-			return retval;
-	}
-	pos++;
-	retval += atoi(Buffer + pos) * 100;
-	while (Buffer[pos] != '.') {
-		pos++;
-		if (pos == strlen(Buffer))
-			return retval;
-	}
-	pos++;
-	return retval + atoi(Buffer + pos);
-}
-
 int ProcessParameters(char start, int argc, char *argv[])
 {
 	int z = 0;
@@ -946,27 +1029,28 @@ int ProcessParameters(char start, int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
-	GSM_File RSS;
-	int rsslevel = 0, oldpos = 0;
-	size_t pos = 0;
 	int start = 0;
 	unsigned int i;
 	int only_config = -1;
-	char *cp, *rss, buff[200];
+	char *cp, *rss;
 	GSM_Config *smcfg;
 	GSM_Config *smcfg0;
 	GSM_Debug_Info *di;
 	GSM_Error error;
 	INI_Section *cfg = NULL;
 
-	s = GSM_AllocStateMachine();
+	gsm = GSM_AllocStateMachine();
 
 	GSM_InitLocales(NULL);
 
+#ifdef CURL_FOUND
+	curl_global_init(CURL_GLOBAL_ALL);
+#endif
+
 	di = GSM_GetGlobalDebug();
 
 #ifdef DEBUG
-	GSM_SetDebugFileDescriptor(stdout, di);
+	GSM_SetDebugFileDescriptor(stdout, false, di);
 	GSM_SetDebugLevel("textall", di);
 #endif
 
@@ -1003,27 +1087,34 @@ int main(int argc, char *argv[])
 			printf_warn("%s\n", _("No configuration file found!"));
 		}
 	}
-	if (cfg == NULL)
-		printf_warn("%s\n", _("No configuration read, using builtin defaults!"));
 
-	smcfg0 = GSM_GetConfig(s, 0);
+	smcfg0 = GSM_GetConfig(gsm, 0);
 
-	for (i = 0; (smcfg = GSM_GetConfig(s, i)) != NULL; i++) {
+	for (i = 0; (smcfg = GSM_GetConfig(gsm, i)) != NULL; i++) {
 		/* Wanted user specific configuration? */
 		if (only_config != -1) {
 			smcfg = smcfg0;
+			error = GSM_ReadConfig(cfg, smcfg, only_config);
 			/* Here we get only in first for loop */
-			if (!GSM_ReadConfig(cfg, smcfg, only_config)) {
+			if (error != ERR_NONE) {
 				printf_err(_("Failed to read [gammu%d] from gammurc!\n"),
 					   only_config);
 				printf_warn("%s\n", _("No configuration read, using builtin defaults!"));
 				GSM_ReadConfig(NULL, smcfg, 0);
 			}
 		} else {
-			if (!GSM_ReadConfig(cfg, smcfg, i) && i != 0)
-				break;
+			error = GSM_ReadConfig(cfg, smcfg, i);
+			if (error != ERR_NONE) {
+				if (i != 0) {
+					/* We just end here, we already have some valid config */
+					break;
+				}
+				if (error == ERR_USING_DEFAULTS) {
+					printf_warn("%s\n", _("No configuration read, using builtin defaults!"));
+				}
+			}
 		}
-		GSM_SetConfigNum(s, GSM_GetConfigNum(s) + 1);
+		GSM_SetConfigNum(gsm, GSM_GetConfigNum(gsm) + 1);
 
 		if (cfg != NULL) {
 			cp = INI_GetValue(cfg, "gammu", "gammucoding", false);
@@ -1041,11 +1132,7 @@ int main(int argc, char *argv[])
 		smcfg->UseGlobalDebugFile = true;
 
 		/* It makes no sense to open several debug logs... */
-		if (i != 0) {
-			strcpy(smcfg->DebugLevel, smcfg0->DebugLevel);
-			free(smcfg->DebugFile);
-			smcfg->DebugFile = strdup(smcfg0->DebugFile);
-		} else {
+		if (i == 0) {
 			/* Just for first config */
 			/* When user gave debug level on command line */
 			if (argc > 1 + start
@@ -1053,7 +1140,7 @@ int main(int argc, char *argv[])
 				/* Debug level from command line will be used with phone too */
 				strcpy(smcfg->DebugLevel, argv[1 + start]);
 				/* When configured from command line, output to stdout */
-				GSM_SetDebugFileDescriptor(stdout, di);
+				GSM_SetDebugFileDescriptor(stdout, false, di);
 				start++;
 			} else {
 				/* Try to set debug level from config file */
@@ -1067,11 +1154,7 @@ int main(int argc, char *argv[])
 		if (i == 0) {
 			rss = INI_GetValue(cfg, "gammu", "rsslevel", false);
 			if (rss) {
-				if (strcasecmp(rss, "teststable") == 0) {
-					rsslevel = 2;
-				} else if (strcasecmp(rss, "stable") == 0) {
-					rsslevel = 1;
-				}
+				printf_warn("Configuration option rsslevel is ignored, use '%s' instead\n", "gammu checkversion");
 			}
 			rss = INI_GetValue(cfg, "gammu", "usephonedb", false);
 			if (rss && strcasecmp(rss, "yes") == 0)
@@ -1098,78 +1181,13 @@ int main(int argc, char *argv[])
 		exit(-1);
 	}
 
-	if (rsslevel > 0) {
-		RSS.Buffer = NULL;
-		if (GSM_ReadHTTPFile("blog.cihar.com", "archives/gammu_releases/index-rss.xml", &RSS)) {
-			while (pos < RSS.Used) {
-				if (RSS.Buffer[pos] != 10) {
-					pos++;
-					continue;
-				}
-				RSS.Buffer[pos] = 0;
-				if (strstr(RSS.Buffer + oldpos, "<title>") ==
-				    NULL
-				    || strstr(RSS.Buffer + oldpos,
-					      "</title>") == NULL
-				    || strstr(RSS.Buffer + oldpos,
-					      "win32") != NULL) {
-					pos++;
-					oldpos = pos;
-					continue;
-				}
-				if (rsslevel > 0
-				    && strstr(RSS.Buffer + oldpos,
-					      "stable version") != NULL) {
-					sprintf(buff,
-						strstr(RSS.Buffer + oldpos,
-						       "stable version") + 15);
-					for (i = 0; i < strlen(buff); i++) {
-						if (buff[i] == '<') {
-							buff[i] = 0;
-							break;
-						}
-					}
-					if (FoundVersion(buff) >
-					    FoundVersion(VERSION)) {
-						printf(_("INFO: there is later stable Gammu (%s instead of %s) available!\n"),
-						       buff, VERSION);
-						break;
-					}
-				}
-				if (rsslevel == 2
-				    && strstr(RSS.Buffer + oldpos,
-					      "test version") != NULL) {
-					sprintf(buff,
-						strstr(RSS.Buffer + oldpos,
-						       "test version") + 13);
-					for (i = 0; i < strlen(buff); i++) {
-						if (buff[i] == '<') {
-							buff[i] = 0;
-							break;
-						}
-					}
-					if (FoundVersion(buff) >
-					    FoundVersion(VERSION)) {
-						printf(_("INFO: there is later testing Gammu (%s instead of %s) available!\n"),
-						       buff, VERSION);
-						break;
-					}
-				}
-				pos++;
-				oldpos = pos;
-			}
-			free(RSS.Buffer);
-		}
-	}
-
 	ProcessParameters(start, argc, argv);
 
-	/* Close debug output if opened */
-	GSM_SetDebugFileDescriptor(NULL, di);
+	Cleanup();
 
-	GSM_FreeStateMachine(s);
+	INI_Free(cfg);
 
-	exit(0);
+	return 0;
 }
 
 /* How should editor hadle tabs in this file? Add editor commands here.
