@@ -94,6 +94,10 @@ void PrintMemorySubEntry(GSM_SubMemoryEntry *entry)
 	case PBK_PictureID	     :
 		printf(LISTFORMAT "0x%x\n", _("Picture ID"),entry->Number);
 		return;
+	case PBK_Photo:
+		printf(LISTFORMAT "%s\n", _("Photo"), _("Displaying not supported"));
+		return;
+	case PBK_Number_Messaging   : printf(LISTFORMAT, _("Favorite messaging number")); break;
 	case PBK_Number_General     : printf(LISTFORMAT, _("General number")); break;
 	case PBK_Number_Mobile      : printf(LISTFORMAT, _("Mobile number")); break;
 	case PBK_Number_Work        : printf(LISTFORMAT, _("Work number")); break;
@@ -156,9 +160,10 @@ void GetAllMemory(int argc UNUSED, char *argv[])
 	signal(SIGINT, interrupt);
 	fprintf(stderr, "%s\n", _("Press Ctrl+C to break..."));
 
-	Entry.MemoryType = MemoryTypeFromString(argv[2]);
+	Entry.MemoryType = GSM_StringToMemoryType(argv[2]);
+	Entry.Location = 0;
 
-	if (Entry.MemoryType==0) {
+	if (Entry.MemoryType == 0) {
 		printf_err(_("Unknown memory type (\"%s\")\n"),argv[2]);
 		exit (-1);
 	}
@@ -188,7 +193,7 @@ void GetMemory(int argc, char *argv[])
 	double version;
 	GSM_Error error;
 
-	entry.MemoryType = MemoryTypeFromString(argv[2]);
+	entry.MemoryType = GSM_StringToMemoryType(argv[2]);
 
 	if (entry.MemoryType==0) {
 		printf_err(_("Unknown memory type (\"%s\")\n"),argv[2]);
@@ -258,7 +263,7 @@ void DeleteMemory(int argc, char *argv[])
 	GSM_MemoryEntry		entry;
 	GSM_Error error;
 
-	entry.MemoryType = MemoryTypeFromString(argv[2]);
+	entry.MemoryType = GSM_StringToMemoryType(argv[2]);
 
 	if (entry.MemoryType==0) {
 		printf_err(_("Unknown memory type (\"%s\")\n"),argv[2]);
@@ -289,18 +294,28 @@ void DeleteMemory(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-#define MemoryLocationToString(x) ( \
-	x == MEM_ON ? "ON" :			\
-	x == MEM_RC ? "RC" :			\
-	x == MEM_MC ? "MC" :			\
-	x == MEM_MT ? "MT" :			\
-	x == MEM_ME ? "ME" :			\
-	x == MEM_SM ? "SM" :			\
-	x == MEM_SL ? "SL" :			\
-	x == MEM_VM ? "VM" :			\
-	x == MEM_FD ? "FD" : "XX")
+void DeleteAllMemory(int argc, char *argv[])
+{
+	GSM_MemoryType MemoryType;
+	GSM_Error error;
 
-void SearchOneEntry(GSM_MemoryEntry *Entry, unsigned char *Text)
+	MemoryType = GSM_StringToMemoryType(argv[2]);
+
+	if (MemoryType == 0) {
+		printf_err(_("Unknown memory type (\"%s\")\n"),argv[2]);
+		exit (-1);
+	}
+
+	GSM_Init(true);
+
+	error = GSM_DeleteAllMemory(gsm, MemoryType);
+
+	if (error != ERR_NONE) Print_Error(error);
+
+	GSM_Terminate();
+}
+
+void SearchOneEntry(GSM_MemoryEntry *Entry, const unsigned char *Text)
 {
 	int i;
 
@@ -313,6 +328,7 @@ void SearchOneEntry(GSM_MemoryEntry *Entry, unsigned char *Text)
 			case PBK_Number_Home        :
 			case PBK_Number_Pager       :
 			case PBK_Number_Other       :
+			case PBK_Number_Messaging:
 			case PBK_Text_Note          :
 			case PBK_Text_Postal        :
 			case PBK_Text_WorkPostal:
@@ -347,7 +363,7 @@ void SearchOneEntry(GSM_MemoryEntry *Entry, unsigned char *Text)
 			case PBK_Caller_Group       :
 				if (mywstrstr(Entry->Entries[i].Text, Text) != NULL) {
 					fprintf(stderr,"\n");
-					printf(_("Memory %s, Location %i\n"),MemoryLocationToString(Entry->MemoryType),Entry->Location);
+					printf(_("Memory %s, Location %i\n"),GSM_MemoryTypeToString(Entry->MemoryType),Entry->Location);
 					PrintMemoryEntry(Entry);
 					return;
 				}
@@ -359,6 +375,7 @@ void SearchOneEntry(GSM_MemoryEntry *Entry, unsigned char *Text)
 			case PBK_PictureID:
 			case PBK_CallLength:
 			case PBK_LastModified:
+			case PBK_Photo:
 #ifndef CHECK_CASES
 			default:
 #endif
@@ -367,7 +384,7 @@ void SearchOneEntry(GSM_MemoryEntry *Entry, unsigned char *Text)
 	}
 }
 
-void SearchOneMemory(GSM_MemoryType MemoryType, char *Title, unsigned char *Text)
+void SearchOneMemory(GSM_MemoryType MemoryType, const char *Title, const unsigned char *Text)
 {
 	GSM_MemoryEntry		Entry;
 	GSM_MemoryStatus	Status;
@@ -455,7 +472,7 @@ void ListMemoryCategoryEntries(int Category)
 		Print_Error(error);
 		for (j=0;j<Entry.EntriesNum;j++) {
 			if (Entry.Entries[j].EntryType == PBK_Category && Entry.Entries[j].Number == Category) {
-				printf(_("Memory %s, Location %i\n"),MemoryLocationToString(Entry.MemoryType),Entry.Location);
+				printf(_("Memory %s, Location %i\n"),GSM_MemoryTypeToString(Entry.MemoryType),Entry.Location);
 				PrintMemoryEntry(&Entry);
 			}
 		}
