@@ -341,7 +341,8 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage *sms, bool eachsms, bool ems, cons
 	GSM_SiemensOTASMSInfo 	SiemensOTA;
 	GSM_MultiPartSMSInfo	SMSInfo;
 	bool			RetVal,udhinfo=true;
-	int			j,Pos,i;
+	int			j,i;
+	size_t Pos;
 	GSM_MemoryEntry		pbk;
 	GSM_Error error;
 
@@ -494,7 +495,7 @@ void IncomingSMS(GSM_StateMachine *sm UNUSED, GSM_SMSMessage sms)
  	IncomingSMSData.Number = 1;
 }
 
-void DisplayIncomingSMS()
+void DisplayIncomingSMS(void)
 {
  	GSM_SMSFolders folders;
 	GSM_Error error;
@@ -991,7 +992,8 @@ void GetMMSFolders(int argc UNUSED, char *argv[] UNUSED)
 
 void DecodeMMSFile(GSM_File *file, int num)
 {
-	int				i,Pos;
+	int				i;
+	size_t Pos;
 	char				buff[200];
 	GSM_EncodedMultiPartMMSInfo 	info;
 	GSM_Error			error;
@@ -2404,6 +2406,7 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 	if (DisplaySMS) {
 		if (SMSCSet != 0) {
 			printf("%s\n", _("Use -smscnumber option to give SMSC number"));
+			GSM_Terminate();
 			exit(-1);
 		}
 
@@ -2422,7 +2425,6 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 		printf("\n");
 		printf(_("Number of messages: %i"), sms.Number);
 		printf("\n");
-		exit(sms.Number);
 	}
 	if (SendSMSDSMS) {
 		if (SMSCSet != 0) {
@@ -2442,7 +2444,6 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 		}
 		error = SMSDaemonSendSMS(argv[4],argv[5],&sms);
 		Print_Error(error);
-		exit(0);
 	}
 	if (SaveSMS || SendSaved) {
 		error=GSM_GetSMSFolders(gsm, &folders);
@@ -2511,7 +2512,9 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 				}
 			}
 		}
-	} else {
+	}
+
+	if (SendSMS) {
 		if (Validity.Format != 0 && SMSCSet != 0) {
 			PhoneSMSC.Location = SMSCSet;
 			error=GSM_GetSMSC(gsm,&PhoneSMSC);
