@@ -8,6 +8,9 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <gammu.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include "common.h"
 #include "misc.h"
@@ -225,7 +228,7 @@ void IncomingCall(GSM_StateMachine *sm UNUSED, GSM_Call call)
 #define CHECK_EXIT \
 { \
 	if (gshutdown) break; \
-	if (error != ERR_NONE && error != ERR_UNKNOWN && error != ERR_NOTSUPPORTED && error != ERR_EMPTY && error != ERR_SOURCENOTAVAILABLE && error != ERR_NOTIMPLEMENTED) break; \
+	if (error != ERR_NONE && error != ERR_UNKNOWN && error != ERR_NOTSUPPORTED && error != ERR_MEMORY && error != ERR_EMPTY && error != ERR_SOURCENOTAVAILABLE && error != ERR_NOTIMPLEMENTED) break; \
 }
 
 void Monitor(int argc, char *argv[])
@@ -434,6 +437,9 @@ void Monitor(int argc, char *argv[])
 
 	printf("%s\n", _("Leaving monitor mode..."));
 
+	/* Report if we failed because of an error */
+	Print_Error(error);
+
 	GSM_Terminate();
 }
 
@@ -545,7 +551,7 @@ void MakeTerminatedCall(int argc, char *argv[])
 	error=GSM_DialVoice(gsm, argv[2], ShowNumber);
 	Print_Error(error);
 
-	my_sleep(atoi(argv[3]));
+	sleep(atoi(argv[3]));
 	GSM_ReadDevice(gsm,true);
 
 	if (TerminateID != -1) {
@@ -1461,7 +1467,9 @@ void CopyBitmap(int argc, char *argv[])
 				case GSM_CallerGroupLogo  :	printf(_("Caller group logo")); 	break;
 				default		     : 					break;
 			}
-			printf(_(", width %i, height %i\n"),Bitmap.Bitmap[i].BitmapWidth,Bitmap.Bitmap[i].BitmapHeight);
+			printf(_(", width %i, height %i\n"),
+				(int)Bitmap.Bitmap[i].BitmapWidth,
+				(int)Bitmap.Bitmap[i].BitmapHeight);
 			GSM_PrintBitmap(stdout,&Bitmap.Bitmap[i]);
 		}
 	} else {
@@ -1668,11 +1676,24 @@ void EnterSecurityCode(int argc UNUSED, char *argv[])
 	} else if (strcasecmp(argv[2],"PIN2") == 0) {	Code.Type = SEC_Pin2;
 	} else if (strcasecmp(argv[2],"PUK2") == 0) {	Code.Type = SEC_Puk2;
 	} else {
-		printf(_("What security code (\"%s\") ?\n"),argv[2]);
+		printf_err("%s: %s\n", _("Invalid security code type"), argv[2]);
 		exit(-1);
 	}
 
-	strcpy(Code.Code,argv[3]);
+	if (strcmp(argv[3], "-") == 0) {
+		/* Read code from stdin */
+#ifdef HAVE_UNISTD_H
+		if (isatty(fileno(stdin))) {
+			printf(_("Enter %s code: "), argv[2]);
+		}
+#endif
+		if (fscanf(stdin, "%15s", Code.Code) != 1) {
+			printf_err("%s\n", _("No PIN code entered!"));
+			exit(-1);
+		}
+	} else {
+		strcpy(Code.Code,argv[3]);
+	}
 
 	GSM_Init(true);
 
