@@ -2,6 +2,7 @@
 #include <string.h>
 
 #include "../common/misc/locales.h"
+#include "../common/misc/coding/coding.h"
 
 #include <gammu.h>
 #include <stdlib.h>
@@ -34,6 +35,7 @@ void NokiaComposer(int argc UNUSED, char *argv[])
 
 	ringtone.Format	= 0;
 	error=GSM_ReadRingtoneFile(argv[2],&ringtone);
+	Print_Error(error);
 
 	if (ringtone.Format != RING_NOTETONE) {
 		printf("%s\n", _("It can be RTTL ringtone only used with this option"));
@@ -451,10 +453,10 @@ void NokiaAddPlayLists(int argc UNUSED, char *argv[] UNUSED)
 }
 
 struct NokiaFolderInfo {
-	char	*model;
-	char 	*parameter;
-	char	*folder;
-	char 	*level;
+	const char	*model;
+	const char 	*parameter;
+	const char	*folder;
+	const char 	*level;
 };
 
 struct NokiaFolderInfo Folder[] = {
