@@ -6,6 +6,17 @@
 
 #ifdef HAVE_PTHREAD
 #  include <pthread.h>
+#  include <sys/types.h>
+#  include <sys/stat.h>
+#  include <fcntl.h>
+#  define THREAD_RETURN void *
+#  define THREAD_RETURN_VAL NULL
+#endif
+#ifdef WIN32
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
+#  define THREAD_RETURN DWORD
+#  define THREAD_RETURN_VAL 0
 #endif
 
 #include "search.h"
@@ -37,7 +48,6 @@ typedef struct {
 	OneConnectionInfo Connections[5];
 } OneDeviceInfo;
 
-int num;
 bool SearchOutput;
 
 /**
@@ -90,9 +100,10 @@ void SearchPrintPhoneInfo(GSM_StateMachine * sm)
 	       GSM_GetModelInfo(sm)->model, buffer);
 }
 
-void SearchPhoneThread(OneDeviceInfo * Info)
+THREAD_RETURN SearchPhoneThread(void * arg)
 {
 	int j;
+	OneDeviceInfo *Info = arg;
 	GSM_Error error;
 	GSM_StateMachine *search_gsm;
 	GSM_Config *cfg;
@@ -104,7 +115,7 @@ void SearchPhoneThread(OneDeviceInfo * Info)
 		/* Allocate state machine */
 		search_gsm = GSM_AllocStateMachine();
 		if (search_gsm == NULL)
-			return;
+			return THREAD_RETURN_VAL;
 
 		/* Get configuration pointers */
 		cfg = GSM_GetConfig(search_gsm, 0);
@@ -155,28 +166,30 @@ void SearchPhoneThread(OneDeviceInfo * Info)
 		/* Free allocated buffer */
 		GSM_FreeStateMachine(search_gsm);
 	}
-	num--;
+	return THREAD_RETURN_VAL;
 }
 
 #ifdef HAVE_PTHREAD
-pthread_t Thread[100];
+pthread_t Threads[100];
+#else
+HANDLE Threads[100];
 #endif
 
 OneDeviceInfo SearchDevices[60];
 
 void MakeSearchThread(int i)
 {
-	num++;
 #ifdef HAVE_PTHREAD
 	if (pthread_create
-	    (&Thread[i], NULL, (void *)SearchPhoneThread,
+	    (&Threads[i], NULL, SearchPhoneThread,
 	     &SearchDevices[i]) != 0) {
 		dbgprintf("Error creating thread\n");
 	}
 #else
-	if (CreateThread((LPSECURITY_ATTRIBUTES) NULL, 0,
+	Threads[i] = CreateThread((LPSECURITY_ATTRIBUTES) NULL, 0,
 			 (LPTHREAD_START_ROUTINE) SearchPhoneThread,
-			 &SearchDevices[i], 0, NULL) == NULL) {
+			 &SearchDevices[i], 0, NULL);
+	if (Threads[i] 	== NULL) {
 		dbgprintf("Error creating thread\n");
 	}
 #endif
@@ -184,35 +197,40 @@ void MakeSearchThread(int i)
 
 void SearchPhone(int argc, char *argv[])
 {
-	int i, dev = 0, dev2 = 0;
+	int i, dev = 0;
+#ifdef HAVE_PTHREAD
+	int fd;
+	void *ret;
+#endif
+
 
 	SearchOutput = false;
 	if (argc == 3 && strcasecmp(argv[2], "-debug") == 0)
 		SearchOutput = true;
 
-	num = 0;
 #ifdef WIN32
 	SearchDevices[dev].Device[0] = 0;
 	sprintf(SearchDevices[dev].Connections[0].Connection, "irdaphonet");
 	sprintf(SearchDevices[dev].Connections[1].Connection, "irdaat");
 	SearchDevices[dev].Connections[2].Connection[0] = 0;
 	dev++;
-	dev2 = dev;
 	for (i = 0; i < 20; i++) {
-		sprintf(SearchDevices[dev2].Device, "com%i:", i + 1);
-		sprintf(SearchDevices[dev2].Connections[0].Connection,
+		sprintf(SearchDevices[dev].Device, "com%i:", i + 1);
+		sprintf(SearchDevices[dev].Connections[0].Connection,
 			"fbusdlr3");
-		sprintf(SearchDevices[dev2].Connections[1].Connection, "fbus");
-		sprintf(SearchDevices[dev2].Connections[2].Connection,
+		sprintf(SearchDevices[dev].Connections[1].Connection, "fbus");
+		sprintf(SearchDevices[dev].Connections[2].Connection,
 			"at19200");
-		sprintf(SearchDevices[dev2].Connections[3].Connection, "mbus");
-		SearchDevices[dev2].Connections[4].Connection[0] = 0;
-		dev2++;
+		sprintf(SearchDevices[dev].Connections[3].Connection, "mbus");
+		SearchDevices[dev].Connections[4].Connection[0] = 0;
+		dev++;
 	}
-#endif
-#ifdef __linux__
+#else
 	for (i = 0; i < 6; i++) {
 		sprintf(SearchDevices[dev].Device, "/dev/ircomm%i", i);
+		fd = open(SearchDevices[dev].Device, O_RDWR);
+		if (fd < 0) continue;
+		close(fd);
 		sprintf(SearchDevices[dev].Connections[0].Connection,
 			"irdaphonet");
 		sprintf(SearchDevices[dev].Connections[1].Connection,
@@ -220,49 +238,57 @@ void SearchPhone(int argc, char *argv[])
 		SearchDevices[dev].Connections[2].Connection[0] = 0;
 		dev++;
 	}
-	dev2 = dev;
 	for (i = 0; i < 10; i++) {
-		sprintf(SearchDevices[dev2].Device, "/dev/ttyS%i", i);
-		sprintf(SearchDevices[dev2].Connections[0].Connection,
+		sprintf(SearchDevices[dev].Device, "/dev/ttyS%i", i);
+		fd = open(SearchDevices[dev].Device, O_RDWR);
+		if (fd < 0) continue;
+		close(fd);
+		sprintf(SearchDevices[dev].Connections[0].Connection,
 			"fbusdlr3");
-		sprintf(SearchDevices[dev2].Connections[1].Connection, "fbus");
-		sprintf(SearchDevices[dev2].Connections[2].Connection,
+		sprintf(SearchDevices[dev].Connections[1].Connection, "fbus");
+		sprintf(SearchDevices[dev].Connections[2].Connection,
 			"at19200");
-		sprintf(SearchDevices[dev2].Connections[3].Connection, "mbus");
-		SearchDevices[dev2].Connections[4].Connection[0] = 0;
-		dev2++;
+		sprintf(SearchDevices[dev].Connections[3].Connection, "mbus");
+		SearchDevices[dev].Connections[4].Connection[0] = 0;
+		dev++;
 	}
 	for (i = 0; i < 8; i++) {
-		sprintf(SearchDevices[dev2].Device, "/dev/ttyD00%i", i);
-		sprintf(SearchDevices[dev2].Connections[0].Connection,
+		sprintf(SearchDevices[dev].Device, "/dev/ttyD00%i", i);
+		fd = open(SearchDevices[dev].Device, O_RDWR);
+		if (fd < 0) continue;
+		close(fd);
+		sprintf(SearchDevices[dev].Connections[0].Connection,
 			"fbusdlr3");
-		sprintf(SearchDevices[dev2].Connections[1].Connection, "fbus");
-		sprintf(SearchDevices[dev2].Connections[2].Connection,
+		sprintf(SearchDevices[dev].Connections[1].Connection, "fbus");
+		sprintf(SearchDevices[dev].Connections[2].Connection,
 			"at19200");
-		sprintf(SearchDevices[dev2].Connections[3].Connection, "mbus");
-		SearchDevices[dev2].Connections[4].Connection[0] = 0;
-		dev2++;
+		sprintf(SearchDevices[dev].Connections[3].Connection, "mbus");
+		SearchDevices[dev].Connections[4].Connection[0] = 0;
+		dev++;
 	}
 	for (i = 0; i < 4; i++) {
-		sprintf(SearchDevices[dev2].Device, "/dev/usb/tts/%i", i);
-		sprintf(SearchDevices[dev2].Connections[0].Connection,
+		sprintf(SearchDevices[dev].Device, "/dev/usb/tts/%i", i);
+		fd = open(SearchDevices[dev].Device, O_RDWR);
+		if (fd < 0) continue;
+		close(fd);
+		sprintf(SearchDevices[dev].Connections[0].Connection,
 			"fbusdlr3");
-		sprintf(SearchDevices[dev2].Connections[1].Connection, "fbus");
-		sprintf(SearchDevices[dev2].Connections[2].Connection,
+		sprintf(SearchDevices[dev].Connections[1].Connection, "fbus");
+		sprintf(SearchDevices[dev].Connections[2].Connection,
 			"at19200");
-		sprintf(SearchDevices[dev2].Connections[3].Connection, "mbus");
-		SearchDevices[dev2].Connections[4].Connection[0] = 0;
-		dev2++;
+		sprintf(SearchDevices[dev].Connections[3].Connection, "mbus");
+		SearchDevices[dev].Connections[4].Connection[0] = 0;
+		dev++;
 	}
 #endif
 	for (i = 0; i < dev; i++)
 		MakeSearchThread(i);
-	while (num != 0)
-		my_sleep(5);
-	for (i = dev; i < dev2; i++)
-		MakeSearchThread(i);
-	while (num != 0)
-		my_sleep(5);
+#ifdef WIN32
+	WaitForMultipleObjects(dev, Threads, TRUE, INFINITE);
+#else
+	for (i = 0; i < dev; i++)
+		pthread_join(Threads[i], &ret);
+#endif
 }
 #endif				/*Support for threads */
 
