@@ -74,15 +74,17 @@ static GSM_Error SMSDFiles_SaveInboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfig
 			strcpy(buffer, DecodeUnicodeString(sms->SMS[i].Number));
 			WriteSMSDLog(_("Delivery report: %s to %s"), DecodeUnicodeString(sms->SMS[i].Text), buffer);
 		} else {
-#ifdef GSM_ENABLE_BACKUP
 			if (strcasecmp(Config->inboxformat, "detail") == 0) {
+#ifndef GSM_ENABLE_BACKUP
+				WriteSMSDLog(_("Saving in detail format not compiled in!"));
+
+#else
 				for (j=0;j<sms->Number;j++) backup.SMS[j] = &sms->SMS[j];
 				backup.SMS[sms->Number] = NULL;
 				error = GSM_AddSMSBackupFile(FullName, &backup);
 				done = true;
-			}
 #endif
-			if (!strcasecmp(Config->inboxformat, "detail") == 0) {
+			} else {
 				file = fopen(FullName, "wb");
 				if (file != NULL) {
 					switch (sms->SMS[i].Coding) {
@@ -143,16 +145,17 @@ static GSM_Error SMSDFiles_FindOutboxSMS(GSM_MultiSMSMessage *sms, GSM_SMSDConfi
  	char				*pos1, *pos2, *options;
 #ifdef WIN32
   	struct _finddata_t 		c_file;
-  	long 				hFile;
+  	intptr_t			hFile;
 
   	strcpy(FullName, Config->outboxpath);
   	strcat(FullName, "OUT*.txt*");
-  	if((hFile = _findfirst( FullName, &c_file )) == -1L ) {
+	hFile = _findfirst(FullName, &c_file);
+  	if (hFile == -1 ) {
   		return ERR_EMPTY;
   	} else {
-  		strcpy(FileName,c_file.name);
+  		strcpy(FileName, c_file.name);
   	}
-  	_findclose( hFile );
+  	_findclose(hFile);
 	error = ERR_NONE;
 #else
 #if defined HAVE_DIRENT_H && defined HAVE_SCANDIR && defined HAVE_ALPHASORT
@@ -314,7 +317,7 @@ static GSM_Error SMSDFiles_MoveSMS(GSM_MultiSMSMessage *sms UNUSED,
 	FILE 	*oFile,*iFile;
 	size_t	ilen = 0, olen = 0;
 	char 	Buffer[(GSM_MAX_SMS_LENGTH*GSM_MAX_MULTI_SMS+1)*2],ifilename[400],ofilename[400];
-	char	*sourcepath, *destpath;
+	const char	*sourcepath, *destpath;
 
 	sourcepath = Config->outboxpath;
 	if (sent) {
