@@ -12,7 +12,7 @@
 #ifdef WIN32
 #  include <windows.h>
 #ifndef __GNUC__
-#  pragma comment(lib, "libmysql.lib")
+#  pragma comment(lib, "libpq.lib")
 #endif
 #endif
 
@@ -20,6 +20,8 @@
 #include "smsdcore.h"
 #include "../../common/misc/locales.h"
 
+#define SMSD_PGSQL_DB_VERSION (8)
+
 /* Connects to database */
 static GSM_Error SMSDPgSQL_Init(GSM_SMSDConfig * Config)
 {
@@ -108,7 +110,7 @@ static GSM_Error SMSDPgSQL_Init(GSM_SMSDConfig * Config)
 		PQfinish(Config->DBConnPgSQL);
 		return ERR_UNKNOWN;
 	}
-	if (atoi(PQgetvalue(Res, 0, 0)) > 7) {
+	if (atoi(PQgetvalue(Res, 0, 0)) > SMSD_PGSQL_DB_VERSION) {
 		WriteSMSDLog(_
 			     ("DataBase structures are from higher Gammu version"));
 		WriteSMSDLog(_("Please update this client application"));
@@ -116,7 +118,7 @@ static GSM_Error SMSDPgSQL_Init(GSM_SMSDConfig * Config)
 		PQfinish(Config->DBConnPgSQL);
 		return ERR_UNKNOWN;
 	}
-	if (atoi(PQgetvalue(Res, 0, 0)) < 7) {
+	if (atoi(PQgetvalue(Res, 0, 0)) < SMSD_PGSQL_DB_VERSION) {
 		WriteSMSDLog(_
 			     ("DataBase structures are from older Gammu version"));
 		WriteSMSDLog(_
@@ -145,7 +147,7 @@ static GSM_Error SMSDPgSQL_InitAfterConnect(GSM_SMSDConfig * Config)
 	dbgprintf("%s\n", buf);
 	Res = PQexec(Config->DBConnPgSQL, buf);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
-		WriteSMSDLog(_("Error deleting from database (Init): %s\n"),
+		WriteSMSDLog(_("Error deleting from database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -169,7 +171,7 @@ static GSM_Error SMSDPgSQL_InitAfterConnect(GSM_SMSDConfig * Config)
 	dbgprintf("%s\n", buf);
 	Res = PQexec(Config->DBConnPgSQL, buf);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
-		WriteSMSDLog(_("Error inserting into database (Init): %s\n"),
+		WriteSMSDLog(_("Error inserting into database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -213,9 +215,9 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage *sms,
 
 			Res = PQexec(Config->DBConnPgSQL, buffer);
 			if ((!Res) || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
-				WriteSMSDLog(_
-					     ("Error reading from database (SaveInbox): %s %s\n"),
-					     buffer, PQresultErrorMessage(Res));
+				WriteSMSDLog(_("Error reading from database (%s): %s\n"),
+						__FUNCTION__, PQresultErrorMessage(Res));
+				WriteSMSDLog(_("Failed query: %s\n"), buffer);
 				PQclear(Res);
 				PQfinish(Config->DBConnPgSQL);
 				return ERR_UNKNOWN;
@@ -286,7 +288,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage *sms,
 				if ((!Res)
 				    || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
 					WriteSMSDLog(_
-						     ("Error writing to database (SaveInboxSMS): %s\n"),
+						     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 						     PQresultErrorMessage(Res));
 					PQclear(Res);
 					PQfinish(Config->DBConnPgSQL);
@@ -397,7 +399,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage *sms,
 		Res = PQexec(Config->DBConnPgSQL, buffer);
 		if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
 			WriteSMSDLog(_
-				     ("Error writing to database (SaveInbox): %s\n"),
+				     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 				     PQresultErrorMessage(Res));
 			PQclear(Res);
 			PQfinish(Config->DBConnPgSQL);
@@ -410,7 +412,7 @@ static GSM_Error SMSDPgSQL_SaveInboxSMS(GSM_MultiSMSMessage *sms,
 }
 
 static GSM_Error SMSDPgSQL_RefreshSendStatus(GSM_SMSDConfig * Config,
-					     unsigned char *ID)
+					     char *ID)
 {
 	unsigned char buffer[10000];
 	PGresult *Res;
@@ -423,7 +425,7 @@ static GSM_Error SMSDPgSQL_RefreshSendStatus(GSM_SMSDConfig * Config,
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
 		WriteSMSDLog(_
-			     ("Error writing to database (RefreshSendStatus): %s\n"),
+			     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -444,7 +446,7 @@ static GSM_Error SMSDPgSQL_RefreshSendStatus(GSM_SMSDConfig * Config,
  */
 static GSM_Error SMSDPgSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms,
 					 GSM_SMSDConfig * Config,
-					 unsigned char *ID)
+					 char *ID)
 {
 	unsigned char buf[400];
 	PGresult *Res;
@@ -460,7 +462,7 @@ static GSM_Error SMSDPgSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms,
 	Res = PQexec(Config->DBConnPgSQL, buf);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
 		WriteSMSDLog(_
-			     ("Error reading from database (FindOutbox): %s\n"),
+			     ("Error reading from database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -513,7 +515,7 @@ static GSM_Error SMSDPgSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms,
 		Res = PQexec(Config->DBConnPgSQL, buf);
 		if ((!Res) || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
 			WriteSMSDLog(_
-				     ("Error reading from database (FindOutbox): %s\n"),
+				     ("Error reading from database (%s): %s\n"), __FUNCTION__,
 				     PQresultErrorMessage(Res));
 			PQclear(Res);
 			PQfinish(Config->DBConnPgSQL);
@@ -616,7 +618,7 @@ static GSM_Error SMSDPgSQL_FindOutboxSMS(GSM_MultiSMSMessage * sms,
 
 /* After sending SMS is moved to Sent Items or Error Items. */
 static GSM_Error SMSDPgSQL_MoveSMS(GSM_MultiSMSMessage * sms UNUSED,
-				   GSM_SMSDConfig * Config, unsigned char *ID,
+				   GSM_SMSDConfig * Config, char *ID,
 				   bool alwaysDelete UNUSED, bool sent UNUSED)
 {
 	unsigned char buffer[10000];
@@ -627,7 +629,7 @@ static GSM_Error SMSDPgSQL_MoveSMS(GSM_MultiSMSMessage * sms UNUSED,
 
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
-		WriteSMSDLog(_("Error deleting from database (MoveSMS): %s\n"),
+		WriteSMSDLog(_("Error deleting from database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -640,7 +642,7 @@ static GSM_Error SMSDPgSQL_MoveSMS(GSM_MultiSMSMessage * sms UNUSED,
 
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
-		WriteSMSDLog(_("Error deleting from database (MoveSMS): %s\n"),
+		WriteSMSDLog(_("Error deleting from database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -666,7 +668,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
 		WriteSMSDLog(_
-			     ("Error reading from database (CreateOutbox): %s\n"),
+			     ("Error reading from database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -827,7 +829,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 				if ((!Res)
 				    || (PQresultStatus(Res) != PGRES_TUPLES_OK)) {
 					WriteSMSDLog(_
-						     ("Error reading from database (CreateOutbox): %s\n"),
+						     ("Error reading from database (%s): %s\n"), __FUNCTION__,
 						     PQresultErrorMessage(Res));
 					PQclear(Res);
 					PQfinish(Config->DBConnPgSQL);
@@ -849,7 +851,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 					    || (PQresultStatus(Res) !=
 						PGRES_TUPLES_OK)) {
 						WriteSMSDLog(_
-							     ("Error writing to database (CreateOutbox): %s\n"),
+							     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 							     PQresultErrorMessage
 							     (Res));
 						PQclear(Res);
@@ -878,7 +880,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 						    || (PQresultStatus(Res) !=
 							PGRES_COMMAND_OK)) {
 							WriteSMSDLog(_
-								     ("Error reading from database (CreateOutbox): %s\n"),
+								     ("Error reading from database (%s): %s\n"), __FUNCTION__,
 								     PQresultErrorMessage
 								     (Res));
 							PQclear(Res);
@@ -899,7 +901,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 			Res = PQexec(Config->DBConnPgSQL, buffer4);
 			if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
 				WriteSMSDLog(_
-					     ("Error writing to database (CreateOutbox): %s\n"),
+					     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 					     PQresultErrorMessage(Res));
 				PQclear(Res);
 				PQfinish(Config->DBConnPgSQL);
@@ -913,7 +915,7 @@ static GSM_Error SMSDPgSQL_CreateOutboxSMS(GSM_MultiSMSMessage * sms,
 
 static GSM_Error SMSDPgSQL_AddSentSMSInfo(GSM_MultiSMSMessage * sms,
 					  GSM_SMSDConfig * Config,
-					  unsigned char *ID, int Part,
+					  char *ID, int Part,
 					  GSM_SMSDSendingError err, int TPMR)
 {
 	PGresult *Res;
@@ -1041,7 +1043,7 @@ static GSM_Error SMSDPgSQL_AddSentSMSInfo(GSM_MultiSMSMessage * sms,
 
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
-		WriteSMSDLog(_("Error writing to database (AddSent): %s\n"),
+		WriteSMSDLog(_("Error writing to database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
@@ -1052,21 +1054,21 @@ static GSM_Error SMSDPgSQL_AddSentSMSInfo(GSM_MultiSMSMessage * sms,
 	return ERR_NONE;
 }
 
-static GSM_Error SMSDPgSQL_RefreshPhoneStatus(GSM_SMSDConfig * Config)
+static GSM_Error SMSDPgSQL_RefreshPhoneStatus(GSM_SMSDConfig * Config, GSM_BatteryCharge *Battery, GSM_SignalQuality *Signal)
 {
 	PGresult *Res;
 
 	unsigned char buffer[500];
 
 	sprintf(buffer,
-		"UPDATE phones SET TimeOut= now() + INTERVAL '10 seconds'");
-	sprintf(buffer + strlen(buffer), " WHERE IMEI = '%s'", Config->IMEI);
+		"UPDATE phones SET TimeOut= now() + INTERVAL '10 seconds', Battery = %d, Signal = %d WHERE IMEI = '%s'",
+		Battery->BatteryPercent, Signal->SignalPercent, Config->IMEI);
 	dbgprintf("%s\n", buffer);
 
 	Res = PQexec(Config->DBConnPgSQL, buffer);
 	if ((!Res) || (PQresultStatus(Res) != PGRES_COMMAND_OK)) {
 		WriteSMSDLog(_
-			     ("Error writing to database (SaveInboxSMS): %s\n"),
+			     ("Error writing to database (%s): %s\n"), __FUNCTION__,
 			     PQresultErrorMessage(Res));
 		PQclear(Res);
 		PQfinish(Config->DBConnPgSQL);
