@@ -9,6 +9,14 @@
 #include <sys/wait.h>
 #endif
 
+/* Some systems let waitpid(2) tell callers about stopped children. */
+#if !defined (WCONTINUED)
+#  define WCONTINUED 0
+#endif
+#if !defined (WIFCONTINUED)
+#  define WIFCONTINUED(s)	(0)
+#endif
+
 #include "../../common/misc/coding/coding.h"
 #include "../../common/misc/locales.h"
 #include "../gammu.h"
@@ -27,7 +35,7 @@ static GSM_Error SendingSMSStatus;
 
 void SMSSendingSMSStatus (GSM_StateMachine *sm, int status, int mr)
 {
-	dbgprintf("Incoming SMS device: \"%s\" status=%d, reference=%d\n",
+	WriteSMSDLog("Incoming SMS device: \"%s\" status=%d, reference=%d\n",
 			GSM_GetConfig(sm, -1)->Device,
 			status,
 			mr);
@@ -39,7 +47,7 @@ void SMSSendingSMSStatus (GSM_StateMachine *sm, int status, int mr)
 	}
 }
 
-void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
+void GSM_Terminate_SMSD(const char *msg, int error, bool exitprogram, int rc)
 {
 	int ret = ERR_NONE;
 
@@ -64,16 +72,15 @@ void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
 }
 
 PRINTF_STYLE(1, 2)
-void WriteSMSDLog(char *format, ...)
+void WriteSMSDLog(const char *format, ...)
 {
 	GSM_DateTime 	date_time;
 	char 		Buffer[2000];
 	va_list		argp;
-	int 		result;
 
 	if (smsd_log_file != NULL) {
 		va_start(argp, format);
-		result = vsprintf(Buffer,format,argp);
+		vsprintf(Buffer,format,argp);
 		va_end(argp);
 
 		GSM_GetCurrentDateTime(&date_time);
@@ -310,7 +317,7 @@ bool SMSD_RunOnReceive(GSM_MultiSMSMessage sms UNUSED, GSM_SMSDConfig *Config)
 	ZeroMemory(&pi, sizeof(pi));
 
 	ret = CreateProcess(NULL,     /* No module name (use command line) */
-			Config->RunOnReceive, /* Command line */
+			(char *)Config->RunOnReceive, /* Command line */
 			NULL,           /* Process handle not inheritable*/
 			NULL,           /* Thread handle not inheritable*/
 			FALSE,          /* Set handle inheritance to FALSE*/
@@ -368,7 +375,7 @@ bool SMSD_RunOnReceive(GSM_MultiSMSMessage sms UNUSED, GSM_SMSDConfig *Config)
 			} else if (WIFCONTINUED(status)) {
 				WriteSMSDLog("Process continued\n");
 			}
-			my_sleep(100);
+			usleep(100000);
 			if (i++ > 1200) {
 				WriteSMSDLog("Waited two minutes for child process, giving up\n");
 				return true;
@@ -487,7 +494,16 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	GSM_MultiSMSMessage  	sms;
 	GSM_DateTime         	Date;
 	GSM_Error            	error;
-	unsigned int         	i, j, z;
+	unsigned int         	j, z;
+	int			i;
+	unsigned int		second;
+	GSM_BatteryCharge  charge;
+	GSM_SignalQuality  network;
+
+	/* Clean structure before use */
+	for (i = 0; i < GSM_MAX_MULTI_SMS; i++) {
+		GSM_SetDefaultSMSData(&sms.SMS[i]);
+	}
 
 	error = Service->FindOutboxSMS(&sms, Config, Config->SMSID);
 
@@ -495,12 +511,14 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 		/* No outbox sms - wait few seconds and escape */
 		for (j=0;j<Config->commtimeout && !gshutdown;j++) {
 			GSM_GetCurrentDateTime (&Date);
-			i=Date.Second;
-	 		while (i==Date.Second && !gshutdown) {
-				my_sleep(10);
+			second = Date.Second;
+	 		while (second == Date.Second && !gshutdown) {
+				usleep(10000);
 				GSM_GetCurrentDateTime(&Date);
 			}
-			Service->RefreshPhoneStatus(Config);
+			GSM_GetBatteryCharge(gsm, &charge);
+			GSM_GetSignalQuality(gsm, &network);
+			Service->RefreshPhoneStatus(Config, &charge, &network);
 		}
 		return true;
 	}
@@ -556,13 +574,15 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				if ((strcmp(Config->deliveryreport, "no") != 0 && (Config->currdeliveryreport == -1))) sms.SMS[i].PDU = SMS_Status_Report;
 			}
 
+			GSM_GetBatteryCharge(gsm, &charge);
+			GSM_GetSignalQuality(gsm, &network);
 			error=GSM_SendSMS(gsm, &sms.SMS[i]);
 			if (error!=ERR_NONE) {
 				Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_SENDING_ERROR, -1);
 				WriteSMSDLog(_("Error sending SMS %s (%i): %s"), Config->SMSID, error,GSM_ErrorString(error));
 				return false;
 			}
-			Service->RefreshPhoneStatus(Config);
+			Service->RefreshPhoneStatus(Config, &charge, &network);
 			j    = 0;
 			TPMR = -1;
 			SendingSMSStatus = ERR_TIMEOUT;
@@ -570,13 +590,13 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				GSM_GetCurrentDateTime (&Date);
 				z=Date.Second;
 				while (z==Date.Second) {
-					my_sleep(10);
+					usleep(10000);
 					GSM_GetCurrentDateTime(&Date);
 					GSM_ReadDevice(gsm,true);
 					if (SendingSMSStatus != ERR_TIMEOUT) break;
 				}
 				Service->RefreshSendStatus(Config, Config->SMSID);
-				Service->RefreshPhoneStatus(Config);
+				Service->RefreshPhoneStatus(Config, &charge, &network);
 				if (SendingSMSStatus != ERR_TIMEOUT) break;
 				j++;
 				if (j>Config->sendtimeout) break;
@@ -641,10 +661,11 @@ void SMSDaemon(int argc UNUSED, char *argv[])
 		/* There were errors in communication - try to recover */
 		if (errors > 2 || errors == -1) {
 			if (errors != -1) {
-				WriteSMSDLog(_("Terminating communication (%i,%i)"), error, errors);
+				WriteSMSDLog(_("Terminating communication %s, (%i, %i times)"),
+						GSM_ErrorString(error), error, errors);
 				error=GSM_TerminateConnection(gsm);
 			}
-			if (initerrors++ > 3) my_sleep(30000);
+			if (initerrors++ > 3) sleep(30);
 			WriteSMSDLog(_("Starting communication"));
 			error=GSM_InitConnection(gsm,2);
 			switch (error) {
@@ -670,7 +691,7 @@ void SMSDaemon(int argc UNUSED, char *argv[])
 							GSM_ErrorString(error),
 							error);
 					lastreset = time(NULL);
-					my_sleep(5000);
+					sleep(5);
 				}
 				break;
 			case ERR_DEVICEOPENERROR:
@@ -678,7 +699,7 @@ void SMSDaemon(int argc UNUSED, char *argv[])
 						error, true, -1);
 				break;
 			default:
-				WriteSMSDLog(_("Error at init connection (%s:%i)"),
+				WriteSMSDLog(_("Error at init connection %s (%i)"),
 						GSM_ErrorString(error), error);
 				errors = 250;
 				break;
