@@ -1,7 +1,7 @@
 /**
  * \file gammu-debug.h
  * \author Michal Čihař
- * 
+ *
  * Debuging handling.
  */
 #ifndef __gammu_debug_h
@@ -37,7 +37,7 @@ PRINTF_STYLE(1, 2)
 int dbgprintf(const char *format, ...);
 #else
 #  ifdef __GNUC__
-#    define dbgprintf(a...) do { } while (0)
+#    define dbgprintf(...) do { } while (0)
 #  else
 #    define dbgprintf
 #  endif
@@ -70,7 +70,7 @@ typedef enum {
 	/**
 	 * Message is printed on all text error log levels.
 	 */
-	D_ERROR,
+	D_ERROR
 } GSM_DebugSeverity;
 
 /**
@@ -95,18 +95,21 @@ int smprintf_level(GSM_StateMachine * s, GSM_DebugSeverity severity, const char
  *
  * \ingroup Debug
  */
-GSM_Error GSM_SetDebugFile(char *info, GSM_Debug_Info * privdi);
+GSM_Error GSM_SetDebugFile(const char *info, GSM_Debug_Info * privdi);
 
 /**
  * Sets debug file.
  *
  * \param fd File descriptor.
  * \param privdi Pointert to debug information data.
+ * \param closable Whether Gammu can close the file when it is no longer
+ * needed for debug output. Please note that stderr or stdout are never
+ * closed.
  * \return Error code.
  *
  * \ingroup Debug
  */
-GSM_Error GSM_SetDebugFileDescriptor(FILE * fd, GSM_Debug_Info * privdi);
+GSM_Error GSM_SetDebugFileDescriptor(FILE * fd, bool closable, GSM_Debug_Info * privdi);
 
 /**
  * Returns global debug settings.
@@ -150,7 +153,7 @@ bool GSM_SetDebugLevel(const char *info, GSM_Debug_Info * privdi);
 bool GSM_SetDebugCoding(const char *info, GSM_Debug_Info * privdi);
 
 /**
- * Enables using of global debugging configuration. Makes no effect on 
+ * Enables using of global debugging configuration. Makes no effect on
  * global debug configuration.
  *
  * \param info Enable global debug usage..
@@ -161,6 +164,17 @@ bool GSM_SetDebugCoding(const char *info, GSM_Debug_Info * privdi);
  */
 bool GSM_SetDebugGlobal(bool info, GSM_Debug_Info * privdi);
 
+/**
+ * Logs error to debug log with additional message.
+ *
+ * \param s State machine structure pointer.
+ * \param message String to be show in message.
+ * \param err Error code.
+ *
+ * \ingroup Debug
+ */
+void GSM_LogError(GSM_StateMachine * s, const char * message, const GSM_Error err);
+
 #endif
 
 /* Editor configuration
