@@ -273,11 +273,19 @@ typedef enum {
 	 * Bad feature string.
 	 */
 	ERR_BADFEATURE,
+	/**
+	 * Some functions not compiled in your OS
+	 */
+	ERR_DISABLED,
+	/**
+	 * Bluetooth configuration requires channel option.
+	 */
+	ERR_SPECIFYCHANNEL,
 
 	/**
 	 * Just marker of highest error code, should not be used.
 	 */
-	ERR_LAST_VALUE,
+	ERR_LAST_VALUE
 } GSM_Error;
 
 /**
