@@ -21,14 +21,14 @@
  *
  * \ingroup Info
  */
-char *GSM_GetNetworkName(char *NetworkCode);
+const unsigned char *GSM_GetNetworkName(const char *NetworkCode);
 
 /**
  * Find country name from given country code.
  *
  * \ingroup Info
  */
-char *GSM_GetCountryName(char *CountryCode);
+const unsigned char *GSM_GetCountryName(const char *CountryCode);
 
 /**
  * Status of network logging
@@ -140,7 +140,7 @@ typedef enum {
 	/**
 	 * Power failure
 	 */
-	GSM_PowerFault,
+	GSM_PowerFault
 } GSM_ChargeState;
 
 /**
@@ -164,7 +164,7 @@ typedef enum {
 	/**
 	 * Lithium Polymer battery
 	 */
-	GSM_BatteryLiPol,
+	GSM_BatteryLiPol
 } GSM_BatteryType;
 
 /**
@@ -528,9 +528,9 @@ typedef enum {
 	 */
 	F_SUBMIT_SIM_ONLY,
 	/**
-	 * Prefer UCS-2 for phone book manipulations.
+	 * Prefer Unicode for phone book manipulations.
 	 */
-	F_PBK_UCS2,
+	F_PBK_UNICODE,
 	/**
 	 * Switching to OBEX mode using AT^SQWE=3.
 	 */
@@ -561,11 +561,23 @@ typedef enum {
 	 * it properly.
 	 */
 	F_CPROT,
+	/**
+	 * Phonebook with favorite messaging numbers
+	 */
+	F_PBKFAVORITEMESSAGE,
+	/**
+	 * No support for postal entry in phonebook.
+	 */
+	F_PBKNOPOSTAL,
+	/**
+	 * Encode number in HEX charset.
+	 */
+	F_PBK_ENCODENUMBER,
 
 	/**
 	 * Just marker of highest feature code, should not be used.
 	 */
-	F_LAST_VALUE,
+	F_LAST_VALUE
 } Feature;
 
 /**
@@ -589,7 +601,7 @@ const char *GSM_FeatureToString(Feature feature);
  *
  * \return Feature value, 0 on failure.
  */
-Feature GSM_FeatureFromString(const char * feature);
+Feature GSM_FeatureFromString(const char *feature);
 
 /**
  * Model identification, used for finding phone features.
@@ -602,20 +614,20 @@ typedef struct {
 	/**
 	 * Model as returned by phone
 	 */
-	char *model;
+	const char *model;
 	/**
 	 * Identification by Gammu
 	 */
-	char *number;
+	const char *number;
 	/**
 	 * Model as used over IrDA
 	 */
-	char *irdamodel;
+	const char *irdamodel;
 	/**
 	 * List of supported features
 	 */
 	Feature features[GSM_MAX_PHONE_FEATURES + 1];
-} OnePhoneModel;
+} GSM_PhoneModel;
 
 /**
  * Checks whether phone supports features.
@@ -627,7 +639,7 @@ typedef struct {
  *
  * \ingroup Info
  */
-bool GSM_IsPhoneFeatureAvailable(OnePhoneModel * model, Feature feature);
+bool GSM_IsPhoneFeatureAvailable(GSM_PhoneModel * model, Feature feature);
 
 /**
  * Adds feature to phone configuration.
@@ -639,7 +651,7 @@ bool GSM_IsPhoneFeatureAvailable(OnePhoneModel * model, Feature feature);
  *
  * \ingroup Info
  */
-bool GSM_AddPhoneFeature(OnePhoneModel *model, Feature feature);
+bool GSM_AddPhoneFeature(GSM_PhoneModel * model, Feature feature);
 
 #include <gammu-statemachine.h>
 
@@ -662,7 +674,7 @@ GSM_Error GSM_GetModel(GSM_StateMachine * s, char *value);
  *
  * \ingroup Info
  */
-OnePhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
+GSM_PhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
 
 /**
  * Reads firmware information from phone.
@@ -671,6 +683,7 @@ OnePhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
  */
 GSM_Error GSM_GetFirmware(GSM_StateMachine * s, char *value, char *date,
 			  double *num);
+
 /**
  * Reads IMEI/serial number from phone.
  *
