@@ -14,6 +14,7 @@
 
 #include <gammu-datetime.h>
 #include <gammu-limits.h>
+#include <gammu-bitmap.h>
 
 /**
  * Enum defines ID for various phone and SIM memories.
@@ -67,6 +68,36 @@ typedef enum {
 	MEM_SL
 } GSM_MemoryType;
 
+/**
+ * Returns string representation of a memory type.
+ *
+ * \param x Memory type to convert
+ *
+ * \return Static string.
+ *
+ * \ingroup Memory
+ */
+#define GSM_MemoryTypeToString(x) ( \
+	(x) == MEM_ME ? "ME" :			\
+	(x) == MEM_SM ? "SM" :			\
+	(x) == MEM_ON ? "ON" :			\
+	(x) == MEM_DC ? "DC" :			\
+	(x) == MEM_RC ? "RC" :			\
+	(x) == MEM_MC ? "MC" :			\
+	(x) == MEM_MT ? "MT" :			\
+	(x) == MEM_FD ? "FD" :			\
+	(x) == MEM_VM ? "VM" :			\
+	(x) == MEM_SL ? "SL" : "XX")
+
+/**
+ * Converts memory type from string.
+ *
+ * \param type String with memory type.
+ *
+ * \return Parsed memory type or 0 on failure.
+ */
+GSM_MemoryType GSM_StringToMemoryType(const char *s);
+
 /**
  * Structure contains info about number of used/free entries in phonebook
  * memory.
@@ -114,7 +145,7 @@ typedef enum {
 	/**
 	 * Home number. (Text)
 	 */
-	PBK_Number_Home,
+	PBK_Number_Home, /* 5 */
 	/**
 	 * Pager number. (Text)
 	 */
@@ -134,8 +165,8 @@ typedef enum {
 	/**
 	 * Email. (Text)
 	 */
-	PBK_Text_Email,
-	/**
+	PBK_Text_Email, /* 10 */
+	/*
 	 * Second email. (Text)
 	 */
 	PBK_Text_Email2,
@@ -154,7 +185,7 @@ typedef enum {
 	/**
 	 * Name (Text)
 	 */
-	PBK_Text_Name,
+	PBK_Text_Name, /* 15 */
 	/**
 	 * Last name. (Text)
 	 */
@@ -174,7 +205,7 @@ typedef enum {
 	/**
 	 * Category. (Number, if -1 then text)
 	 */
-	PBK_Category,
+	PBK_Category, /* 20 */
 	/**
 	 * Whether entry is private. (Number)
 	 */
@@ -194,7 +225,7 @@ typedef enum {
 	/**
 	 * Zip code. (Text)
 	 */
-	PBK_Text_Zip,
+	PBK_Text_Zip, /* 25 */
 	/**
 	 * Country. (Text)
 	 */
@@ -214,7 +245,7 @@ typedef enum {
 	/**
 	 * Custom information 4. (Text)
 	 */
-	PBK_Text_Custom4,
+	PBK_Text_Custom4, /* 30 */
 	/**
 	 * Ringtone ID. (Number)
 	 */
@@ -234,7 +265,7 @@ typedef enum {
 	/**
 	 * LUID - Unique Identifier used for synchronisation
 	 */
-	PBK_Text_LUID,
+	PBK_Text_LUID, /* 35 */
 	/**
 	 * Date of last modification
 	 */
@@ -254,7 +285,7 @@ typedef enum {
 	/**
 	 * Work city. (Text)
 	 */
-	PBK_Text_WorkCity,
+	PBK_Text_WorkCity, /* 40 */
 	/**
 	 * Work state. (Text)
 	 */
@@ -274,11 +305,19 @@ typedef enum {
 	/**
 	 * Picture name (on phone filesystem). (Text)
 	 */
-	PBK_Text_PictureName,
+	PBK_Text_PictureName, /* 45 */
  	/**
  	 * Push-to-talk ID (Text)
  	 */
  	PBK_PushToTalkID,
+	/**
+	 * Favorite messaging number. (Text)
+	 */
+	PBK_Number_Messaging,
+	/**
+	 * Photo (Picture).
+	 */
+	PBK_Photo,
 } GSM_EntryType;
 
 /**
@@ -314,6 +353,10 @@ typedef struct {
 	 * Text of entry (if applicable, see @ref GSM_EntryType).
 	 */
 	unsigned char Text[(GSM_PHONEBOOK_TEXT_LENGTH + 1) * 2];
+	/**
+	 * Picture data.
+	 */
+	GSM_BinaryPicture Picture;
 } GSM_SubMemoryEntry;
 
 /**
@@ -492,7 +535,7 @@ GSM_Error GSM_SetSpeedDial(GSM_StateMachine * s, GSM_SpeedDial * Speed);
  *
  * \ingroup Memory
  */
-unsigned char *GSM_PhonebookGetEntryName(GSM_MemoryEntry * entry);
+unsigned char *GSM_PhonebookGetEntryName(const GSM_MemoryEntry * entry);
 
 /**
  * Finds default name, number and group for entry.
@@ -504,7 +547,7 @@ unsigned char *GSM_PhonebookGetEntryName(GSM_MemoryEntry * entry);
  *
  * \ingroup Memory
  */
-void GSM_PhonebookFindDefaultNameNumberGroup(GSM_MemoryEntry * entry, int *Name,
+void GSM_PhonebookFindDefaultNameNumberGroup(const GSM_MemoryEntry * entry, int *Name,
 					     int *Number, int *Group);
 /**
  * Types of vCard.
@@ -529,24 +572,40 @@ typedef enum {
 	 * vCard 2.1 hacked for Sony-Ericsson (should be standard
 	 * vCard).
 	 */
-	SonyEricsson_VCard21
+	SonyEricsson_VCard21,
+	/**
+	 * vCard 2.1 hacked for Sony-Ericsson (should be standard
+	 * vCard) from phone (no parsing of location and memory type).
+	 */
+	SonyEricsson_VCard21_Phone,
 } GSM_VCardVersion;
 
 /**
  * Encodes memory entry to vCard.
  *
+ * \param[out] Buffer Buffer to store vCard text.
+ * \param[in] buff_len Size of output buffer.
+ * \param[in,out] Pos Position in output buffer.
+ * \param[in,out] pbk Phonebook data, AddError will be set on non
+ * converted entries.
+ * \param[in] header Whether to include vCard header in output.
+ * \param[in] Version What vCard version to create.
+ *
+ * \return Error code.
+ *
  * \ingroup Memory
  */
-void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry * pbk,
-		     bool header, GSM_VCardVersion Version);
+GSM_Error GSM_EncodeVCARD(char *Buffer, const size_t buff_len, size_t *Pos,
+			GSM_MemoryEntry * pbk, const bool header,
+			const GSM_VCardVersion Version);
 
 /**
  * Decodes memory entry from vCard.
  *
  * \ingroup Memory
  */
-GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos,
-			  GSM_MemoryEntry * Pbk, GSM_VCardVersion Version);
+GSM_Error GSM_DecodeVCARD(char *Buffer, size_t *Pos,
+			  GSM_MemoryEntry * Pbk, const GSM_VCardVersion Version);
 
 #endif
 
