@@ -123,7 +123,7 @@ typedef enum {
 	GCT_BLUEFBUS2,
 	GCT_BLUEAT,
 	GCT_BLUEPHONET,
-	GCT_BLUEOBEX,
+	GCT_BLUEOBEX
 } GSM_ConnectionType;
 
 /**
