@@ -11,13 +11,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 
 int main(int argc, char **argv)
 {
-	int pos = 0;
+	size_t pos = 0;
 	GSM_MemoryEntry pbk;
 	GSM_Error error;
 	char buffer[65536];
+	char vcard_buffer[65536];
 	FILE *f;
 	size_t len;
 	bool generate = false;
@@ -27,7 +29,7 @@ int main(int argc, char **argv)
 
 	/* Configure debugging */
 	debug_info = GSM_GetGlobalDebug();
-	GSM_SetDebugFileDescriptor(stderr, debug_info);
+	GSM_SetDebugFileDescriptor(stderr, false, debug_info);
 	GSM_SetDebugLevel("textall", debug_info);
 
 	/* Check parameters */
@@ -68,10 +70,22 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	/* Encode vCard back */
+	pos = 0;
+	error = GSM_EncodeVCARD(vcard_buffer, sizeof(vcard_buffer), &pos, &pbk, true, SonyEricsson_VCard21);
+	assert(error == ERR_NONE);
+
+	/*
+	 * Best would be to compare here, but we never can get
+	 * absolutely same as original.
+	 */
+	printf("ORIGINAL:\n%s\n----\nENCODED:\n%s\n", buffer, vcard_buffer);
+
 	/* Generate file if we should */
 	if (generate) {
 		GSM_ClearBackup(&backup);
 		strcpy(backup.Creator, "vCard tester");
+		pbk.Location = 0;
 		backup.PhonePhonebook[0] = &pbk;
 		backup.PhonePhonebook[1] = NULL;
 		if (GSM_SaveBackupFile(argv[2], &backup, true) != ERR_NONE) {
@@ -95,6 +109,13 @@ int main(int argc, char **argv)
 
 	/* Compare content */
 	for (i = 0; i < pbk.EntriesNum; i++) {
+		if (pbk.Entries[i].EntryType != backup.PhonePhonebook[0]->Entries[i].EntryType) {
+			printf("Field %d is not the same type (%d/%d)!\n", i,
+				pbk.Entries[i].EntryType,
+				backup.PhonePhonebook[0]->Entries[i].EntryType);
+			return 1;
+		}
+		printf("Entry type: %d\n", pbk.Entries[i].EntryType);
 		switch (pbk.Entries[i].EntryType) {
 			case PBK_Number_General     :
 			case PBK_Number_Mobile      :
@@ -103,6 +124,7 @@ int main(int argc, char **argv)
 			case PBK_Number_Home        :
 			case PBK_Number_Pager       :
 			case PBK_Number_Other       :
+			case PBK_Number_Messaging:
 			case PBK_Text_Note          :
 			case PBK_Text_Postal        :
 			case PBK_Text_WorkPostal:
@@ -134,7 +156,6 @@ int main(int argc, char **argv)
 			case PBK_Text_UserID:
 			case PBK_Text_PictureName:
 			case PBK_PushToTalkID:
-			case PBK_Caller_Group       :
 				if (!mywstrncmp(
 					pbk.Entries[i].Text,
 					backup.PhonePhonebook[0]->Entries[i].Text,
@@ -143,13 +164,32 @@ int main(int argc, char **argv)
 					return 1;
 				}
 				break;
+			case PBK_Photo       :
+				if ((pbk.Entries[i].Picture.Length !=
+					backup.PhonePhonebook[0]->Entries[i].Picture.Length) ||
+					memcmp(
+					pbk.Entries[i].Picture.Buffer,
+					backup.PhonePhonebook[0]->Entries[i].Picture.Buffer,
+					pbk.Entries[i].Picture.Length) != 0) {
+					printf("Field %d is not the same!\n", i);
+					return 1;
+				}
+				free(pbk.Entries[i].Picture.Buffer);
+				free(backup.PhonePhonebook[0]->Entries[i].Picture.Buffer);
+				break;
 			case PBK_Date:
+			case PBK_LastModified:
+				break;
 			case PBK_Category:
 			case PBK_Private:
 			case PBK_RingtoneID:
 			case PBK_PictureID:
 			case PBK_CallLength:
-			case PBK_LastModified:
+			case PBK_Caller_Group       :
+				if (pbk.Entries[i].Number != backup.PhonePhonebook[0]->Entries[i].Number) {
+					printf("Field %d is not the same!\n", i);
+					return 1;
+				}
 				break;
 		}
 	}
